/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifdef _WINDOWS
#pragma warning(disable : 4244)
#endif

#include "PluginClient.h"
#include "PluginUtils.h"
#include "EI_Dialog.h"
#include "EI_Control.h"
#include "EI_ResourceFile.h"
#include "ColorMesh.hpp"
#include "EI_GraphData.h"
#include "EI_GradientData.h"
#include "EI_GraphControl.h"
#include "EI_GradientControl.h"

// global variables

SInt32 theError;
UInt32 theCreator;
PluginInfoRec * theInfo;
EIPluginRoutine theHostProc;


namespace {

#define pluginType			'TEST'
#define pluginName			"Color Mesh"
#define pluginVersion		0x00000001			
#define pluginCopyright		PS("Copyright  2010 EIAS3D")
#define pluginAuthor		PS("Igors")
#define pluginDescription	PS("API Tester")
#define pluginPermission	0x0F

#define resourceName 		"ColorMesh.rsc"

#define DLG_INTERFACE		21000

enum {
	ID_OK = 3,
	ID_CANCEL = 4,
	ID_INTERP = 5,
	ID_GRAPH = 6,
	ID_GRADIENT = 8,
	ID_STEPS = 10,
	ID_SCALEX = 11,
	ID_SCALEY = 12
};

bool HostCall( UInt32 cmd, SInt32 size, void * data )
{
	static SInt32 debugTrap = 0;
	theError = theHostProc(cmd, size, data);
	if (theError)
		++debugTrap;
	return theError == 0;
}

void * MAlloc( UInt32 size )
{
	HostMemMallocRec hmr;
	hmr.theReference = theInfo->theHostReference;
	hmr.theSize = size;
	hmr.thePtr = 0;			
	if (!HostCall(hostMemMalloc, sizeof(hmr), &hmr)) return 0;
	return hmr.thePtr;
}

void MFree( void * thePtr )
{
	HostMemFreeRec hfr;
	if (thePtr) {
		hfr.theReference = theInfo->theHostReference;
		hfr.thePtr = thePtr;
		HostCall(hostMemFree, sizeof(hfr), &hfr);
	} 
} 

bool AllocControlData( UInt32 newSize )
{
	HostAllocateRec har;
	har.theReference = theInfo->theHostReference;
	har.theControlData = theInfo->theControlData;	
	har.theControlDataSize = newSize;
	har.thePluginData = 0;
	har.thePluginDataSize = 0;
	if (!HostCall('ALOC', sizeof(har), &har)) return false;
	
	theInfo->thePluginData = har.thePluginData;
	theInfo->theControlData = har.theControlData;
	theInfo->thePluginDataSize = har.thePluginDataSize;
	theInfo->theControlDataSize = har.theControlDataSize;
	return true;
}	 

void Initialize( void )
{
	HostCreatorRec hcr;
	hcr.theReference = theInfo->theHostReference;
	theCreator = 'UNKN';
	if (HostCall(hostCreator, sizeof(hcr), &hcr))
		theCreator = hcr.theCreator;
}

void Information( void )
{
	PluginInformationPtr theParam  = PluginInformationPtr(theInfo);
	theParam->thePluginType        = pluginType;
	theParam->thePluginVersion     = pluginVersion;
	PStrCpy(theParam->thePluginName, PS(pluginName));
	PStrCpy(theParam->thePluginCopyright, pluginCopyright);
	PStrCpy(theParam->thePluginAuthor, pluginAuthor);
	PStrCpy(theParam->thePluginDescription, pluginDescription);
	theParam->thePluginPermission  = pluginPermission;
}

static void ModelReset( void )
{
	HostModelResetRec hmr;
	hmr.theReference = theInfo->theHostReference;
	HostCall('RSET', sizeof(hmr), &hmr);
}

void ModelSetup( void )
{
	CColorMesh * mesh = CColorMesh::ReadControlData();
	if (!mesh) return;
	
	PluginModelSetupRec * pms = (PluginModelSetupRec *) theInfo;
	mesh->mGroupMatrix= pms->theGroupMatrix;
	mesh->mGroupRotMatrix = pms->theGroupRotMatrix;
	mesh->mGroupBlurMatrix = pms->theGroupBlurMatrix;
	mesh->SaveSelf();
	delete mesh;
	
	ModelReset();
}	

inline EI_Control * ID2Ctl( EI_Dialog * dlg, short ID )
{
	return EI_FindControlByID(dlg, ID);
}

void ControlHit( EI_Control * ctl, EI_DrawContext * )
{
	short ID = EI_GetControlID(ctl);
	if (ID != ID_INTERP) return;
	EI_Control * graph = ID2Ctl(EI_GetControlDialog(ctl), ID_GRAPH);
	
	EI_GraphData gd;
	SInt32 err = EI_GraphControlRequest(graph, EI_GRAPH_CONTROL_REQUEST_GET_GRAPHDATA, 0, &gd);
	if (!err) {
		SInt32 val = EI_GetControlValue(ctl);
		err = EI_GraphDataRequest(gd, EI_GRAPH_DATA_REQUEST_SET_INTERP, 0, &val);
		EI_InvalControl(graph);
	} 
}

void Interface( void )
{
	CColorMesh * mesh = CColorMesh::ReadControlData();
	if (!mesh) return;
	
	EI_ResourceFile * theRF = OpenResourceFile(resourceName);
	if (!theRF) theError = pluginResourceErr;
	else {
		EI_Dialog * dlg = EI_MakeDialog(theRF, DLG_INTERFACE);
		if (!dlg) theError = pluginResourceErr;
		else {	 
			mesh->SetData(dlg);
			EI_SetControlHitFunction(ID2Ctl(dlg, ID_INTERP), ControlHit);
			if (!EI_ExecuteDialog(dlg)) theError = pluginCancelErr;
			else {
				mesh->GetData(dlg);
				ModelReset();
			} 
			EI_DestroyDialog(dlg);
		} 
		EI_CloseResourceFile(theRF);
	}
	
	delete mesh;
} 

void ModelGenerate( void )
{
// add a new group to guarantee UVs and vertex color
	HostModelGetGroupRec tgrp;
	HostModelAddGroupRec taddg;
	tgrp.theReference = taddg.theReference = theInfo->theHostReference;
	tgrp.theGroupIndex = 0;
	if (!HostCall(hostModelGetGroup, sizeof(tgrp), &tgrp)) return;

	taddg.theGroupTemplate = tgrp.theGroup;
	taddg.theGroupVertexFlags = tgrp.theGroupVertexFlags;
	BSET(taddg.theGroupVertexFlags, pluginVertexColorFlag);
	BSET(taddg.theGroupVertexFlags, pluginVertexTextureFlag);
	if (!HostCall(hostModelAddGroup, sizeof(taddg), &taddg)) return;
	
	CColorMesh * mesh = CColorMesh::ReadControlData();
	if (!mesh) return;
	
	mesh->Generate();
	delete mesh;
}

bool AddQuad( SInt32 v0, SInt32 v1, SInt32 v2, SInt32 v3, const ARGB_Real color[2] )
{
	HostModelAddFacetRec taddf;
	taddf.theReference = theInfo->theHostReference;

	float a = (color[0].a + color[1].a) * 0.5f;
	float r = (color[0].r + color[1].r) * 0.5f;
	float g = (color[0].g + color[1].g) * 0.5f;
	float b = (color[0].b + color[1].b) * 0.5f;
	taddf.theColor.argb.a = uchar(MIN(a, 1) * 255 + 0.5f);
	taddf.theColor.argb.r = uchar(MIN(r, 1) * 255 + 0.5f);
	taddf.theColor.argb.g = uchar(MIN(g, 1) * 255 + 0.5f);
	taddf.theColor.argb.b = uchar(MIN(b, 1) * 255 + 0.5f);

	taddf.theNumVertices = 4;
	taddf.theVertices[0] = v0;
	taddf.theVertices[1] = v1;
	taddf.theVertices[2] = v2;
	taddf.theVertices[3] = v3;

	return HostCall('ADDF', sizeof(taddf), &taddf);
}

bool WriteMatrix( const Matrix4 & M, CStream & dst )
{
	const double * E = &M.m.m00;
	for (SInt32 i = 0; i < 16; ++i)
		dst.WriteDouble(E[i]);
	return dst.mError == 0; 
}

bool ReadMatrix( Matrix4 & M, CStream & src )
{
	double * E = &M.m.m00;
	for (SInt32 i = 0; i < 16; ++i)
		E[i] = src.ReadDouble();
	return src.mError == 0; 
}

} // namespace

//---------------- Main ---------------------------

EXPDLL SInt32 Main(UInt32 theCommand, SInt32 theCommandRecSize, void* theCommandRec)
{
	theInfo = (PluginInfoRec *) theCommandRec;
	theHostProc = theInfo->theHostProc;
	theError = 0;
	
	switch (theCommand) { 
		case pluginInitialize: 	
			Initialize();
			break;
	  
		case pluginInformation: 	
			Information();	 
			break;
	  
		case pluginInterface:   	
			Interface();	 
			break;

		case pluginModelSetup:   	
			ModelSetup();	 
			break;

		case pluginModelGenerate:   	
			ModelGenerate();	 
			break;
	}
	
	return theError;
}  

//---------------- CCustomSaver ---------------------------

void CCustomSaver::Init( void )
{
	mData = 0;
	mSize = 0;
}

void CCustomSaver::Destroy( void )
{
	if (mData) MFree(mData);
	Init();
}

bool CCustomSaver::HasData( void ) const
{
	return mData != 0 && mSize > 0;
}
	
bool CCustomSaver::Read( CStream & src )
{
	SInt32 size = src.ReadULong();
	if (src.mError || !Alloc(size)) return false;

	src.ReadBlk(mData, mSize);
	if (src.mError == 0) return true;
	Destroy();
	return false;
}

bool CCustomSaver::Alloc( UInt32 size )
{
	Destroy();
	if (!size) return true;
	mSize = size;
	mData = MAlloc(mSize);
	return mData != 0;
}

bool CCustomSaver::Write( CStream & dst ) const
{
	dst.WriteULong(mSize);
	if (mSize) dst.WriteBlk(mData, mSize);
	return dst.mError == 0;
}

//---------------- CColorMesh ---------------------------

void CColorMesh::Init( void )
{
	mSign = SIGN_CMESH;
	mVers = VERS_CMESH;
	mStepX = 100;
	mScaleX = 200;
	mScaleY = 20;
	
	InitMatrix4(&mGroupMatrix);
	InitMatrix4(&mGroupRotMatrix);
	InitMatrix4(&mGroupBlurMatrix);

	mSavedGraph.Init();	
	mSavedGradient.Init();	
}

bool CColorMesh::Read( CStream & src )
{
	mSign = src.ReadULong();
	if (mSign != SIGN_CMESH) return false;

	mVers = src.ReadULong();
	if (mVers != VERS_CMESH) return false;

	mStepX = src.ReadLong();
	mInterp = src.ReadLong();
	mScaleX = src.ReadFloat();
	mScaleY = src.ReadFloat();

	ReadMatrix(mGroupMatrix, src);
	ReadMatrix(mGroupRotMatrix, src);
	ReadMatrix(mGroupBlurMatrix, src);
	
	return mSavedGraph.Read(src) && mSavedGradient.Read(src);
}

bool CColorMesh::Write( CStream & dst ) const
{
	dst.WriteULong(mSign);
	dst.WriteULong(mVers);
	dst.WriteLong(mStepX);
	dst.WriteLong(mInterp);
	dst.WriteFloat(mScaleX);
	dst.WriteFloat(mScaleY);
	WriteMatrix(mGroupMatrix, dst);
	WriteMatrix(mGroupRotMatrix, dst);
	WriteMatrix(mGroupBlurMatrix, dst);
	return mSavedGraph.Write(dst) && mSavedGradient.Write(dst);
}

void CColorMesh::SetData( EI_Dialog * dlg )
{
	SInt32 err;
	EI_Control * ctl = 0;	
	SetTheINum(dlg, ID_STEPS, mStepX);
	SetTheIPopUp(dlg, ID_INTERP, mInterp);
	SetTheIReal(dlg, ID_SCALEX, mScaleX, 1);
	SetTheIReal(dlg, ID_SCALEY, mScaleY, 1);

// set graph
	err = EI_GraphControlRequest(ID2Ctl(dlg, ID_GRAPH), EI_GRAPH_CONTROL_REQUEST_CREATE, 0, &ctl);
	if (!err && ctl && mSavedGraph.HasData()) {
		EI_GraphData gd = 0;
		err = EI_GraphControlRequest(ctl, EI_GRAPH_CONTROL_REQUEST_GET_GRAPHDATA, 0, &gd);
		if (!err && gd) 
			err = EI_GraphDataRequest(gd, EI_GRAPH_DATA_REQUEST_IMPORT, mSavedGraph.mSize, mSavedGraph.mData);
	} 

// set gradient
	err = EI_GradientControlRequest(ID2Ctl(dlg, ID_GRADIENT), EI_GRADIENT_CONTROL_REQUEST_CREATE, 0, &ctl);
	if (!err && ctl && mSavedGradient.HasData()) {
		EI_GradientData gr = 0;
		err = EI_GradientControlRequest(ctl, EI_GRADIENT_CONTROL_REQUEST_GET_GRADIENTDATA, 0, &gr);
		if (!err && gr) 
			err = EI_GradientDataRequest(gr, EI_GRADIENT_DATA_REQUEST_IMPORT, mSavedGradient.mSize, mSavedGradient.mData);
	} 

	EI_SetDialogTitle(dlg, pluginName);
}

void CColorMesh::GetData( EI_Dialog * dlg )
{
	SInt32 err, size;
	mStepX = GetTheINum(dlg, ID_STEPS);
	mInterp = GetTheIPopUp(dlg, ID_INTERP);
	mScaleX = GetTheIReal(dlg, ID_SCALEX);
	mScaleY = GetTheIReal(dlg, ID_SCALEY);

// read graph
	EI_GraphData gd = 0;
	err = EI_GraphControlRequest(ID2Ctl(dlg, ID_GRAPH), EI_GRAPH_CONTROL_REQUEST_GET_GRAPHDATA, 0, &gd);
	if (!err && gd) {
		err = EI_GraphDataRequest(gd, EI_GRAPH_DATA_REQUEST_GET_DATASIZE, 0, &size);
		if (!err && size && mSavedGraph.Alloc(size)) 
			err = EI_GraphDataRequest(gd, EI_GRAPH_DATA_REQUEST_EXPORT, mSavedGraph.mSize, mSavedGraph.mData);
	} 
	
// read gradient
	EI_GradientData gr = 0;
	err = EI_GradientControlRequest(ID2Ctl(dlg, ID_GRADIENT), EI_GRADIENT_CONTROL_REQUEST_GET_GRADIENTDATA, 0, &gr);
	if (!err && gr) {
		err = EI_GradientDataRequest(gr, EI_GRADIENT_DATA_REQUEST_GET_DATASIZE, 0, &size);
		if (!err && size && mSavedGradient.Alloc(size)) 
			err = EI_GradientDataRequest(gr, EI_GRADIENT_DATA_REQUEST_EXPORT, mSavedGradient.mSize, mSavedGradient.mData);
	} 

	SaveSelf();	
}

void CColorMesh::SaveSelf( void )
{
	CStream strm;
	Write(strm);
	if (!AllocControlData(strm.mPos)) return;
	CMemStream strm2(theInfo->theControlData, theInfo->theControlDataSize);
	Write(strm2);
}

CColorMesh * CColorMesh::ReadControlData( void )
{
	CColorMesh * mesh = new CColorMesh();
	if (theInfo->theControlData && theInfo->theControlDataSize) {
		CMemStream strm2(theInfo->theControlData, theInfo->theControlDataSize);
		if (!mesh->Read(strm2)) mesh->Init();
	} 
	return mesh;
}

void CColorMesh::Generate( void )
{
	SInt32 i, err, v0, v1, v2, v3;
	mStepX = MAX(mStepX, 2);
	float step = 1.0f / (mStepX - 1);

// install GraphData 
	EI_GraphData gd = 0;
	err = EI_GraphDataRequest(0, EI_GRAPH_DATA_REQUEST_CREATE, 0, &gd);
	if (err || !gd) return;

	if (mSavedGraph.mSize) {
		err = EI_GraphDataRequest(gd, EI_GRAPH_DATA_REQUEST_IMPORT, mSavedGraph.mSize, mSavedGraph.mData);
		if (err) return;
	} 
		
// install GradientData 
	EI_GradientData gr = 0;
	err = EI_GradientDataRequest(0, EI_GRADIENT_DATA_REQUEST_CREATE, 0, &gr);
	if (err || !gr) return;

	if (mSavedGradient.mSize) {
		err = EI_GradientDataRequest(gr, EI_GRADIENT_DATA_REQUEST_IMPORT, mSavedGradient.mSize, mSavedGradient.mData);
		if (err) return;
	} 
	
// generate mesh		
	ARGB_Real color[2];
	for (i = 0; i < mStepX; ++i) {
		float x = i * step;
		float y = x;
		err = EI_GraphDataRequest(gd, EI_GRAPH_DATA_REQUEST_GET_VALUE, 0, &y);
		if (err) break;
		color[1].a = x;
		err = EI_GradientDataRequest(gr, EI_GRADIENT_DATA_REQUEST_GET_COLOR, 0, color + 1);
		if (err) break;
		v3 = AddVertex(x, -y, color[1]);
		v2 = AddVertex(x, +y, color[1]);
		if (v2 < 0 || v3 < 0) break;
		if (i && !AddQuad(v0, v1, v2, v3, color)) break;
		v0 = v3;
		v1 = v2;
		color[0] = color[1];
	}
	
// destroy customs	
	EI_GraphDataRequest(gd, EI_GRAPH_DATA_REQUEST_DESTROY, 0, 0);
	EI_GradientDataRequest(gr, EI_GRADIENT_DATA_REQUEST_DESTROY, 0, 0);
}

SInt32 CColorMesh::AddVertex( float x, float y, const ARGB_Real & color )
{
	HostModelAddVertexRec taddv;
	taddv.theReference = theInfo->theHostReference;
	
	taddv.theTexturePosition.x = x;
	taddv.theTexturePosition.y = y;
	taddv.theTexturePosition.z = 0;

	taddv.thePosition.x = x * mScaleX * 0.5f;
	taddv.thePosition.y = y * mScaleY;
	taddv.thePosition.z = 0;
	taddv.theBlurPosition = taddv.thePosition;
	TransformDCoordinate(&taddv.thePosition, &mGroupMatrix);
	TransformDCoordinate(&taddv.theBlurPosition, &mGroupBlurMatrix);
	
	taddv.theNormal.x = 0;
	taddv.theNormal.y = 0;
	taddv.theNormal.z = -1;
	TransformDCoordinate(&taddv.theNormal, &mGroupRotMatrix);

	taddv.theColor.argb.a = uchar(MIN(color.a, 1) * 255 + 0.5f);
	taddv.theColor.argb.r = uchar(MIN(color.r, 1) * 255 + 0.5f);
	taddv.theColor.argb.g = uchar(MIN(color.g, 1) * 255 + 0.5f);
	taddv.theColor.argb.b = uchar(MIN(color.b, 1) * 255 + 0.5f);

	return HostCall('ADDV', sizeof(taddv), &taddv) ? taddv.theIndex : -1;
}

