/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef STREAMEI_HPP
#define STREAMEI_HPP

#include "Stream.hpp"
#include "EI_Stream.h"

struct CStreamEI : public CStream {
	CStreamEI( EI_Stream * src ) : strm(src) {}
	
	virtual void Seek	 ( slong pos );
	virtual slong GetSize( void );

	virtual void Read    ( void * dst, SInt32 size );
	virtual void Write   ( const void * dst, SInt32 size );
	virtual bool NextTag ( UInt32 sign );
	virtual void ReadBlk ( void * dst, SInt32 size );
	virtual void WriteBlk( const void * dst, SInt32 size );

// data members
	EI_Stream * strm;
};

inline void CStreamEI::Read ( void * dst, SInt32 size )
{
	if (mError) return;

	SInt32 count = size;
	SInt32 result = EI_ReadStream(strm, (char *) dst,	&count);
	if (result != 0 || count != size) mError = STRM_ERR_READ;
	else {
		mPos += size; 
#ifdef __LITTLE_ENDIAN__
		ReverseField(dst, size);
#endif
	} 
}

inline void CStreamEI::Write( const void * dst, SInt32 size )
{
	if (mError) return;

#ifdef __LITTLE_ENDIAN__
	ReverseField((void *) dst, size);
#endif

	SInt32 count = size;
	SInt32 result = EI_WriteStream(strm, (char *) dst, &count);

#ifdef __LITTLE_ENDIAN__
	ReverseField((void *) dst, size);
#endif

	if (result != 0 || count != size) mError = STRM_ERR_READ;
	else mPos += size; 
}

inline bool CStreamEI::NextTag( UInt32 sign )
{
	if (mError) return false;
	if (slong(mPos + sizeof(UInt32)) > EI_GetStreamSize(strm)) return false;
	if (ReadULong() == sign && !mError) return true;
	if (!mError) EI_SetStreamPosition(strm, mPos - sizeof(UInt32));
	return false;
}

inline void CStreamEI::ReadBlk( void * dst, SInt32 size )
{
	if (mError) return;

	SInt32 count = size;
	SInt32 result = EI_ReadStream(strm, (char *) dst,	&count);
	if (result != 0 || count != size) mError = STRM_ERR_READ;
	else mPos += size; 
}

inline void CStreamEI::WriteBlk( const void * dst, SInt32 size )
{
	if (mError) return;

	SInt32 count = size;
	SInt32 result = EI_WriteStream(strm, (char *) dst, &count);

	if (result != 0 || count != size) mError = STRM_ERR_READ;
	else mPos += size; 
}

inline void CStreamEI::Seek( slong pos ) 
{ 
	if (mError) return;
	if (EI_SetStreamPosition(strm, pos) != 0) mError = STRM_ERR_SEEK;
	else mPos = pos; 
}

inline slong CStreamEI::GetSize( void )
{
	slong size = EI_GetStreamSize(strm);
	if (size < 0) mError = STRM_ERR_SEEK;
	return size; 
}


#endif // STREAMEI_HPP