/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifdef _WINDOWS
#pragma warning(disable : 4244)
#endif

#include <math.h>
#include "EI_ImageBuffer.h"
#include "FACTStuff.h"
#include "PluginServer.h"
#include "PluginFPU.h"
#include "PluginUtils.h"
#include "PString.h"
#include "Noise.h"

/********************************************************************************************************/
/*			Datatypes																					*/
/********************************************************************************************************/
 typedef struct PluginDataRec {
    DCoordinate theLightPosition;
    L_Rect theImageRect;
    double theLightRadius;
    ARGB theLightColor;
  } PluginDataRec, *PluginDataPtr;

/********************************************************************************************************/
/*			Prototypes																					*/
/********************************************************************************************************/

SInt32 Main (UInt32 theCommand, SInt32 theCommandRecSize, void* theCommandRec);

/********************************************************************************************************/
/*									AllocateMemory														*/
/********************************************************************************************************/
static SInt32 AllocateMemory(PluginInfoRec *theInfo)
{
 	HostAllocateRec theHostAllocate;
	SInt32 theError = 0;

 	theHostAllocate.theReference = theInfo->theHostReference;
	theHostAllocate.theControlData = theInfo->theControlData;
	theHostAllocate.theControlDataSize = 0;
	theHostAllocate.thePluginData = theInfo->thePluginData;
	theHostAllocate.thePluginDataSize = sizeof(PluginDataRec);

	theError = theInfo->theHostProc(PLUGCOMMAND(hostAllocate), PARAMETERSIZE(sizeof(HostAllocateRec)), &theHostAllocate);

	if (theError == 0) {
 		theInfo->theControlData = theHostAllocate.theControlData;
		theInfo->theControlDataSize = theHostAllocate.theControlDataSize;
		theInfo->thePluginData = theHostAllocate.thePluginData;
		theInfo->thePluginDataSize = theHostAllocate.thePluginDataSize;
	}
	return theError;
}  /* AllocateMemory */

/********************************************************************************************************/
/*									Status																*/
/********************************************************************************************************/
static void StatusCopyString(unsigned char *dst, unsigned char *src)
{
	SInt32 i;
	
	dst[0] = src[0];
	for (i = 1; i <= src[0]; i++)
		dst[i] = src[i];
}  /* StatusCopyString */

#define StatusTime(theStatus) (((theStatus)->theStatusTick == 0) || ((theStatus)->theStatusTick <= TICKCOUNT()))

static void StatusUpdate(PluginInfoPtr theInfo, HostStatusRec *theStatus, float thePos, SInt32 theValue, SInt32 *theResult)
{
	if (*theResult == 0) {
		theStatus->theReference = theInfo->theHostReference;
		theStatus->theStatusValue = theValue;
		theStatus->theStatusPos = thePos;
		*theResult = theInfo->theHostProc(PLUGCOMMAND(hostStatus), PARAMETERSIZE(sizeof(HostStatusRec)), theStatus);
	}
}  /* EIPluginStatusUpdate */

static void StatusSetup(PluginInfoPtr theInfo, HostStatusRec *theStatus, PStrPtr theUnit, PStrPtr theHeading, SInt32 min, SInt32 max, float thePos, SInt32 theValue, SInt32 *theResult)
{
	theStatus->theReference = theInfo->theHostReference;
	theStatus->theStatusPos = thePos;
	StatusCopyString(theStatus->theStatusUnit, theUnit);
	StatusCopyString(theStatus->theStatusInfo, theHeading);
	theStatus->theStatusMin = min;
	theStatus->theStatusMax = max;
	theStatus->theStatusValue = theValue;
	theStatus->theStatusTick = 0;
	StatusUpdate(theInfo, theStatus, thePos, theValue, theResult);
}  /* EIPluginStatusSetup */

static void StatusFinish(PluginInfoPtr theInfo, HostStatusRec *theStatus, SInt32 *theResult)
{
	if ((*theResult == 0) && theStatus->theStatusTick)
		StatusUpdate(theInfo, theStatus, 1.0, theStatus->theStatusMax, theResult);

}  /* EIPluginStatusFinish */

/********************************************************************************************************/
/*									FlareSetup															*/
/********************************************************************************************************/
static SInt32 FlareSetup(SInt32 theCommandRecSize, void* theCommandRec)
{
	PluginFlareSetupPtr theFlareSetupPtr = (PluginFlareSetupPtr)theCommandRec;
	PluginInfoRec theInfo;
	PluginDataPtr thePluginDataPtr;
  	SInt32 theError = 0;

	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginFlareSetupRec)))  
		theError = pluginParameterErr;
	else {
		theInfo = theFlareSetupPtr->theInfo;
		thePluginDataPtr = (PluginDataPtr)theInfo.thePluginData;
 		thePluginDataPtr->theLightPosition.x = theFlareSetupPtr->theLightFramePosition.x;
		thePluginDataPtr->theLightPosition.y = theFlareSetupPtr->theLightFramePosition.y;
		thePluginDataPtr->theImageRect = theFlareSetupPtr->theFrameRect;
		thePluginDataPtr->theLightRadius = theFlareSetupPtr->theLightFrameRadius;
		thePluginDataPtr->theLightColor = theFlareSetupPtr->theLightColor;
 	}
	return theError;
}   /* FlareSetup */

/********************************************************************************************************/
/*									FlareGenerate														*/
/********************************************************************************************************/
/*	This function draws into the appropriate buffer and flare elements that may appear in the rectangle */
/*	requested by the Host.																				*/
/********************************************************************************************************/
static SInt32 FlareGenerate (SInt32 theCommandRecSize, void* theCommandRec)
{
	PluginFlareGeneratePtr theFlareGenerate = (PluginFlareGeneratePtr)theCommandRec;
	PluginInfoRec theInfo;
	HostStatusRec theStatus;
	FlareBuffer theFlareBuffer;
	PluginDataPtr thePluginData;
	char *theScan;
	double xDist, yDist, theDistance, theRadius;
	L_Rect theFlareRect, theVisRect;
	SInt32 x, y, lightX, lightY;
	SInt32 r, g, b;
	SInt32 theError = 0;

	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginFlareGenerateRec)))  
		theError = pluginParameterErr;
	else {
		theInfo = theFlareGenerate->theInfo;
		thePluginData = (PluginDataPtr)theInfo.thePluginData;
		theFlareBuffer = theFlareGenerate->theFlareBuffer;

		/* Find the flare's center, radius and extent rectangle */
		
		lightX = thePluginData->theLightPosition.x;
		lightY = thePluginData->theLightPosition.y;
		theRadius = thePluginData->theLightRadius;
		if (theRadius <= 0.0)
			theRadius = 100.0;
		SetLRect(&theFlareRect, lightX - theRadius * 1.5, lightY - theRadius * 1.5, lightX + theRadius * 1.5, lightY + theRadius * 1.5);
		if (SectLRect(&theFlareBuffer.bufferRect, &theFlareRect, &theVisRect))

			/* If the bufferDepth is 48 bits, draw a color flare (up to version 2.7.5) */
			
			if (theFlareBuffer.bufferDepth == 48) {
				theRadius = -4.0 / (theRadius * theRadius);
	
				/* Calculate the pixel address of the top left corner of the visible rectangle */
				
				theScan = theFlareBuffer.bufferBase;
				theScan += (theVisRect.top - theFlareBuffer.bufferRect.top) * theFlareBuffer.bufferRowBytes;
				theScan += (theVisRect.left - theFlareBuffer.bufferRect.left) * sizeof(FlareColor);
	
				/* Find the flare's color */
				
				r = (thePluginData->theLightColor.argb.r * 16320L) / 255;
				g = (thePluginData->theLightColor.argb.g * 16320L) / 255;
				b = (thePluginData->theLightColor.argb.b * 16320L) / 255;
				
				/* Setup the status */
				
				StatusSetup(&theInfo, &theStatus, PS("Scanline"), PS("Generating Glow Flare"), theVisRect.top, theVisRect.bottom, 0.0, theVisRect.top, &theError);
				
				/* Draw the flare into the buffer */
				
				for (y = theVisRect.top; (y < theVisRect.bottom) && (theError == 0); y++) {
					FlareColorPtr thePixel = (FlareColorPtr)theScan;
					for (x = theVisRect.left; x < theVisRect.right; x++) {
						xDist = FABS(lightX - x);
						yDist = FABS(lightY - y);
						theDistance = xDist * xDist + yDist * yDist;
						theDistance = EXP(theDistance * theRadius);
						thePixel->red += MIN(thePixel->red + r * theDistance, 16320);
						thePixel->green += MIN(thePixel->green + g * theDistance, 16320);
						thePixel->blue += MIN(thePixel->blue + b * theDistance, 16320);
						thePixel++;
					}
					theScan += theFlareBuffer.bufferRowBytes;
					
					/* Update the status */
					
					if (StatusTime(&theStatus))
						StatusUpdate(&theInfo, &theStatus, (y - theVisRect.top) / (double)(theVisRect.height), y, &theError);
				}
				
				/* Finish the status */
				
				StatusFinish(&theInfo, &theStatus, &theError);

			/* If the bufferDepth is 32 bits, draw a color flare (version 2.8 and beyond) */
			
			} else if (theFlareBuffer.bufferDepth == 32) {
				theRadius = -4.0 / (theRadius * theRadius);
	
				/* Calculate the pixel address of the top left corner of the visible rectangle */
				
				theScan = theFlareBuffer.bufferBase;
				theScan += (theVisRect.top - theFlareBuffer.bufferRect.top) * theFlareBuffer.bufferRowBytes;
				theScan += (theVisRect.left - theFlareBuffer.bufferRect.left) * sizeof(ARGB);
	
				/* Find the flare's color */
				
				r = thePluginData->theLightColor.argb.r;
				g = thePluginData->theLightColor.argb.g;
				b = thePluginData->theLightColor.argb.b;
				
				/* Setup the status */
				
				StatusSetup(&theInfo, &theStatus, PS("Scanline"), PS("Generating Glow Flare"), theVisRect.top, theVisRect.bottom, 0.0, theVisRect.top, &theError);

				/* Draw the flare into the buffer */

				for (y = theVisRect.top; (y < theVisRect.bottom) && (theError == 0); y++) {
					EI_32BitPixel *thePixel = (EI_32BitPixel *)theScan;
					UInt32 seed = y;
					
					for (x = theVisRect.left; x < theVisRect.right; x++) {
						SInt32 rSum, gSum, bSum;
						SInt32 factor;
						SInt32 noise = RANDOMBITS(seed, 14);
						
						xDist = FABS(lightX - x);
						yDist = FABS(lightY - y);
						theDistance = xDist * xDist + yDist * yDist;
						factor = 16384.0 * EXP(theDistance * theRadius);

						rSum = EI_GET_PIXEL_RED(*thePixel) + ((r * factor + noise) >> 14);
						gSum = EI_GET_PIXEL_GREEN(*thePixel) + ((g * factor + noise) >> 14);
						bSum = EI_GET_PIXEL_BLUE(*thePixel) + ((b * factor + noise) >> 14);
						
  						EI_SET_PIXEL_RED(*thePixel, MIN(rSum, 255));
 						EI_SET_PIXEL_GREEN(*thePixel, MIN(gSum, 255));
						EI_SET_PIXEL_BLUE(*thePixel, MIN(bSum, 255));
						thePixel++;
					}
					theScan += theFlareBuffer.bufferRowBytes;
					
					/* Update the status */
					
					if (StatusTime(&theStatus))
						StatusUpdate(&theInfo, &theStatus, (y - theVisRect.top) / (double)(theVisRect.height), y, &theError);
				}
				
				/* Finish the status */
				
				StatusFinish(&theInfo, &theStatus, &theError);
			} else if (theFlareBuffer.bufferDepth == 8) {
				
				/* The bufferDepth is 8 bits, so draw the flare's outline */
	
				/* Calculate the pixel address of the top left corner of the visible rectangle */
				
				theScan = theFlareBuffer.bufferBase;
				theScan += (theVisRect.top - theFlareBuffer.bufferRect.top) * theFlareBuffer.bufferRowBytes;
				theScan += theVisRect.left - theFlareBuffer.bufferRect.left;
				
				/* Draw the top edge */
				
				if (theFlareRect.top == theVisRect.top) {
					char *thePixel = theScan;
					for (x = theVisRect.left; x < theVisRect.right; x++) {
						*thePixel = theFlareBuffer.bufferIndex;
						thePixel++;
					}
				}
				
				/* Draw the bottom edge */
				
				if (theFlareRect.bottom == theVisRect.bottom) {
					char *thePixel = theScan + (theVisRect.height - 1) * theFlareBuffer.bufferRowBytes;
					for (x = theVisRect.left; x < theVisRect.right; x++) {
						*thePixel = theFlareBuffer.bufferIndex;
						thePixel++;
					}
				}
				
				/* Draw the left edge */
				
				if (theFlareRect.left == theVisRect.left) {
					char *thePixel = theScan;
					for (y = theVisRect.top; y < theVisRect.bottom; y++) {
						*thePixel = theFlareBuffer.bufferIndex;
						thePixel += theFlareBuffer.bufferRowBytes;
					}
				}
				
				/* Draw the right edge */
				
				if (theFlareRect.right == theVisRect.right) {
					char *thePixel = theScan + (theVisRect.width - 1);
					for (y = theVisRect.top; y < theVisRect.bottom; y++) {
						*thePixel = theFlareBuffer.bufferIndex;
						thePixel += theFlareBuffer.bufferRowBytes;
					}
				}
		}
	}
	return theError;
}   /* FlareGenerate */

/********************************************************************************************************/
/*									Initialize															*/
/********************************************************************************************************/
static SInt32 Initialize (SInt32 theCommandRecSize, void* theCommandRec) 
{
	PluginInitializePtr thePluginInitializePtr = (PluginInitializePtr)theCommandRec;
	SInt32 theError = 0;

	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginInitializeRec))) 
		theError = pluginParameterErr;
	else
 		theError = AllocateMemory(&thePluginInitializePtr->theInfo);
	return theError;
}   /* Initialize */

/********************************************************************************************************/
/*									main																*/
/********************************************************************************************************/
EXPDLL SInt32 Main (UInt32 theCommand, SInt32 theCommandRecSize, void* theCommandRec)
{	
	SInt32 theError = 0;

	switch (HOSTCOMMAND(theCommand))
	{
		case pluginInitialize:		theError = Initialize(theCommandRecSize, theCommandRec);		break;
		case pluginFlareSetup:		theError = FlareSetup(theCommandRecSize, theCommandRec);		break;
		case pluginFlareGenerate:	theError = FlareGenerate(theCommandRecSize, theCommandRec);		break;
	}
	return theError;

}	/* Main */
