/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

/* This code presents an example of a surface patch generator model plugin. */

#ifdef _WINDOWS
#pragma warning(disable : 4244)
#pragma warning(disable : 4305)
#endif

#include "EI_Dialog.h"
#include "EI_ResourceFile.h"
#include "PString.h"
#include "FACTStuff.h"
#include "PluginServer.h"
#include "PluginUtils.h"
#include "FPU.h"

#include <stddef.h>

/* #define __FACETVERTEX__ */

#define interfaceDialogID	16001

enum {
	cFramesBetweenET = 4
};

/********************************************************************************************************/
/*			Datatypes																					*/
/********************************************************************************************************/
typedef struct MorphInfoRec {
	double theSourceFactor;
	double theTargetFactor;
} MorphInfoRec;

typedef struct ControlDataRec {
	int theNumOfMorphFrames;
 } ControlDataRec, *ControlDataPtr;
	
 typedef struct PluginDataRec {
	SInt32 theCurrentFrameIndex;
} PluginDataRec, *PluginDataPtr;

/********************************************************************************************************/
/*			Prototypes																					*/
/********************************************************************************************************/
static void MorphUpdateDialog (EI_Dialog *theDialog, ControlDataRec *theControlData);
static SInt32 DialogInterface (PluginInfoRec theInfo);
static SInt32 theInterface(SInt32 theCommandRecSize, void* theCommandRec);
static void CalculateNewVertex (HostModelAddVertexRec *theNewVertex, HostModelGetVertexRec *theHostModelGetSourceVertex, HostModelGetVertexRec *theHostModelGetTargetVertex, MorphInfoRec *theMorphInfo);
static SInt32 InterpolateVertex(PluginInfoRec *theInfo, HostModelGetGroupRec *theSourceGroup, HostModelGetGroupRec *theTargetGroup, MorphInfoRec *theMorphInfo, SInt32* theVertex); 
static SInt32 InterpolateVertices(PluginInfoRec *theInfo, HostModelGetGroupRec *theSourceGroup, HostModelGetGroupRec *theTargetGroup, MorphInfoRec *theMorphInfo) ;
static SInt32 InterpolateFacets(PluginInfoRec *theInfo, HostModelGetGroupRec *theSourceGroup, HostModelGetGroupRec *theTargetGroup, MorphInfoRec *theMorphInfo);
static SInt32 CreateCurrentGroup (PluginInfoRec *theInfo, int theNumOfChildGroups);
static SInt32 ModelGenerate (SInt32 theCommandRecSize, void* theCommandRec);
static SInt32 ModelSetup (SInt32 theCommandRecSize, void* theCommandRec);
static SInt32 AllocateMemory (PluginInfoRec *theInfo);
static SInt32 Initialize (SInt32 theCommandRecSize, void* theCommandRec);
static SInt32 DoInformation(SInt32 theCommandRecSize, void* theCommandRec);

#define pluginType			QUADBYTE('M', 'O', 'R', 'F')		/* The owner plugin's unique type */
#define pluginVersion		0x00000002							/* The owner plugin's version number 0:0:version:revision */
#define pluginName			PS("ElectricImage Mesh")
#define pluginCopyright		PS("Copyright  1990-1998 ElectricImage, Inc.")
#define pluginAuthor		PS("Electric Image")
#define pluginDescription	PS("This model plugin is a simple morph example.")
#define pluginPermission	BIT(pluginSensitivityFlag) + BIT(pluginTimeSensitivityFlag) + BIT(pluginChildSensitivityFlag)

/********************************************************************************************************/
/*									UpdateDialog														*/
/********************************************************************************************************/
static void MorphUpdateDialog (EI_Dialog *theDialogPtr, ControlDataRec *theControlData)
{
	SetTheINum(theDialogPtr, cFramesBetweenET, theControlData->theNumOfMorphFrames);
}   /* UpdateDialog */

/********************************************************************************************************/
/*									DialogInterface														*/
/********************************************************************************************************/
static SInt32 DialogInterface (PluginInfoRec theInfo) 
{
	EI_ResourceFile		*resourceFile;
	EI_Dialog			*theDialog;
	ControlDataPtr		theControlDataPtr = (ControlDataPtr)theInfo.theControlData;
	SInt32				theError = 0;
	
	resourceFile = OpenResourceFile("Morph.rsc");
	if (resourceFile == NULL)
		theError = pluginResourceErr;
	
	if (theError == 0) {
		theDialog = EI_MakeDialog(resourceFile, interfaceDialogID);
		if (theDialog != NULL) {
			MorphUpdateDialog(theDialog, theControlDataPtr);
			if (EI_ExecuteDialog(theDialog))
				theControlDataPtr->theNumOfMorphFrames = GetTheINum(theDialog, cFramesBetweenET);
			else
				theError = pluginCancelErr;
			EI_DestroyDialog(theDialog);
		}
		EI_CloseResourceFile(resourceFile);
	}
 	return theError;
}   /* DialogInterface */

/********************************************************************************************************/
/*									theInterface														*/
/********************************************************************************************************/
static SInt32 theInterface(SInt32 theCommandRecSize , void* theCommandRec) 
{
	PluginInterfacePtr thePluginInterfacePtr = (PluginInterfacePtr)theCommandRec;
	PluginInfoRec theInfo = thePluginInterfacePtr->theInfo;
 	SInt32 theError = 0;

 	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginInterfaceRec)))  
		theError = pluginParameterErr;
	else 
		theError = DialogInterface(theInfo);
	return theError;
}   /* theInterface */

/********************************************************************************************************/
/*									CalculateNewVertex													*/
/********************************************************************************************************/
/*	This routine interpolates the source and target vertex data and returns the interpolated vertex 	*/
/********************************************************************************************************/
static void CalculateNewVertex (HostModelAddVertexRec *theNewVertex, HostModelGetVertexRec *theHostModelGetSourceVertex, HostModelGetVertexRec *theHostModelGetTargetVertex, MorphInfoRec *theMorphInfo)
{
	double distance;
	double theSourceFactor = theMorphInfo->theSourceFactor;
	double theTargetFactor = theMorphInfo->theTargetFactor;

	theNewVertex->thePosition.x = theHostModelGetSourceVertex->thePosition.x * theSourceFactor + theHostModelGetTargetVertex->thePosition.x * theTargetFactor;
	theNewVertex->thePosition.y = theHostModelGetSourceVertex->thePosition.y * theSourceFactor + theHostModelGetTargetVertex->thePosition.y * theTargetFactor;
	theNewVertex->thePosition.z = theHostModelGetSourceVertex->thePosition.z * theSourceFactor + theHostModelGetTargetVertex->thePosition.z * theTargetFactor;

 	theNewVertex->theColor.argb.a = theHostModelGetSourceVertex->theColor.argb.a * theSourceFactor + theHostModelGetTargetVertex->theColor.argb.a * theTargetFactor + 0.5;
 	theNewVertex->theColor.argb.r = theHostModelGetSourceVertex->theColor.argb.r * theSourceFactor + theHostModelGetTargetVertex->theColor.argb.r * theTargetFactor + 0.5;
	theNewVertex->theColor.argb.g = theHostModelGetSourceVertex->theColor.argb.g * theSourceFactor + theHostModelGetTargetVertex->theColor.argb.g * theTargetFactor + 0.5;
	theNewVertex->theColor.argb.b = theHostModelGetSourceVertex->theColor.argb.b * theSourceFactor + theHostModelGetTargetVertex->theColor.argb.b * theTargetFactor + 0.5;

 	theNewVertex->theNormal.x = theHostModelGetSourceVertex->theNormal.x * theSourceFactor + theHostModelGetTargetVertex->theNormal.x * theTargetFactor;
	theNewVertex->theNormal.y = theHostModelGetSourceVertex->theNormal.y * theSourceFactor + theHostModelGetTargetVertex->theNormal.y * theTargetFactor;
	theNewVertex->theNormal.z = theHostModelGetSourceVertex->theNormal.z * theSourceFactor + theHostModelGetTargetVertex->theNormal.z * theTargetFactor;
	distance = SQRT(SQR(theNewVertex->theNormal.x) + SQR(theNewVertex->theNormal.y) + SQR(theNewVertex->theNormal.z));
	if (distance > 1.0e-6) {
		theNewVertex->theNormal.x /= distance;
		theNewVertex->theNormal.y /= distance;
		theNewVertex->theNormal.z /= distance;
	}

 	theNewVertex->theBlurPosition.x = theHostModelGetSourceVertex->theBlurPosition.x * theSourceFactor + theHostModelGetTargetVertex->theBlurPosition.x * theTargetFactor;
	theNewVertex->theBlurPosition.y = theHostModelGetSourceVertex->theBlurPosition.y * theSourceFactor + theHostModelGetTargetVertex->theBlurPosition.y * theTargetFactor;
	theNewVertex->theBlurPosition.z = theHostModelGetSourceVertex->theBlurPosition.z * theSourceFactor + theHostModelGetTargetVertex->theBlurPosition.z * theTargetFactor;

 	theNewVertex->theTexturePosition.x = theHostModelGetSourceVertex->theTexturePosition.x * theSourceFactor + theHostModelGetTargetVertex->theTexturePosition.x * theTargetFactor;
	theNewVertex->theTexturePosition.y = theHostModelGetSourceVertex->theTexturePosition.y * theSourceFactor + theHostModelGetTargetVertex->theTexturePosition.y * theTargetFactor;
	theNewVertex->theTexturePosition.z = theHostModelGetSourceVertex->theTexturePosition.z * theSourceFactor + theHostModelGetTargetVertex->theTexturePosition.z * theTargetFactor;
}   /* CalculateNewVertex */

/********************************************************************************************************/
/*									InterpolateVertex													*/
/********************************************************************************************************/
/*	This routine interpolates a single vertex by retrieving two vertices at the same index from the 	*/
/*	source and target and passing them to CalculateNewVertex for interpolation.							*/
/********************************************************************************************************/
static SInt32 InterpolateVertex(PluginInfoRec *theInfo, HostModelGetGroupRec *theSourceGroup, HostModelGetGroupRec *theTargetGroup, MorphInfoRec *theMorphInfo, SInt32* theVertex) 
{
	HostModelAddVertexRec theNewVertex;
 	HostModelGetVertexRec theSourceVertex, theTargetVertex;
   	SInt32 theError = 0;
 
 	theSourceVertex.theReference = theInfo->theHostReference;
	theSourceVertex.theGroup = theSourceGroup->theGroup;

 	theTargetVertex.theReference = theInfo->theHostReference;
	theTargetVertex.theGroup = theTargetGroup->theGroup;
 
 	theNewVertex.theReference = theInfo->theHostReference;
	
	if (*theVertex > theSourceGroup->theGroupVertices)
 		theSourceVertex.theVertexIndex = theSourceGroup->theGroupVertices; 
	else
 		theSourceVertex.theVertexIndex = *theVertex; 
	if (*theVertex > theTargetGroup->theGroupVertices)
 		theTargetVertex.theVertexIndex = theTargetGroup->theGroupVertices; 
	else
		theTargetVertex.theVertexIndex = *theVertex;

	theError = theInfo->theHostProc(PLUGCOMMAND(hostModelGetVertex), PARAMETERSIZE(sizeof(HostModelGetVertexRec)), &theSourceVertex);   

	if (theError == 0)
		theError = theInfo->theHostProc(PLUGCOMMAND(hostModelGetVertex), PARAMETERSIZE(sizeof(HostModelGetVertexRec)), &theTargetVertex);

	if (theError == 0) {
		CalculateNewVertex(&theNewVertex, &theSourceVertex, &theTargetVertex, theMorphInfo);  
		
		theError = theInfo->theHostProc(PLUGCOMMAND(hostModelAddVertex), PARAMETERSIZE(sizeof(HostModelAddVertexRec)), &theNewVertex);
		*theVertex = theNewVertex.theIndex;	
	}
	return theError;
}	/* InterpolateVertex */

/********************************************************************************************************/
/*									InterpolateVertices													*/
/********************************************************************************************************/
/*	This routine interpolates the vertices by retrieving them in pairs from the source and target and 	*/
/*	passing them to CalculateNewVertex for interpolation.												*/
/********************************************************************************************************/
static SInt32 InterpolateVertices(PluginInfoRec *theInfo, HostModelGetGroupRec *theSourceGroup, HostModelGetGroupRec *theTargetGroup, MorphInfoRec *theMorphInfo) 
{
	int i;
   	SInt32 theVertex;
	HostStatusRec theHostStatus;
   	SInt32 theError = 0;

 	theHostStatus.theReference = theInfo->theHostReference;
	PStrCpy(theHostStatus.theStatusUnit, PS("Morph"));
	PStrCpy(theHostStatus.theStatusInfo, PS("Interpolating Vertices"));
	theHostStatus.theStatusMin = 1;
	theHostStatus.theStatusMax = theSourceGroup->theGroupVertices;
	theHostStatus.theStatusTick = 0;
	
	for (i = 1; (i <= theSourceGroup->theGroupVertices) && (theError == 0); i++) {
		if ((theHostStatus.theStatusTick == 0) || (TICKCOUNT() >= theHostStatus.theStatusTick)) {
 			theHostStatus.theStatusValue = i;
			theHostStatus.theStatusPos = (float)i / theSourceGroup->theGroupVertices;
 			theError = theInfo->theHostProc(PLUGCOMMAND(hostStatus), PARAMETERSIZE(sizeof(HostStatusRec)), &theHostStatus);      
		}
		
		if (theError == 0) {
			theVertex = i;
			InterpolateVertex(theInfo, theSourceGroup, theTargetGroup, theMorphInfo, &theVertex);
		}
 	} 
	return theError;
}	/* InterpolateVertices */

/********************************************************************************************************/
/*									InterpolateFacets													*/
/********************************************************************************************************/
/*	This routine interpolates the source and target facet data and returns the interpolated facet. 		*/
/*	Since the facets only contain indices to the vertices, these indices are just copied into the new	*/
/*	facet.   The vertices are interpolated by the function InterpolateVertices.							*/
/********************************************************************************************************/
static SInt32 InterpolateFacets(PluginInfoRec *theInfo, HostModelGetGroupRec *theSourceGroup, HostModelGetGroupRec *theTargetGroup, MorphInfoRec *theMorphInfo) 
{
	HostStatusRec  theHostStatus;
	SInt32 i, j;
	SInt32 theVertex;
	HostModelGetFacetRec theSourceFacet, theTargetFacet;
	HostModelAddFacetRec theNewFacet;
	SInt32 theError = 0;

	double theSourceFactor = theMorphInfo->theSourceFactor;
	double theTargetFactor = theMorphInfo->theTargetFactor;

 	theSourceFacet.theReference = theInfo->theHostReference;
	theSourceFacet.theGroup = theSourceGroup->theGroup;

	theTargetFacet.theReference = theInfo->theHostReference;
	theTargetFacet.theGroup = theTargetGroup->theGroup;

 	theNewFacet.theReference = theInfo->theHostReference;
 
 	theHostStatus.theReference = theInfo->theHostReference;
	PStrCpy(theHostStatus.theStatusUnit, PS("Morph"));
	PStrCpy(theHostStatus.theStatusInfo, PS("Interpolating Facets"));
	theHostStatus.theStatusMin = 1;
	theHostStatus.theStatusMax = theSourceGroup->theGroupFacets;
	theHostStatus.theStatusTick = 0;

	for (i = 1; (i <= theSourceGroup->theGroupFacets) && (theError == 0); i++) {
 		theSourceFacet.theFacetIndex = i;
 		if (i > theTargetGroup->theGroupFacets)
 			theTargetFacet.theFacetIndex = theTargetGroup->theGroupFacets;
		else
 			theTargetFacet.theFacetIndex = i;

		if ((theHostStatus.theStatusTick == 0) || (TICKCOUNT() >= theHostStatus.theStatusTick)) {
			theHostStatus.theStatusValue = i;
			theHostStatus.theStatusPos = (float)i / theSourceGroup->theGroupFacets;
 			theError = theInfo->theHostProc(PLUGCOMMAND(hostStatus), PARAMETERSIZE(sizeof(HostStatusRec)), &theHostStatus); 
 		}    

		if (theError == 0)
			theError = theInfo->theHostProc(PLUGCOMMAND(hostModelGetFacet), PARAMETERSIZE(sizeof(HostModelGetFacetRec)), &theSourceFacet);
		if (theError == 0)
			theError = theInfo->theHostProc(PLUGCOMMAND(hostModelGetFacet), PARAMETERSIZE(sizeof(HostModelGetFacetRec)), &theTargetFacet);

		if (theError == 0) {
	 		theNewFacet.theNumVertices = theSourceFacet.theNumVertices;
			for (j = 0; (j < theNewFacet.theNumVertices) && (theError == 0); j++) {
				theVertex = theSourceFacet.theVertices[j];
	#ifdef __FACETVERTEX__
				theError = InterpolateVertex(theInfo, theSourceGroup, theTargetGroup, theMorphInfo, &theVertex);
	#endif
				theNewFacet.theVertices[j] = theVertex;
			}
			for (; j <= 3; j++)
				theNewFacet.theVertices[j] = 0;
		}
 		
		if (theError == 0) {
			theNewFacet.theColor.argb.a = theSourceFacet.theColor.argb.a * theSourceFactor + theTargetFacet.theColor.argb.a * theTargetFactor + 0.5;  /* Interpolate the color of Facets */
	 		theNewFacet.theColor.argb.r = theSourceFacet.theColor.argb.r * theSourceFactor + theTargetFacet.theColor.argb.r * theTargetFactor + 0.5;
			theNewFacet.theColor.argb.g = theSourceFacet.theColor.argb.g * theSourceFactor + theTargetFacet.theColor.argb.g * theTargetFactor + 0.5;
			theNewFacet.theColor.argb.b = theSourceFacet.theColor.argb.b * theSourceFactor + theTargetFacet.theColor.argb.b * theTargetFactor + 0.5;
	
	  		theError = theInfo->theHostProc(PLUGCOMMAND(hostModelAddFacet), PARAMETERSIZE(sizeof(HostModelAddFacetRec)), &theNewFacet);
	  	}
	}
	return theError;
}   /* InterpolateFacets */

/********************************************************************************************************/
/*									CreateCurrentGroup													*/
/********************************************************************************************************/
/*	This routine is responsible for the morphing.  First, it takes as input the current time.  It uses 	*/
/*	this to determine which two children of the plugin to use for the current morph.  It calls routines	*/
/*	to interpolate their data, and then calls the appropriate number of 'hostModelDeleteGroup's to hide */
/*	the children.																						*/
/********************************************************************************************************/
static SInt32 CreateCurrentGroup (PluginInfoRec *theInfo, int theNumOfChildGroups) 
{
	ControlDataPtr theControlDataPtr = (ControlDataPtr)theInfo->theControlData;
	PluginDataPtr thePluginDataPtr = (PluginDataPtr)theInfo->thePluginData;
 	int i, theSource, theTarget;
 	HostModelGetGroupRec theSourceGroup, theTargetGroup;
  	HostModelDeleteGroupRec theHostModelDeleteGroup;
	MorphInfoRec theMorphInfo;
 	SInt32 theError = 0;

	theSource = ((thePluginDataPtr->theCurrentFrameIndex / theControlDataPtr->theNumOfMorphFrames) % theNumOfChildGroups) + 1;
	theTarget = (theSource % theNumOfChildGroups) + 1;
	
  	theSourceGroup.theReference = theInfo->theHostReference;
	theSourceGroup.theGroupIndex = theSource;

	theTargetGroup.theReference = theInfo->theHostReference;
	theTargetGroup.theGroupIndex = theTarget;

	theError = theInfo->theHostProc(PLUGCOMMAND(hostModelGetGroup), PARAMETERSIZE(sizeof(HostModelGetGroupRec)), &theSourceGroup);
	if (theError == 0)
		theError = theInfo->theHostProc(PLUGCOMMAND(hostModelGetGroup), PARAMETERSIZE(sizeof(HostModelGetGroupRec)), &theTargetGroup);

 	theMorphInfo.theTargetFactor = ((float)(thePluginDataPtr->theCurrentFrameIndex % theControlDataPtr->theNumOfMorphFrames)) / (float)(theControlDataPtr->theNumOfMorphFrames);
	theMorphInfo.theSourceFactor = 1 - theMorphInfo.theTargetFactor;

#ifndef __FACETVERTEX__
	if (theError == 0)
	  	theError = InterpolateVertices (theInfo, &theSourceGroup, &theTargetGroup, &theMorphInfo);
#endif
	if (theError == 0)
	 	theError = InterpolateFacets (theInfo, &theSourceGroup, &theTargetGroup, &theMorphInfo);
  
	for (i = 1; (i <= theNumOfChildGroups) && (theError == 0); i++) {
  		theSourceGroup.theReference = theInfo->theHostReference;
		theSourceGroup.theGroupIndex = i;
		
		theError = theInfo->theHostProc(PLUGCOMMAND(hostModelGetGroup), PARAMETERSIZE(sizeof(HostModelGetGroupRec)), &theSourceGroup);

 		if (theError == 0) {
			theHostModelDeleteGroup.theReference = theInfo->theHostReference;
			theHostModelDeleteGroup.theGroup = theSourceGroup.theGroup;
			theError = theInfo->theHostProc(PLUGCOMMAND(hostModelDeleteGroup), PARAMETERSIZE(sizeof(HostModelDeleteGroupRec)), &theHostModelDeleteGroup);
		}
	}
	return theError;
}   /* CreateCurrentGroup */

/********************************************************************************************************/
/*									ModelGenerate														*/
/********************************************************************************************************/
static SInt32 ModelGenerate (SInt32 theCommandRecSize, void* theCommandRec)
{
	PluginModelGeneratePtr thePluginModelGeneratePtr = (PluginModelGeneratePtr)theCommandRec;
	PluginInfoRec theInfo = thePluginModelGeneratePtr->theInfo;
 	SInt32 theError = 0;
 	
 	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelGenerateRec))) 
		theError = pluginParameterErr;
	else {
		theInfo = thePluginModelGeneratePtr->theInfo;
 		if (thePluginModelGeneratePtr->theChildGroups > 1) 
			theError = CreateCurrentGroup(&theInfo, thePluginModelGeneratePtr->theChildGroups);
	}
	return theError;
}   /* ModelGenerate */

/********************************************************************************************************/
/*									ModelSetup															*/
/********************************************************************************************************/
/*	The current frame time is extracted here and stored for use in a subsequent ModelGenerate call		*/
/********************************************************************************************************/
static SInt32 ModelSetup (SInt32 theCommandRecSize, void* theCommandRec)
{
	PluginModelSetupPtr thePluginModelSetupPtr = (PluginModelSetupPtr)theCommandRec;
	PluginInfoRec theInfo;
	ControlDataPtr theControlDataPtr;
	PluginDataPtr thePluginDataPtr;
 	HostModelResetRec theHostModelReset;
	SInt32 theError = 0;

	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginModelSetupRec))) 
		theError = pluginParameterErr;
	else
	{
		theInfo = thePluginModelSetupPtr->theInfo;
		theControlDataPtr = (ControlDataPtr)theInfo.theControlData;
		thePluginDataPtr = (PluginDataPtr)theInfo.thePluginData;
 		thePluginDataPtr->theCurrentFrameIndex = thePluginModelSetupPtr->theFrameIndex;
 		theHostModelReset.theReference = theInfo.theHostReference;

		theError = theInfo.theHostProc(PLUGCOMMAND(hostModelReset), PARAMETERSIZE(sizeof(HostModelResetRec)), &theHostModelReset);
	}
	return theError;
}   /* ModelSetup */

/********************************************************************************************************/
/*									AllocateMemory														*/
/********************************************************************************************************/
/*	This function allocates memory if the ControlData or PluginData records is of the wrong	size.  The	*/
/*	ControlData record is initialized if the size of this record changes during	the call to allocate 	*/
/*	memory.							.																	*/
/********************************************************************************************************/
static SInt32 AllocateMemory (PluginInfoRec *theInfo) 
{
 	ControlDataPtr theControlDataPtr;
  	SInt32 theOldControlDataSize;
	HostAllocateRec theHostAllocate;
 	SInt32 theError = 0;
 							
 	if ((theInfo->thePluginDataSize != sizeof(PluginDataRec)) || (theInfo->theControlDataSize != sizeof(ControlDataRec))) {
 		theOldControlDataSize = theInfo->theControlDataSize;

		theHostAllocate.theReference = theInfo->theHostReference;

		theHostAllocate.theControlDataSize = sizeof(ControlDataRec);
		theHostAllocate.theControlData = theInfo->theControlData;
		theHostAllocate.thePluginDataSize = sizeof(PluginDataRec);
		theHostAllocate.thePluginData = theInfo->thePluginData;

		theError = theInfo->theHostProc(PLUGCOMMAND(hostAllocate), PARAMETERSIZE(sizeof(HostAllocateRec)), &theHostAllocate);

		if (theError == 0) {
 			theInfo->thePluginData = theHostAllocate.thePluginData;
			theInfo->thePluginDataSize = theHostAllocate.thePluginDataSize;
			theInfo->theControlData = theHostAllocate.theControlData;
			theInfo->theControlDataSize = theHostAllocate.theControlDataSize;
  			
			if (theOldControlDataSize != theHostAllocate.theControlDataSize) {
				theControlDataPtr = (ControlDataPtr)theInfo->theControlData;	
				if (theOldControlDataSize < theHostAllocate.theControlDataSize) 
					theControlDataPtr->theNumOfMorphFrames = 30;
 			}
		}
	}
	return theError;
}   /* AllocateMemory */

/********************************************************************************************************/
/*									Initialize															*/
/********************************************************************************************************/
static SInt32 Initialize (SInt32 theCommandRecSize, void* theCommandRec) 
{
	PluginInitializePtr thePluginInitializePtr = (PluginInitializePtr) theCommandRec;
	SInt32 theError = 0;

	if (PLUGPARAMETERFAIL(theCommandRecSize, sizeof(PluginInitializeRec)))
		theError = pluginParameterErr;
	else
		theError = AllocateMemory(&thePluginInitializePtr->theInfo);
	return theError;
}   /* Initialize */

/* Get information about the plugin */

static SInt32 DoInformation(SInt32 theCommandRecSize, void* theCommandRec)
{
	PluginInformationPtr theParameters;
	SInt32 theResult;

	theResult = pluginNoErr;
	theParameters = (PluginInformationPtr)theCommandRec;
	if (theCommandRecSize < sizeof(PluginInformationRec))
		theResult = pluginParameterErr;
	else {
		theParameters->thePluginType = pluginType;
		theParameters->thePluginVersion = pluginVersion;
		PStrCpy(theParameters->thePluginName, pluginName);
		PStrCpy(theParameters->thePluginCopyright, pluginCopyright);
		PStrCpy(theParameters->thePluginAuthor, pluginAuthor);
		PStrCpy(theParameters->thePluginDescription, pluginDescription);
		theParameters->thePluginPermission = pluginPermission;
	}
	return theResult;
}  /* DoInformation */

/********************************************************************************************************/
/*									main																*/
/********************************************************************************************************/

EXPDLL SInt32 Main (UInt32 theCommand, SInt32 theCommandRecSize, void* theCommandRec)
{	
	SInt32 theError = 0; // pluginCommandErr;

	switch (HOSTCOMMAND(theCommand))
	{
		case pluginInitialize:		theError = Initialize(theCommandRecSize, theCommandRec);	break;
		case pluginInterface:		theError = theInterface(theCommandRecSize, theCommandRec);	break;
		case pluginModelSetup:		theError = ModelSetup(theCommandRecSize, theCommandRec);	break;
		case pluginModelGenerate:	theError = ModelGenerate(theCommandRecSize, theCommandRec);	break;
		case pluginInformation:		theError = DoInformation(theCommandRecSize, theCommandRec); break;
	}
	return theError;
}	/* main */
