/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifdef _WINDOWS
#pragma warning(disable : 4244)
#endif

#include "ColorSpline.h"
#include "EIShader.h"
#include "FPU.h"
#include "Matrix.h"
#include "Noise.h"
#include "PString.h"
#include <stddef.h>
#include "System.h"

#define MIX(a, b, f) ((a) + (f) * ((b) - (a)))

#define shaderType				'CRMP'
#define shaderVersion			0x00000001
#define shaderRevVersion		0x01000000

#define defShaderXOff			0.0
#define defShaderYOff			0.0
#define defShaderZOff			0.0

#define defShaderNoiseLevel		5

#define defShaderNoiseFactor	0.5
#define defShaderNoiseScale		1.5

#define defShaderDensity		1.0

#define defShaderBumpScale		0.5
#define defShaderBumpStrength	0.75

#define defShaderHighColorA		255
#define defShaderHighColorR		255
#define defShaderHighColorG		255
#define defShaderHighColorB		255

#define defShaderLowColorA		255
#define defShaderLowColorR		255
#define defShaderLowColorG		255
#define defShaderLowColorB		255

typedef struct {
	UInt32 shaderOwnerType;
	UInt32 shaderVersionNum;
	SInt32 shaderNoiseLevel;
	float shaderNoiseScale, shaderNoiseFactor;
	float shaderDensity;
	float shaderBumpScale, shaderBumpStrength;
	float shaderXOff, shaderYOff, shaderZOff;
	ARGB shaderHighColor, shaderLowColor;
} ShaderInterfaceRec, *ShaderInterfacePtr;

typedef struct {
	SInt32 shaderNoiseLevel;
	float shaderNoiseScale, shaderNoiseFactor;
	float shaderDensity;
	float shaderBumpScale, shaderBumpStrength;
	float shaderXOff, shaderYOff, shaderZOff;
	ARGB_Real shaderHighColor, shaderLowColor;
} ShaderDataRec, *ShaderDataPtr;

/* Static functions */

static float CalcFractal(EIShaderShadeRec *theShadePtr)
{
	EIShaderMapPtr theMap = theShadePtr->shaderMap;
	ShaderDataPtr theShaderData = (ShaderDataPtr)theShadePtr->shaderData;

	float x = theMap->map_X0 * theShaderData->shaderDensity;
	float y = theMap->map_Y0 * theShaderData->shaderDensity;
	float z = theMap->map_Z0 * theShaderData->shaderDensity;
	float xOff = theShaderData->shaderXOff;
	float yOff = theShaderData->shaderYOff;
	float zOff = theShaderData->shaderZOff;
	float noiseFactor = theShaderData->shaderNoiseFactor;
	float noiseScale = theShaderData->shaderNoiseScale;
	SInt32 detail = theShaderData->shaderNoiseLevel;
	float sum = 0.0;
	
	while (detail--) {
		CellularData f;

		CellularNoise(&f, 0, x + xOff, y + yOff, z + zOff);
		sum += f.distance * noiseScale;
		x += x;
		y += y;
		z += z;
		noiseScale *= noiseFactor;
	}
	
	/* Return the fractal noise value */
	
	return sum;
}  /* CalcFactal */

static float CalcFractalBump(EIShaderShadeRec *theShadePtr)
{
	EIShaderMapPtr theMap = theShadePtr->shaderMap;
	ShaderDataPtr theShaderData = (ShaderDataPtr)theShadePtr->shaderData;

	float x = theMap->map_X0 * theShaderData->shaderDensity;
	float y = theMap->map_Y0 * theShaderData->shaderDensity;
	float z = theMap->map_Z0 * theShaderData->shaderDensity;
	float xOff = theShaderData->shaderXOff;
	float yOff = theShaderData->shaderYOff;
	float zOff = theShaderData->shaderZOff;
	float noiseFactor = theShaderData->shaderNoiseFactor;
	float noiseScale = theShaderData->shaderNoiseScale;
	float bumpStrength = theShaderData->shaderBumpStrength;
	float bumpScale = theShaderData->shaderBumpScale;
	SInt32 detail = theShaderData->shaderNoiseLevel;
	float bumpSum = 1.0;
	float sum = 0.0;
	float aSum = 0.0;
	float bSum = 0.0;
	float cSum = 0.0;
	
	while (detail--) {
		CellularData f;
		float scale;
		
		CellularNoise(&f, 0, x + xOff, y + yOff, z + zOff);
		sum += f.distance * noiseScale;
		scale = 1.0 / bumpSum * SQRT(f.distance);
		aSum += scale * f.vx;
		bSum += scale * f.vy;
		cSum += scale * f.vz;

		x += x;
		y += y;
		z += z;
		noiseScale *= noiseFactor;
		bumpSum += bumpStrength;
	}
	
	/* Return the fractal noise value */
	
	theShadePtr->shaderBumpA = aSum * bumpScale;
	theShadePtr->shaderBumpB = bSum * bumpScale;
	theShadePtr->shaderBumpC = cSum * bumpScale;
	return sum;
}  /* CalcFactalBump */

static void DefaultShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	theShaderInterface->shaderOwnerType = shaderType;
	theShaderInterface->shaderVersionNum = shaderVersion;
	theShaderInterface->shaderNoiseLevel = defShaderNoiseLevel;
	theShaderInterface->shaderNoiseScale = defShaderNoiseScale;
	theShaderInterface->shaderNoiseFactor = defShaderNoiseFactor;
	theShaderInterface->shaderDensity = defShaderDensity;
	theShaderInterface->shaderBumpScale = defShaderBumpScale;
	theShaderInterface->shaderBumpStrength = defShaderBumpStrength;
	theShaderInterface->shaderXOff = defShaderXOff;
	theShaderInterface->shaderYOff = defShaderYOff;
	theShaderInterface->shaderZOff = defShaderZOff;
	theShaderInterface->shaderHighColor.argb.a = defShaderHighColorA;
	theShaderInterface->shaderHighColor.argb.r = defShaderHighColorR;
	theShaderInterface->shaderHighColor.argb.g = defShaderHighColorG;
	theShaderInterface->shaderHighColor.argb.b = defShaderHighColorB;
	theShaderInterface->shaderLowColor.argb.a = defShaderLowColorA;
	theShaderInterface->shaderLowColor.argb.r = defShaderLowColorR;
	theShaderInterface->shaderLowColor.argb.g = defShaderLowColorG;
	theShaderInterface->shaderLowColor.argb.b = defShaderLowColorB;
}  /* DefaultShaderInterface */

static void FixShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	theShaderInterface;
	
	FIXFIELD(theShaderInterface->shaderOwnerType);
	FIXFIELD(theShaderInterface->shaderVersionNum);
	FIXFIELD(theShaderInterface->shaderNoiseLevel);
	FIXFIELD(theShaderInterface->shaderNoiseScale);
	FIXFIELD(theShaderInterface->shaderNoiseFactor);
	FIXFIELD(theShaderInterface->shaderDensity);
	FIXFIELD(theShaderInterface->shaderBumpScale);
	FIXFIELD(theShaderInterface->shaderBumpStrength);
	FIXFIELD(theShaderInterface->shaderXOff);
	FIXFIELD(theShaderInterface->shaderYOff);
	FIXFIELD(theShaderInterface->shaderZOff);
	
}  /* FixShaderInterface */

static void GetShaderInterface(void* theInterface, SInt32 theSize, ShaderInterfacePtr theShaderInterface)
{
	/* Copy the shader interface data */
	
	if (theSize == sizeof(ShaderInterfaceRec))
		*theShaderInterface = *(ShaderInterfacePtr)theInterface;
	else {
		theShaderInterface->shaderOwnerType = 0;
		theShaderInterface->shaderVersionNum = 0;
	}
	
	/* Validate the shader interface data */
	
	FixShaderInterface(theShaderInterface);
	if (theShaderInterface->shaderOwnerType != shaderType ||
		theShaderInterface->shaderVersionNum != shaderVersion)
		DefaultShaderInterface(theShaderInterface);
}  /* GetShaderInterface */

/* Export functions */

SInt32 EIShaderInformation(EIShaderInformationRec *theInformationPtr)
{
	ShaderInterfaceRec theShaderInterface;
		
	/* Get the shader interface parameters */
		
	GetShaderInterface(theInformationPtr->shaderInterface, theInformationPtr->shaderInterfaceSize, &theShaderInterface);
	
	/* Setup the shader feature flags */
	
	BSET(theInformationPtr->shaderFeatureFlags, shaderAntialiasFlag);
	BSET(theInformationPtr->shaderFeatureFlags, shaderColorFlag);
	SETFLAG(theInformationPtr->shaderFeatureFlags, shaderBumpFlag, (theShaderInterface.shaderBumpStrength != 0.0));
	return shaderNoError;
}  /* EIShaderInformation */

SInt32 EIShaderInitialize(EIShaderInitializeRec *theInitializePtr)
{
	ShaderDataPtr theShaderData;
	
	/* Initialize the noise library */
	
	if (!InitCellularNoise((NoiseMallocFunction)theInitializePtr->HostMalloc, NULL))
		return shaderMemoryError;
	
	/* Allocate the shader's private data record */
	
	theShaderData = (ShaderDataPtr)theInitializePtr->HostMalloc(sizeof(ShaderDataRec));
	
	/* Allocate the shader data record */
	
	theShaderData = (ShaderDataPtr)theInitializePtr->HostMalloc(sizeof(ShaderDataRec));
	if (theShaderData) {
		ShaderInterfaceRec theShaderInterface;
		
		/* Get the shader interface parameters */
		
		GetShaderInterface(theInitializePtr->shaderInterface, theInitializePtr->shaderInterfaceSize, &theShaderInterface);
				
		/* Setup the shader's private data */
		
		theShaderData->shaderNoiseLevel = theShaderInterface.shaderNoiseLevel;
		theShaderData->shaderNoiseScale = theShaderInterface.shaderNoiseScale;
		theShaderData->shaderNoiseFactor = theShaderInterface.shaderNoiseFactor;
		theShaderData->shaderDensity = theShaderInterface.shaderDensity;
		theShaderData->shaderBumpScale = -theShaderInterface.shaderBumpScale;
		theShaderData->shaderBumpStrength = 1.0 - theShaderInterface.shaderBumpStrength;
		theShaderData->shaderXOff = theShaderInterface.shaderXOff;
		theShaderData->shaderYOff = theShaderInterface.shaderYOff;
		theShaderData->shaderZOff = theShaderInterface.shaderZOff;
		theShaderData->shaderHighColor.a = theShaderInterface.shaderHighColor.argb.a / 255.0;
		theShaderData->shaderHighColor.r = theShaderInterface.shaderHighColor.argb.r / 255.0;
		theShaderData->shaderHighColor.g = theShaderInterface.shaderHighColor.argb.g / 255.0;
		theShaderData->shaderHighColor.b = theShaderInterface.shaderHighColor.argb.b / 255.0;
		theShaderData->shaderLowColor.a = theShaderInterface.shaderLowColor.argb.a / 255.0;
		theShaderData->shaderLowColor.r = theShaderInterface.shaderLowColor.argb.r / 255.0;
		theShaderData->shaderLowColor.g = theShaderInterface.shaderLowColor.argb.g / 255.0;
		theShaderData->shaderLowColor.b = theShaderInterface.shaderLowColor.argb.b / 255.0;
		
		/* Return the pointer to the shader's private data to the host */
		
		theInitializePtr->shaderData = theShaderData;
		return shaderNoError;
	} else
		return shaderMemoryError;
}  /* EIShaderInitialize */

SInt32 EIShaderFinish(EIShaderFinishRec *theFinishPtr)
{
	/* Get rid of the shader's private data */
	
	theFinishPtr->HostFree(theFinishPtr->shaderData);
	
	/* Finish the noise library */
	
	FinishCellularNoise((NoiseFreeFunction)theFinishPtr->HostFree);
	return shaderNoError;
}  /* EIShaderFinish */

SInt32 EIShaderShade(EIShaderShadeRec *theShadePtr)
{
	ShaderDataPtr theShaderData = (ShaderDataPtr)theShadePtr->shaderData;
	float factor;
	
	if (BTST(theShadePtr->shaderFeatureFlags, shaderBumpFlag))
		factor = CalcFractalBump(theShadePtr);
	else
		factor = CalcFractal(theShadePtr);
	
	if (BTST(theShadePtr->shaderFeatureFlags, shaderColorFlag))
		MixARGBColors(&theShaderData->shaderLowColor, &theShaderData->shaderHighColor, &theShadePtr->shaderColor, factor);
	return shaderNoError;
}  /* EIShaderShade */
