/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifdef _WINDOWS
#pragma warning(disable : 4244)
#endif

#include "System.h"
#include "EIShader.h"
#include "EI_Dialog.h"
#include "EI_Control.h"
#include "EI_GraphControl.h"
#include "EI_GraphData.h"
#include "EI_GradientControl.h"
#include "EI_GradientData.h"

#define  shaderType   'TEST'
#define  shaderVers   0x0001

namespace {

#pragma pack(push, 4)
struct ShIntfRec {
	UInt32  mType, mVers;
	SInt32  mInitializedFlag;
	EI_GraphData mGraphData;
	SInt32  mInterp;
	EI_GradientData mGradientData;
	float  mRange[2];
};
#pragma pack(pop)

bool GetShaderInterface( void * src, SInt32 theSize, ShIntfRec * SI)
{
	SInt32 tstSize = sizeof(ShIntfRec);
	if (theSize == tstSize) 
	 *SI = *(ShIntfRec *) src;
	else SI->mType = 0;

	FIXFIELD(SI->mType);
	FIXFIELD(SI->mVers);
	FIXFIELD(SI->mInitializedFlag);
	FIXFIELD(SI->mInterp);
	FIXFIELD(SI->mRange[0]);
	FIXFIELD(SI->mRange[1]);
	
	if (SI->mType != shaderType || SI->mVers != shaderVers) return false;
	return true;
} 

//--------------- Custom shader's interface support -------------------- 

#define ID_FLAG_NEW	4
#define ID_GRAPH	5
#define ID_INTERP	6
#define ID_GRADIENT	7

typedef struct {
	EI_Dialog * theDialog;
	EI_ResourceFile * theResource;
	EI_ResourceFile theResourceFile;
	EIHostInterfaceSetupProc SetupProc;
} InterfaceSetupRec, *InterfaceSetupPtr;

InterfaceSetupRec theSetupRec;
EI_GraphData * theGD;

inline EI_Control * ID2Ctl ( SInt32 ID )	{ return EI_FindControlByID(theSetupRec.theDialog, ID); }
inline SInt32		CtrlVal( SInt32 ID )	{ return EI_GetControlValue(ID2Ctl(ID)); }

void ControlHitFunc( EI_Control *iControl, EI_DrawContext * )
{
	SInt32 itemNo = EI_GetControlID(iControl);
	switch (itemNo) {
	 case ID_INTERP:
	  if (theGD) {
	   SInt32 val = CtrlVal(ID_INTERP);
	   EI_GraphDataRequest(theGD, EI_GRAPH_DATA_REQUEST_SET_INTERP, 0, &val);
	   EI_InvalControl(ID2Ctl(ID_GRAPH));
	  } 
	  break;
	}	  
}

void SetControlHitFunc( SInt32 itemNo )
{
	EI_SetControlHitFunction(ID2Ctl(itemNo), ControlHitFunc);
}

}	// namespace


//------------------------- Export functions -------------------------

SInt32 EIShaderInformation( EIShaderInformationRec * SI )
{
	BSET(SI->shaderFeatureFlags, shaderColorFlag);
	return shaderNoError;
} 

SInt32 EIShaderInitialize( EIShaderInitializeRec * SI )
{
	SI->shaderData = 0;
	
	ShIntfRec intf;
	if (!GetShaderInterface(SI->shaderInterface, SI->shaderInterfaceSize, &intf)) return shaderNoError;
	SI->shaderData = SI->HostMalloc(sizeof(ShIntfRec));
	if (!SI->shaderData) return shaderMemoryError;
	*(ShIntfRec *) SI->shaderData = intf;
	return shaderNoError;
}

SInt32 EIShaderFinish( EIShaderFinishRec * FI )
{
	if (FI->shaderData) FI->HostFree(FI->shaderData);
	return shaderNoError;
}

SInt32 EIShaderShade( EIShaderShadeRec * SH )
{
	SInt32 err;
	ShIntfRec * intf = (ShIntfRec *) SH->shaderData;
	if (!intf) return shaderNoError;
	if (!intf->mGraphData || !intf->mGradientData) return shaderNoError;

	float d = SH->shaderPixel->shadeDistance;
	const float * R = intf->mRange;
	float minR = MIN(R[0], R[1]); 
	float maxR = MAX(R[0], R[1]); 
	d = MAX(d, minR);
	d = MIN(d, maxR);
	d = (d - minR) / MAX(maxR - minR, 1.0e-4f);
	
	err = EI_GraphDataRequest(intf->mGraphData, EI_GRAPH_DATA_REQUEST_GET_VALUE, 0, &d);
	if (err) return shaderNoError;

	ARGB_Real color;	
	color.a = d;
	err = EI_GradientDataRequest(intf->mGradientData, EI_GRADIENT_DATA_REQUEST_GET_COLOR, 0, &color);
	if (err) return shaderNoError;
	
	SH->shaderColor = color;
	return shaderNoError;
}  

SInt32 EIShaderHostInfo( EIShaderHostInfoRec * )
{
	return shaderNoError;
}

SInt32 EIShaderSerial( EIShaderSerialRec * )
{	
	return shaderNoError;
}

SInt32 EIShaderInterfaceSetup( EIShaderInterfaceSetupRec * theInterfaceSetup )
{
// Save theInterfaceSetup data 
	theSetupRec.theDialog = (EI_Dialog *) theInterfaceSetup->hostDialogReference;
	theSetupRec.theResource = &theSetupRec.theResourceFile;
	theSetupRec.theResourceFile.data = theInterfaceSetup->hostResourceFile;
	theSetupRec.SetupProc = theInterfaceSetup->HostInterfaceSetup;
	
	EI_SetDialogTitle(theSetupRec.theDialog, "Custom Controls Test");
	EI_SetControlEnable(ID2Ctl(ID_FLAG_NEW), 0); 
	SetControlHitFunc(ID_INTERP);

// get GraphData pointer
	theGD = 0;
	SInt32 err = EI_GraphControlRequest(ID2Ctl(ID_GRAPH), EI_GRAPH_CONTROL_REQUEST_GET_GRAPHDATA, 0, &theGD);
	if (err) theGD = 0;

	if (theGD) ControlHitFunc(ID2Ctl(ID_INTERP), 0);
			
	return shaderNoError;
}

