/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifdef _WINDOWS
#pragma warning(disable : 4244)
#endif

#include "EIShader.h"
#include "PString.h"
#include <stddef.h>
#include "System.h"
#include <stdio.h>

#include "EI_Dialog.h"
#include "EI_Control.h"
#include "EI_FileRef.h"
#include "EI_Stream.h"
#include "EI_DiskImage.h"

#define shaderType			'LIST'
#define shaderVersion		0x00000001

typedef struct {
	UInt32 shaderOwnerType;
	UInt32 shaderVersionNum;
} ShaderInterfaceRec, *ShaderInterfacePtr;

/* Static functions */

static void DefaultShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	theShaderInterface->shaderOwnerType = shaderType;
	theShaderInterface->shaderVersionNum = shaderVersion;
}  /* DefaultShaderInterface */

static void FixShaderInterface(ShaderInterfacePtr theShaderInterface)
{
	theShaderInterface;
	
	FIXFIELD(theShaderInterface->shaderOwnerType);
	FIXFIELD(theShaderInterface->shaderVersionNum);
	
}  /* FixShaderInterface */

static void GetShaderInterface(void* theInterface, SInt32 theSize, ShaderInterfacePtr theShaderInterface)
{
	/* Copy the shader interface data */
	
	if (theSize == sizeof(ShaderInterfaceRec))
		*theShaderInterface = *(ShaderInterfacePtr)theInterface;
	else {
		theShaderInterface->shaderOwnerType = 0;
		theShaderInterface->shaderVersionNum = 0;
	}
	
	/* Validate the shader interface data */
	FixShaderInterface(theShaderInterface);
	if (theShaderInterface->shaderOwnerType != shaderType ||
		theShaderInterface->shaderVersionNum != shaderVersion)
		DefaultShaderInterface(theShaderInterface);
} 

/* Export functions */

SInt32 EIShaderInformation(EIShaderInformationRec *theInformationPtr)
{
	(void) theInformationPtr;
	return shaderNoError;
} 

SInt32 EIShaderInitialize(EIShaderInitializeRec *theInitializePtr)
{
	(void) theInitializePtr;
	return shaderNoError;
}

SInt32 EIShaderFinish(EIShaderFinishRec *theFinishPtr)
{
	(void) theFinishPtr;
	return shaderNoError;
}

SInt32 EIShaderShade(EIShaderShadeRec *theShadePtr)
{
	(void) theShadePtr;
	return shaderNoError;
}  


SInt32 EIShaderHostInfo( EIShaderHostInfoRec * theHostInfoPtr )
{
	(void) theHostInfoPtr;
	return shaderNoError;
}

SInt32 EIShaderSerial( EIShaderSerialRec * theSerialPtr )
{	
	(void) theSerialPtr;
	return shaderNoError;
}


/*--------------- Custom shader's interface support -------------------- */

#define FMNU_ID				136

#define ID_OK				1
#define ID_CANCEL			2
#define ID_LIST				4

#define ID_LFLAG_MULTI		5
#define ID_LFLAG_REORDER	6
#define ID_LFLAG_HASICON	7
#define ID_LFLAG_HASENABLE	8
#define ID_LFLAG_HASLOCK	9
#define ID_LFLAG_HASEDIT	10

#define ID_ICN_ENABLE		11
#define ID_ICN_LOCKED		12

#define ID_INFO_TXT_1		13
#define ID_INFO_TXT_2		14

#define ITEMCOUNT(a)		(sizeof(a) / sizeof(a[0]))

#define ICN_ITEM_0			0
#define ICN_ITEM_1			1
#define ICN_ITEM_2			2
#define ICN_ITEM_3			3

#define ICN_LOCKED_0		4
#define ICN_LOCKED_1		5
#define ICN_LOCKED_2		6

#define ICN_ENABLE_0		7
#define ICN_ENABLE_1		8
#define ICN_ENABLE_2		9
		
static const char * theItmName[] = { 
	"One", "Two", "Three", "Four",
	"Five", "Six", "Seven", "Eight",	
	"Nine", "Ten", "Eleven", "Twelve" 
};	

struct Icon {
	SInt32 mID, mRsrc;
	EI_DiskImage * mDiskImg;
	const EI_ImageBuffer * mImgBuf;
};	

Icon theIcons[] = {	
// items icons
	{ 750, 'PICT', 0, 0 },		
	{ 751, 'PICT', 0, 0 },
	{ 752, 'PICT', 0, 0 },
	{ 753, 'PICT', 0, 0 },
	
// lock icons	
	{ 720, 'argb', 0, 0 },
	{ 722, 'argb', 0, 0 },
	{ 723, 'argb', 0, 0 },

// enabled icons	
	{ 721, 'argb', 0, 0 },
	{ 730, 'argb', 0, 0 },
	{ 731, 'argb', 0, 0 },
};

typedef struct {
	EI_Dialog * theDialog;
	EI_ResourceFile * theResource;
	EI_ResourceFile theResourceFile;
	EIHostInterfaceSetupProc SetupProc;
} InterfaceSetupRec, *InterfaceSetupPtr;

static InterfaceSetupRec theSetupRec;

/* Get control by ID */

static EI_Control * ID2Ctl( SInt32 ID )
{
	return EI_FindControlByID(theSetupRec.theDialog, ID);
}

static void SetText( SInt32 itemNo, const char * txt )
{
	EI_SetControlTitle(ID2Ctl(itemNo), txt);
}	

inline SInt32 CtrlVal( SInt32 itemNo )
{
	return EI_GetControlValue(ID2Ctl(itemNo));
}

static void InitIcons( void )
{
	SInt32 i, numIcn = ITEMCOUNT(theIcons);
	for (i = 0; i < numIcn; ++i) {
	 theIcons[i].mDiskImg = EI_MakeDiskImage(theSetupRec.theResource, theIcons[i].mRsrc, theIcons[i].mID);
	 theIcons[i].mImgBuf = EI_GetDiskImageBuffer(theIcons[i].mDiskImg);
	} 
}

static void DestroyIcons( void )
{
	SInt32 i, numIcn = ITEMCOUNT(theIcons);
	for (i = 0; i < numIcn; ++i) {
	 if (theIcons[i].mDiskImg)
	  EI_DestroyDiskImage(theIcons[i].mDiskImg);
	} 
}

static SInt32 GetListFlags( void )
{
	SInt32 flag = 0;
	if (CtrlVal(ID_LFLAG_MULTI))     flag |= cEI_ListMultipleSelect;
	if (CtrlVal(ID_LFLAG_REORDER))   flag |= cEI_ListDragReorder;
	if (CtrlVal(ID_LFLAG_HASICON))   flag |= cEI_ListHasItemIcons;
	if (CtrlVal(ID_LFLAG_HASENABLE)) flag |= cEI_ListHasVisibilityColumn;
	if (CtrlVal(ID_LFLAG_HASLOCK))	 flag |= cEI_ListHasLockedColumn;
	if (CtrlVal(ID_LFLAG_HASEDIT))   flag |= cEI_ListCanInlineEdit;
	return flag;
}

static void ListHit( void )
{
	EI_Control * ctl = ID2Ctl(ID_LIST);

	int itm, part, x, y, size = 64;
	if (!EI_GetListHitInfo(ctl, &itm, &part, &x, &y)) return;
	if (itm < 0) return;

	int ID = EI_GetListItemID(ctl, itm);
	int flag = EI_GetListItemFlags(ctl, itm);

	char buf[256];
	sprintf(buf, "Last hit item %ld, ID = %ld, flags = 0x%lx", itm, ID, flag);
	SetText(ID_INFO_TXT_1, buf);

	EI_GetListString(ctl, itm, buf, &size);
	SetText(ID_INFO_TXT_2, buf);
	
	if (part != cEI_IconClickHit) return;

	int icnID = EI_PopupMenuTrack(theSetupRec.theDialog, FMNU_ID, 0, 0, &x, &y);
	if (icnID > 0) EI_SetListItemIcon(ctl, itm, theIcons[(icnID - 1) % 4].mImgBuf);
}

static void ControlHitFunc( EI_Control *iControl, EI_DrawContext * iContext )
{
	(void) iContext;
	EI_Control * ctl = ID2Ctl(ID_LIST);
	switch (EI_GetControlID(iControl)) {
	 case ID_OK:
	 case ID_CANCEL:
	  DestroyIcons();
	  break;
	 
	 case ID_LIST:
	  ListHit();
	  break;
	  
	 case ID_LFLAG_MULTI:
	 case ID_LFLAG_REORDER:
	 case ID_LFLAG_HASICON:
	 case ID_LFLAG_HASENABLE:
	 case ID_LFLAG_HASLOCK:
	 case ID_LFLAG_HASEDIT:
	  EI_SetListFlags(ctl, GetListFlags());
	  EI_InvalControl(ctl);
	  break;
	  
	 case ID_ICN_ENABLE:
	  EI_SetListIcon(ctl, cEI_ItemEnabled, theIcons[ICN_ENABLE_0 + CtrlVal(ID_ICN_ENABLE)].mImgBuf);
	  EI_InvalControl(ctl);
	  break;

	 case ID_ICN_LOCKED:
	  EI_SetListIcon(ctl, cEI_ItemLocked,  theIcons[ICN_LOCKED_0 + CtrlVal(ID_ICN_LOCKED)].mImgBuf);
	  EI_InvalControl(ctl);
	  break;
	}
}

static void SetControlHitFunc( SInt32 itemNo )
{
	EI_SetControlHitFunction(ID2Ctl(itemNo), ControlHitFunc);
}

static void SetupList( void )
{
	SInt32 i;
	EI_Control * ctl = ID2Ctl(ID_LIST);
	EI_SetListFlags(ctl, GetListFlags());

	EI_SetListIcon(ctl, cEI_ItemEnabled, theIcons[ICN_ENABLE_0 + CtrlVal(ID_ICN_ENABLE)].mImgBuf);
	EI_SetListIcon(ctl, cEI_ItemLocked,  theIcons[ICN_LOCKED_0 + CtrlVal(ID_ICN_LOCKED)].mImgBuf);

	for (i = 0; i < ITEMCOUNT(theItmName); ++i) {
	 SInt32 flag = cEI_ItemEnabled;
	 if (i & 1) flag |= cEI_ItemLocked;
	 EI_InsertListItemEx(ctl, i, theItmName[i], i, flag, 0, theIcons[i % 4].mImgBuf);
	} 

	EI_ScrollList2Item(ctl, 0);
	SetText(ID_INFO_TXT_1, "Last hit item: ");
	SetText(ID_INFO_TXT_2, "");
}

SInt32 EIShaderInterfaceSetup( EIShaderInterfaceSetupRec * theInterfaceSetup )
{
	/* Save theInterfaceSetup data */
	
	theSetupRec.theDialog = (EI_Dialog *) theInterfaceSetup->hostDialogReference;
	theSetupRec.theResource = &theSetupRec.theResourceFile;
	theSetupRec.theResourceFile.data = theInterfaceSetup->hostResourceFile;
	theSetupRec.SetupProc = theInterfaceSetup->HostInterfaceSetup;
	
	/* Init all icons */
	
	InitIcons();
	
	/* Set dialog title */	
	
	EI_SetDialogTitle(theSetupRec.theDialog, "List Tester");

	/* Setup List */
	
	SetupList();
		
	/* Install control hit function */	
	
	SetControlHitFunc(ID_LIST);
	SetControlHitFunc(ID_LFLAG_MULTI);
	SetControlHitFunc(ID_LFLAG_REORDER);
	SetControlHitFunc(ID_LFLAG_HASICON);
	SetControlHitFunc(ID_LFLAG_HASENABLE);
	SetControlHitFunc(ID_LFLAG_HASLOCK);
	SetControlHitFunc(ID_LFLAG_HASEDIT);
	SetControlHitFunc(ID_ICN_ENABLE);
	SetControlHitFunc(ID_ICN_LOCKED);
	SetControlHitFunc(ID_OK);
	SetControlHitFunc(ID_CANCEL);

	return shaderNoError;
}