/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef		EI_GRADIENTCONTROL_H
#define		EI_GRADIENTCONTROL_H

#ifndef		EI_TYPES_H
#include	"EI_Types.h"
#endif

#ifdef		__cplusplus
extern "C" {
#endif

typedef SInt32 EI_GradientControlError;

/* Generic Request Function */

EI_GradientControlError EI_GradientControlRequest( EI_Control * iControl, SInt32 iCommand, SInt32 iParam, void * ioData );

/* Error codes returned by EI_GradientControlRequest */

#define EI_GRADIENT_CONTROL_ERROR_NONE				 0		/* no error */
#define EI_GRADIENT_CONTROL_ERROR_INVALID_CTL		-1		/* the iControl parameter is not an EI Graph control */
#define EI_GRADIENT_CONTROL_ERROR_INVALID_CMD		-2		/* unknown iCommand */
#define EI_GRADIENT_CONTROL_ERROR_INVALID_PARAM		-3		/* invalid iParam */
#define EI_GRADIENT_CONTROL_ERROR_INVALID_DATA		-4		/* invalid ioData */
#define EI_GRADIENT_CONTROL_ERROR_UNKNOWN			-5		/* unknown error  */

/* Commands are passed to EI_GradientControlRequest */

#define EI_GRADIENT_CONTROL_REQUEST_GET_GRADIENTDATA	100 
/* Input:
	iControl - EI gradient control
	iCommand - EI_GRADIENT_CONTROL_REQUEST_GET_GRADIENTDATA
	iParam   - not used, must be zero
	
   Output:
    ioData - returned EI_GradientData if success
*/

#define EI_GRADIENT_CONTROL_REQUEST_CREATE				101
/* Input:
	iControl - (EI_Control *) of "group" item to replace
	iCommand - EI_GRADIENT_CONTROL_REQUEST_CREATE
	iParam   - not used, must be zero
	ioData	 - pointer to output (EI_Control *)
	
   Output:
    (EIControl **) ioData - newly created GradientControl
*/

#define EI_GRADIENT_CONTROL_REQUEST_SET_PTIMAGES		102
/* Input:
	iControl - EI gradient control
	iCommand - EI_GRADIENT_CONTROL_REQUEST_SET_PTIMAGES
	iParam   - not used, must be zero
	ioData	 - pointer to array of long[4] with resID ('cicn' or 'PICT')
	
   Output: none
*/


#define EI_GRADIENT_CONTROL_REQUEST_SET_THICKFRAME		103 
/* Input:
	iControl - EI gradient control
	iCommand - EI_GRADIENT_CONTROL_REQUEST_SET_THICKFRAME
	iParam   - not used, must be zero
    ioData   - pointer to long to set
	
   Output: none
*/


#define EI_GRADIENT_CONTROL_REQUEST_SET_UPLEFT			104
/* Input:
	iControl - EI gradient control
	iCommand - EI_GRADIENT_CONTROL_REQUEST_SET_UPLEFT
	iParam   - not used, must be zero
    ioData   - pointer to long to set (1..3)
	
   Output: none
*/


#define EI_GRADIENT_CONTROL_REQUEST_GET_TRACKPOINT		105 
/* Input:
	iControl - EI gradient control
	iCommand - EI_GRADIENT_CONTROL_REQUEST_GET_TRACKPOINT
	iParam   - not used, must be zero
	ioData   - pointer to long
	
   Output: 
    (long *) ioData - index of track point (or -1 if no track)
*/


#ifdef		__cplusplus
}
#endif

#endif 
