/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef		EI_GRADIENTATA_H
#define		EI_GRADIENTATA_H

#ifndef		EI_TYPES_H
	#include	"EI_Types.h"
#endif

#ifndef		__FACTStuff_h__
	#include	"FactStuff.h"
#endif

#ifdef		__cplusplus
extern "C" {
#endif

typedef SInt32	EI_GradientDataError;
typedef void *	EI_GradientData;

#pragma pack(push, 4)

typedef struct {
	float		x;
	ARGB_Real	color;
	SInt32		flag;
} GradientPointRec, * GradientPointPtr;

#pragma pack(pop)


/* Functions */

EI_GradientDataError EI_GradientDataRequest( EI_GradientData iGradData, SInt32 iCommand, SInt32 iParam, void * ioData );

/* Error codes returned by EI_GradientDataRequest */

#define EI_GRADIENT_DATA_ERROR_NONE				 0		/* no error */
#define EI_GRADIENT_DATA_ERROR_INVALID_DATA		-1		/* the iData parameter is not an EI_GraphData */
#define EI_GRADIENT_DATA_ERROR_INVALID_CMD		-2		/* unknown iCommand */
#define EI_GRADIENT_DATA_ERROR_INVALID_PARAM		-3		/* invalid iParam */
#define EI_GRADIENT_DATA_ERROR_INVALID_IODATA	-4		/* invalid ioData */
#define EI_GRADIENT_DATA_ERROR_UNKNOWN			-5		/* unknown error */

/* Commands are passed to EI_GraphControlRequest */

#define EI_GRADIENT_DATA_REQUEST_CREATE				100
/* Input:
	iGradData  - not used, must be zro
	iCommand   - EI_GRADIENT_DATA_REQUEST_CREATE
	iParam     - not used, must be zero
    ioData	   - pointer to EI_GradData to fill
	
   Output:
    (EI_GradientData *) ioData - newly created EI gradient data if success
*/

#define EI_GRADIENT_DATA_REQUEST_DESTROY			101
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_DESTROY
	iParam     - not used, must be zero
    ioData	   - not used, must be zero
	
   Output: none
*/

#define EI_GRADIENT_DATA_REQUEST_GET_COLOR			200
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_GET_COLOR
	iParam     - not used, must be zero
    ioData	   - pointer to "x" argument (float)
	
   Output:
    (ARGB_Real *) ioData - filled by ARGB_Real value if success
*/


#define EI_GRADIENT_DATA_REQUEST_SET_SMOOTH			201
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_SET_SMOOTH
	iParam     - not used, must be zero
    ioData	   - pointer to float smooth value to set
	
   Output: none
*/


#define EI_GRADIENT_DATA_REQUEST_SET_AMPLIFY		202
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_SET_SMOOTH
	iParam     - not used, must be zero
    ioData	   - pointer to float amplify value to set
	
   Output: none
*/


#define EI_GRADIENT_DATA_REQUEST_RESET				203
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_RESET
	iParam     - not used, must be zero
    ioData	   - not used, must be zero
	
   Output: none
*/


#define EI_GRADIENT_DATA_REQUEST_GET_DATASIZE		204
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_GET_DATASIZE
	iParam     - not used, must be zero
	
   Output:
    (SInt32 *) ioData - size of data in bytes
*/


#define EI_GRADIENT_DATA_REQUEST_EXPORT				205
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_EXPORT
	iParam     - size of data, can be obtained via EI_GRADIENT_DATA_REQUEST_GET_DATASIZE
	ioData 	   - pointer to memory block to fill,
			     should have a size not less than 
			     returned by EI_GRADIENT_DATA_REQUEST_GET_DATASIZE
	
   Output:
    ioData 	   - filled buffer if success
*/


#define EI_GRADIENT_DATA_REQUEST_IMPORT				206
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_IMPORT
	iParam     - size of data
	ioData 	   - pointer to block previously 
				 filled via EI_GRAPH_DATA_REQUEST_EXPORT	
	
   Output: none
*/


#define EI_GRADIENT_DATA_REQUEST_GET_NUMPOINT		207 
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_CONTROL_REQUEST_GET_NUMPOINT
	iParam     - not used, must be zero
    ioData     - pointer to SInt32 to receive count of points
	
   Output:
    (SInt32 *) ioData - number of points if success
*/


#define EI_GRADIENT_DATA_REQUEST_GET_POINT			208
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_CONTROL_REQUEST_GET_POINT
	iParam     - index of requested point
    ioData     - pointer to GraphPointRec to fill
	
   Output: 
    (GraphPointRec *) ioData - filled GraphPointRec if success
*/

#define EI_GRADIENT_DATA_REQUEST_SET_POINT			209
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_CONTROL_REQUEST_SET_POINT
	iParam     - index of point to set
    ioData     - pointer to filled GradientPointRec to set
	
   Output: none
*/


#define EI_GRADIENT_DATA_REQUEST_ADD_POINT			210
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_CONTROL_REQUEST_ADD_POINT
	iParam     - insert index
    ioData     - pointer to filled GradientPointRec to add
	
   Output: none
*/

#define EI_GRADIENT_DATA_REQUEST_SET_ALL_POINTS		211
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_DATA_REQUEST_SET_ALL_POINTS
	iParam     - count of points to set
    ioData     - pointer to array of GradientPointRec to set
	
   Output: none
*/

#define EI_GRADIENT_DATA_REQUEST_DEL_POINT			212
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_CONTROL_REQUEST_DEL_POINT
	iParam     - index of point to delete
    ioData     - not used, must be zero
	
   Output: none
*/


#define EI_GRADIENT_DATA_REQUEST_GET_PTLIMIT		213
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_CONTROL_REQUEST_GET_PTLIMIT
	iParam     - not used, must be zero
    ioData     - not used, must be zero
	
   Output: 
    (SInt32 *) ioData - points limit if success
*/

#define EI_GRADIENT_DATA_REQUEST_SET_PTLIMIT		214
/* Input:
	iGradData  - EI gradient data
	iCommand   - EI_GRADIENT_CONTROL_REQUEST_SET_PTLIMIT
	iParam     - not used, must be zero
    ioData     - pointer to new limit to set
	
   Output: none
*/


#ifdef		__cplusplus
}
#endif

#endif	// EI_GRADIENTATA_H
