/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef		EI_GRAPHCONTROL_H
#define		EI_GRAPHCONTROL_H

#ifndef		EI_TYPES_H
#include	"EI_Types.h"
#endif

#ifdef		__cplusplus
extern "C" {
#endif

typedef SInt32 EI_GraphControlError;

/* Sizes of standard Graph Control */

#define  DEFAULT_GRAPH_SIZE_X	 233
#define  DEFAULT_GRAPH_SIZE_Y	 132

/* Generic Request Function */

EI_GraphControlError EI_GraphControlRequest( EI_Control * iControl, SInt32 iCommand, SInt32 iParam, void * ioData );

/* Error codes returned by EI_GraphControlRequest */

#define EI_GRAPH_CONTROL_ERROR_NONE				 0		/* no error */
#define EI_GRAPH_CONTROL_ERROR_INVALID_CTL		-1		/* the iControl parameter is not an EI Graph control */
#define EI_GRAPH_CONTROL_ERROR_INVALID_CMD		-2		/* unknown iCommand */
#define EI_GRAPH_CONTROL_ERROR_INVALID_PARAM	-3		/* invalid iParam */
#define EI_GRAPH_CONTROL_ERROR_INVALID_DATA		-4		/* invalid ioData */
#define EI_GRAPH_CONTROL_ERROR_UNKNOWN			-5		/* unknown error  */

/* CallBacks */

typedef void (*EI_Graph_BackgroundProc)( const EI_Control * iControl, EI_DrawContext * iContext, const EI_Rect * iRect );

/* Commands are passed to EI_GraphControlRequest */

#define EI_GRAPH_CONTROL_REQUEST_GET_GRAPHDATA	100 
/* Input:
	iControl - EI graph control
	iCommand - EI_GRAPH_CONTROL_REQUEST_GET_GRAPHDATA
	iParam   - not used, must be zero
	
   Output:
    ioData - returned EI_GraphData if success
*/

#define EI_GRAPH_CONTROL_REQUEST_CREATE			101
/* Input:
	iControl - (EI_Control *) of "group" item to replace
	iCommand - EI_GRAPH_CONTROL_REQUEST_CREATE
	iParam   - not used, must be zero
	ioData	 - pointer to output (EI_Control *)
	
   Output:
    (EIControl **) ioData - newly created GraphControl
*/

#define EI_GRAPH_CONTROL_REQUEST_SET_WORKRECT	102
/* Input:
	iControl - EI graph control
	iCommand - EI_GRAPH_CONTROL_REQUEST_SET_WORKRECT
	iParam   - not used, must be zero
    ioData   - pointer to EI_Rect to set
	
   Output: none
*/


#define EI_GRAPH_CONTROL_REQUEST_GET_TRACKPOINT	103 
/* Input:
	iControl - EI graph control
	iCommand - EI_GRAPH_CONTROL_REQUEST_GET_TRACKPOINT
	iParam   - not used, must be zero
	ioData   - pointer to long
	
   Output: 
    (long *) ioData - index of track point (or -1 if no track)
*/


#define EI_GRAPH_CONTROL_REQUEST_SET_BKIMAGE	104
/* Input:
	iControl - EI graph control
	iCommand - EI_GRAPH_CONTROL_REQUEST_SET_BKIMAGE
	iParam   - image resource ID ('PICT' or 'cicn')
	
   Output: none
*/


#define EI_GRAPH_CONTROL_REQUEST_SET_BKDRAW		105
/* Input:
	iControl - EI graph control
	iCommand - EI_GRAPH_CONTROL_REQUEST_SET_BKDRAW
	iParam   - not used, must be zero
	ioData	 - EI_Graph_BackgroundProc
	
   Output: none
*/

#define EI_GRAPH_CONTROL_REQUEST_SET_LINECOLOR	106
/* Input:
	iControl - EI graph control
	iCommand - EI_GRAPH_CONTROL_REQUEST_SET_LINECOLOR
	iParam   - not used, must be zero
	ioData	 - pointer to EI_Color to draw graph lines
	
   Output: none
*/


#ifdef		__cplusplus
}
#endif

#endif 
