/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef		EI_GRAPHDATA_H
#define		EI_GRAPHDATA_H

#ifndef		EI_TYPES_H
#include	"EI_Types.h"
#endif

#ifdef		__cplusplus
extern "C" {
#endif

typedef SInt32	EI_GraphDataError;
typedef void *	EI_GraphData;

#pragma pack(push, 4)

typedef struct {
	float	x, y;
	SInt32	flag;
} GraphPointRec, * GraphPointPtr;

#pragma pack(pop)

/* Functions */

EI_GraphDataError EI_GraphDataRequest( EI_GraphData iGraphData, SInt32 iCommand, SInt32 iParam, void * ioData );

/* Error codes returned by EI_GraphDataRequest */

#define EI_GRAPH_DATA_ERROR_NONE				 0		/* no error */
#define EI_GRAPH_DATA_ERROR_INVALID_DATA			-1		/* the iData parameter is not an EI_GraphData */
#define EI_GRAPH_DATA_ERROR_INVALID_CMD			-2		/* unknown iCommand */
#define EI_GRAPH_DATA_ERROR_INVALID_PARAM		-3		/* invalid iParam */
#define EI_GRAPH_DATA_ERROR_INVALID_IODATA		-4		/* invalid ioData */
#define EI_GRAPH_DATA_ERROR_UNKNOWN				-5		/* unknown error */

/* Commands are passed to EI_GraphControlRequest */

#define EI_GRAPH_DATA_REQUEST_CREATE			100
/* Input:
	iGraphData - not used, must be zro
	iCommand   - EI_GRAPH_DATA_REQUEST_CREATE
	iParam     - not used, must be zero
    ioData	   - pointer to EI_GraphData to fill
	
   Output:
    (EI_GraphData *) ioData - newly created EI graph data if success
*/

#define EI_GRAPH_DATA_REQUEST_DESTROY			101
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_DESTROY
	iParam     - not used, must be zero
    ioData	   - not used, must be zero
	
   Output: none
*/

#define EI_GRAPH_DATA_REQUEST_GET_VALUE			200
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_GET_VALUE
	iParam     - not used, must be zero
    ioData	   - pointer to "x" argument (float)
	
   Output:
    (float *) ioData - filled with float "y" value if success
*/


#define EI_GRAPH_DATA_REQUEST_RESET				201
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_RESET
	iParam     - not used, must be zero
    ioData	   - not used, must be zero
	
   Output: none
*/

#define EI_GRAPH_DATA_REQUEST_GET_INTERP		202
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_GET_INTERP
	iParam     - not used, must be zero
    ioData	   - pointer to SInt32 to fill
	
   Output:
    (SInt32 *) ioData - filled with interpolation method (SInt32)
*/


#define EI_GRAPH_DATA_REQUEST_SET_INTERP		203
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_SET_INTERP
	iParam     - not used, must be zero
    ioData	   - pointer to SInt32 to set
	
   Output: none
*/


#define EI_GRAPH_DATA_REQUEST_GET_DATASIZE		204
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_GET_DATASIZE
	iParam     - not used, must be zero
	
   Output:
    (SInt32 *) ioData - size of data in bytes
*/


#define EI_GRAPH_DATA_REQUEST_EXPORT			205
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_EXPORT
	iParam     - size of data, can be obtained via EI_GRAPH_DATA_REQUEST_GET_DATASIZE
	ioData 	   - pointer to buffer to fill
	
   Output:
    ioData 	   - filled buffer if success
*/


#define EI_GRAPH_DATA_REQUEST_IMPORT			206
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_IMPORT
	iParam     - size of data
	iGraphData - pointer to block previously 
				 filled via EI_GRAPH_DATA_REQUEST_EXPORT	
	
   Output:	none	
*/


#define EI_GRAPH_DATA_REQUEST_GET_NUMPOINT		207
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_CONTROL_REQUEST_GET_NUMPOINT
	iParam     - not used, must be zero
    ioData     - pointer to SInt32 to receive count of points
	
   Output:
    (SInt32 *) ioData - number of points if success
*/


#define EI_GRAPH_DATA_REQUEST_GET_POINT			208
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_CONTROL_REQUEST_GET_POINT
	iParam     - index of requested point
    ioData     - pointer to GraphPointRec to fill
	
   Output: 
    (GraphPointRec *) ioData - filled GraphPointRec if success
*/

#define EI_GRAPH_DATA_REQUEST_SET_POINT			209
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_CONTROL_REQUEST_SET_POINT
	iParam     - index of point to set
    ioData     - pointer to filled GraphPointRec to set
	
   Output: none
*/


#define EI_GRAPH_DATA_REQUEST_ADD_POINT			210
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_CONTROL_REQUEST_ADD_POINT
	iParam     - insert index
    ioData     - pointer to GraphPointRec to add
	
   Output: none
*/


#define EI_GRAPH_DATA_REQUEST_SET_ALL_POINTS	211
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_DATA_REQUEST_SET_ALL_POINTS
	iParam     - count of points to set
    ioData     - pointer to array of GraphPointRec to set
	
   Output: none
*/


#define EI_GRAPH_DATA_REQUEST_DEL_POINT			212
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_CONTROL_REQUEST_DEL_POINT
	iParam     - index of point to delete
    ioData     - not used, must be zero
	
   Output: none
*/


#define EI_GRAPH_DATA_REQUEST_GET_PTLIMIT		213
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_CONTROL_REQUEST_GET_PTLIMIT
	iParam     - not used, must be zero
    ioData     - not used, must be zero
	
   Output: 
    (SInt32 *) ioData - points limit if success
*/

#define EI_GRAPH_DATA_REQUEST_SET_PTLIMIT		214
/* Input:
	iGraphData - EI graph data
	iCommand   - EI_GRAPH_CONTROL_REQUEST_SET_PTLIMIT
	iParam     - not used, must be zero
    ioData     - pointer to new limit to set
	
   Output: none
*/


#ifdef		__cplusplus
}
#endif

#endif
