/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __EIShader_h__
#define __EIShader_h__

#ifndef __FACTStuff_h__
#include "FACTStuff.h"
#endif

#ifndef __PString_h__
#include "PString.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* Shader API version number	*/
/* Version 3 in EI8				*/

#define EIShaderAPIVersion			3

/* Shader file type and creator */

#define shaderFileType				QUADBYTE('E', 'I', 'P', 'S')
#define shaderFileCreator			QUADBYTE('E', 'I', 'P', 'S')

/* Shader template constants */

#define shaderTemplateID			128
#define shaderTemplateType			QUADBYTE('P', 'T', 'K', 'C')

/* Shader dialog constants */

#define shaderDialogID				21000
#define shaderOKItem				1
#define shaderCancelItem			2
#define shaderPreviewItem			3

/* Shader control item flags */

#define shaderUnlimitedFlag			31
#define shaderAnimateFlag			30

/* Shader control item data type */

#define shaderSignedFloatingPoint	QUADBYTE('R', 'E', 'A', 'L')
#define shaderUnsignedFloatingPoint	QUADBYTE('U', 'R', 'E', 'L')
#define shaderSignedInteger			QUADBYTE('S', 'I', 'N', 'T')
#define shaderUnsignedInteger		QUADBYTE('U', 'I', 'N', 'T')
#define shaderBoolean				QUADBYTE('B', 'O', 'O', 'L')
#define shaderPopup					QUADBYTE('P', 'O', 'P', 'U')
#define shaderColorRGB				QUADBYTE('C', 'R', 'G', 'B')
#define shaderColorARGB				QUADBYTE('A', 'R', 'G', 'B')

/* shaderAccessFlags -- Shader attribute value access flags */
/* The shader must also examine these flags before modifing each value because they may have */
/* been disabled by the host */

#define accessReferenceFlag					0
#define accessAmbientFlag					1
#define accessSpecularFlag					2
#define accessTransparencyFlag				3
#define accessReflectionFlag				4
#define accessLuminanceFlag					5
#define accessDiffuseFlag					6
#define accessTransmissionFlag				7
#define accessEdgeTransparencyFlag			8
#define accessEdgeTransparencyValueFlag		9
#define accessEdgeDiffuseValueFlag			10
#define accessEdgeSpecularValueFlag			11
#define accessEdgeReflectionValueFlag		12
#define accessEdgeAmbientValueFlag			13
#define accessEdgeLuminanceValueFlag		14
#define accessEdgeTransmissionValueFlag		15
#define accessHighlightFlag					16
#define accessRefractionFlag				17
#define accessTerminatorFlag				18
#define accessGlowFlag						19
#define accessBloomFlag						20
#define accessAlphaFlag						21
#define accessOpacityFlag					22
#define accessGlossFlag						23
#define accessShadeFlag						24
#define accessEdgeTransparencyDropoffFlag	25
#define accessEdgeDiffuseDropoffFlag		26
#define accessEdgeSpecularDropoffFlag		27
#define accessEdgeReflectionDropoffFlag		28
#define accessEdgeAmbientDropoffFlag		29
#define accessEdgeLuminanceDropoffFlag		30
#define accessEdgeTransmissionDropoffFlag	31

/* shaderFeatureFlags -- mode flags -- if none are set, the shader generates a surface material */

#define shaderIlluminationFlag		0	/* lightsource shading is performed by the shader */
#define shaderColorFlag				1	/* shader is designed to calculate a specific channel color */

/* shaderFeatureFlags -- bits 4..7 is the shade mode -- see "Shader Shade modes"  */

#define shaderModeFlag0				4
#define shaderModeFlag1				5
#define shaderModeFlag2				6
#define shaderModeFlag3				7

/* shaderFeatureFlags -- write flags */
/* The shader sets these flags to indicate which values are written by the shader */
/* The shader must also examine these flags before modifing each value because they may have */
/* been disabled by the host */

#define shaderClipFlag				12	/* shader writes shaderClip */
#define shaderTransparencyFlag		13	/* shader writes shaderTransparency */
#define shaderDisplacementFlag		14	/* shader writes shaderDisplacement */
#define shaderReflectionFlag		15	/* shader writes shaderReflection */
#define shaderBumpFlag				16	/* shader writes shaderBump(ABC) */

/* Version 2 */

#define shaderBRDFFlag				17	/* shader writes BRDF pixel values */
#define shaderRayTraceFlag			18	/* shader casts rays */
#define shaderHighlightFlag			19	/* shader writes shaderHighlight */
#define shaderRefractionFlag		20	/* shader writes shaderRefraction */

/* Version EI 7.0 */

#define shaderFillLayersFlag		21	/* shader receives EIShaderFillLayers call */
#define shaderSlabLayersFlag		22	/* shader receives EIShaderSlabLayers call */
#define shaderProcLayersFlag		23	/* shader receives EIShaderProcLayers call */

/* shaderFeatureFlags -- read flags */
/* The shader sets these flags to indicate which values are read by the shader */

#define shaderPositionVectorsFlag	28	/* shader reads shadeD(XYZ)D(UV) */
#define shaderAntialiasPositionFlag	29	/* shader reads shade(XYZ)(12) */
#define shaderAntialiasNormalFlag	30	/* shader reads shadeNormal(ABC)(12), shaderNormal(XYZ)(12) */
#define shaderAntialiasFlag			31	/* shader reads map_(XYZ)(12), map_D(XYZ) */

/* Shader shade modes (API version 3, bits 4..7 of shaderFeatureFlags) */
/* The shader can read these flags to examinate for what it's called */

#define shaderShadeNormal			0	/* the screen point is shaded */
#define shaderShadeZBuffer			1	/* the Z-Buffer point is shaded */
#define shaderShadeGlow				2	/* the glow point is shaded */
#define shaderShadeReflectionRT		3	/* the RT reflection point is shaded */
#define shaderShadeRefractionRT		4	/* the RT refraction point is shaded */
#define shaderShadeShadowRT			5	/* the RT shadow point is shaded */
#define shaderShadeHostCastRay		6	/* the point hit by HostCastRay is shaded */
#define shaderShadeBufferGI			7	/* the GI buffer point is shaded */
#define shaderShadeRayGI			8	/* the GI reverse illumination point is shaded */
#define shaderShadeSky				9	/* the sky (background/foreground) point is shaded */
#define shaderShadeReflectionSky	10	/* the sky reflection (background/foreground) point is shaded */
#define shaderShadeReflectionSkyRT	11	/* the RT reflection sky point is shaded */
#define shaderShadeRefractionSkyRT	12	/* the RT refraction sky point is shaded */
#define shaderShadeUnknown			13	/* the shaded point is unknown */

/* Shader error codes */

#define shaderNoError				0	/* Everything was fine */
#define shaderMemoryError			1	/* Could not allocate enough memory */

/*	Host error codes */

#define hostNoError					0	/*	Everything was fine */
#define hostDialogError 	 	 -100	/*	Invalid dialog reference */
#define hostCommandError 	 	 -101	/*	Invalid command */
#define hostItemError 			 -102	/*	Invalid item number */
#define hostDataError			 -103	/*	Invalid data */

/* Host structure passed via hostSetShaderItem to control with "Memory Block" type  */

typedef struct {
	UInt32	blockSize;
	void *	blockData;
	SInt32	blockSign;					
} EI_MemoryBlock, EI_MemoryBlockPtr;

/* Host calls */

#define hostSetShaderItem		QUADBYTE('S', 'E', 'T', 'I')		/* Setup shader interface item */
#define hostMemBlockSign 		QUADBYTE('M', 'E', 'M', 'O')		/* Signature of passed memory block  */

/* Light flags */

#define lightDropoffFlag			0		/* 1 if the light intensity drops off over distance */
#define lightSpotFlag				1		/* 1 if the light is a spotlight */
#define lightGlowFlag				2		/* 1 if the light has a glow radius */
#define lightFogFlag				3		/* 1 if the light has a fog radius */
#define lightShadowFlag				4		/* 1 if the light casts a shadow */
#define lightTubeFlag				5		/* 1 if the light is a tube */
#define lightFlareFlag				6		/* 1 if the light has a flare plugin */
#define lightFadeObjectFlag			7		/* 1 if the light's flare is faded when it is behind an object */
#define lightProjectorFlag			8		/* 1 if the light has a projector map */
#define lightFadeOffscreenFlag		9		/* 1 if the light's flare is faded when it is out of frame */
#define lightFadeFogFlag			10		/* 1 if the light's flare is faded when it is in fog */
#define lightFadeDistanceFlag		11		/* 1 if the light's flare fades over distance */
#define lightFadeIntensityFlag		12		/* 1 if the light's flare is faded by the light intensity */
#define lightFlareBackgroundFlag	13		/* 1 if the light's flare is behind the objects in the scene */
#define lightCameraProjectorFlag	14		/* 1 if the light is being used to generate a camera projecton map */
#define lightFadeSpotFlag			15		/* 1 if the light's flare is faded when it is hidden by the spot cone */
#define lightRTShadowFlag			16		/* 1 if the light has ray traced shadows */
#define lightParallelFlag			17		/* 1 if the light's rays are parallel */
#define lightDropoffShadowFlag		18		/* 1 if the light's ray traced shadows have a dropoff distance */
#define lightSoftShadowFlag			19		/* 1 if the light's ray traced shadows have soft edges */
#define lightInverseBufferFlag		20		/* 1 if the light's rays are inverted */
#define lightFogVolumeFlag			21		/* 1 if the light's fog has a volume dropoff */
#define lightGlowVolumeFlag			22		/* 1 if the light's glow has a volume dropoff */
#define lightFogSmokeFlag			23		/* 1 if the light's fog is smokey */
#define lightGlowSmokeFlag			24		/* 1 if the light's glow is smokey */
#define lightFogBufferFlag			25		/* 1 if the light's buffer is to be used to shadow the fog */
#define lightGlowBufferFlag			26		/* 1 if the light's buffer is to be used to shadow the glow */
#define lightFogRGBFlag				27		/* 1 if the light's fog color is interpolated with RGB */
#define lightGlowRGBFlag			28		/* 1 if the light's glow color is interpolated with RGB */
#define lightIlluminationFlag		29		/* 1 if the light does not cast surface illumination */
#define lightSpecularFlag			30		/* 1 if the light does not cast specular highlights */
#define lightGlowAddFlag			31		/* 1 if the light's glow is added to other glows rather than maxed */

/* Light flags 2 */

#define lightUseSamplingGIFlag		0		/* 1 if the light uses GI sampling engine */
#define lightAdaptiveSamplingFlag	1		/* 1 if the light texture uses adaptive sampling */
#define lightHasPortalFlag			2		/* 1 if the light has portal objects */
#define lightAreaFlag				3		/* 1 if the light has area */
#define lightAreaSkyTextureFlag		4		/* 1 if the area light textures should be sampled as environment */
#define lightAreaInvertNormalFlag	5		/* 1 if the area portal objects should be used with inversed normal */
#define lightAreaHiddenFlag			6		/* 1 if the area portal objects should be hidden */

#ifdef __MACINTOSH__
#pragma options align = native
#endif

#ifdef __WIN32__
#define EISHADER_FUNCTION __declspec( dllexport ) 
#else
#define EISHADER_FUNCTION
#endif

/* Shader channel item record */

typedef struct {
	PStr31 channelName;
	UInt32 channelFlags;
	IFFType channelType;
	union {
		struct {
			float defaultValue;
			float minimumValue;
			float maximumValue;
		} REAL;
		struct {
			float defaultValue;
			float minimumValue;
			float maximumValue;
		} UREL;
		struct {
			SInt32 defaultValue;
			SInt32 minimumValue;
			SInt32 maximumValue;
		} SINT;
		struct {
			UInt32 defaultValue;
			UInt32 minimumValue;
			UInt32 maximumValue;
		} UINT;
		struct {
			UInt32 defaultValue;
		} BOOL;
		struct {
			UInt32 defaultValue;
			short menuResourceID;
		} POPU;
		struct {
			short defaultRed;
			short defaultGreen;
			short defaultBlue;
		} CRGB;
		struct {
			short defaultRed;
			short defaultGreen;
			short defaultBlue;
			short defaultAlpha;
		} ARGB;
	} value;
} EIShaderChannel;

/* Shader control parameter template record */

typedef struct {
	IFFType shaderType;
	UInt32	shaderVersion;
	UInt32	channelCount;
	EIShaderChannel channel[1];
} EIShaderTemplate, *EIShaderTemplatePtr;

/* Shading attributes record */

typedef struct {
	RGB_Real reference;					/* surface color */
	RGB_Real ambient;					/* ambient reflection color */
	RGB_Real specular;					/* specular reflection color */
	RGB_Real transparency;				/* transparency components */
	RGB_Real reflection;				/* reflection color factor for material */
	RGB_Real edgeTransparency;			/* edge reflection color (for backward compatibility) */
	RGB_Real luminance;					/* self illumination components */
	RGB_Real diffuse;					/* diffuse reflection color */
	RGB_Real transmission;				/* The diffuse transmission color */
	RGB_Real glow;						/* The glow color */
	float alpha;						/* surface alpha mask (1 = solid, 0 = invisible) */
	float opacity;						/* opacity transparency (1 = solid, 0 = transparent) */
	float edgeOpacity;					/* edge opacity (for backward compatibility) */
	float gloss;						/* gloss factor (1 = no gloss, 0 = 100% gloss) */
	float shade;						/* shade factor (1 = no shading, 0 = 100% shading) */
	float bloom;						/* bloom factor (1 = full bloom, 0 = no bloom) */
	float spread;			   			/* specular highlight spread */
	float refraction;					/* transparency refraction index */
	float terminator;					/* edge terminator factor */
	float highlight;					/* highlight dropoff factor */
	float edgeTransparencyValue;		/* the edge transparency value */
	float edgeDiffuseValue;				/* The edge diffuse value */
	float edgeSpecularValue;				/* The edge specular value */
	float edgeReflectionValue;			/* The edge reflection value */
	float edgeAmbientValue;				/* The edge ambient value */
	float edgeLuminanceValue;			/* The edge luminance value */
	float edgeTransmissionValue;			/* The edge transmission value */
	float edgeTransparencyDropoff;		/* the degree to which the transparency changes from the center to the edge (0.0=none) */
	float edgeDiffuseDropoff;			/* the degree to which the diffuse changes from the center to the edge (0.0=none) */
	float edgeSpecularDropoff;			/* the degree to which the specular changes from the center to the edge (0.0=none) */
	float edgeReflectionDropoff;			/* the degree to which the reflection changes from the center to the edge (0.0=none) */
	float edgeAmbientDropoff;			/* the degree to which the ambient changes from the center to the edge (0.0=none) */
	float edgeLuminanceDropoff;			/* the degree to which the luminance changes from the center to the edge (0.0=none) */
	float edgeTransmissionDropoff;		/* the degree to which the transmission changes from the center to the edge (0.0=none) */
	UInt32 shadeFlags;					/* see "GATR Group Shading Flags" in FACTStuff.h */
	UInt32 shadeFlags2;					/* see "GATR Group Shading Flags2" in FACTStuff.h */
	float dissolve;						/* the dissolve factor (1 = solid, 0 = invisible */
	float reflectionBlur;				/* the reflection blur factor (0.0 = no blur) */
	float refractionBlur;				/* the refraction blur factor (0.0 = no blur) */
	
// added October 2003
	UInt32  reflectionBlurType;
	UInt32  reflectionBlurSamples;
	float	reflectionBlurAngle;
	
	SInt32  refractionBlurType;
	SInt32  refractionBlurSamples;
	float	refractionBlurAngle;
	
// added February 2004	
	float refrCausticsAmount;
	float refrCausticsFactor;
	float refrCausticsDropoff;
	float refrCausticsReceive;
	
	float reflCausticsAmount;
	RGB_Real reflCausticsColor;
	float reflCausticsFactor;
	float reflCausticsDropoff;
	float reflCausticsReceive;
	
// added May 2005
	float translucentAmount;
	float translucentThickness;
	RGB_Real translucentColor;
	float translucentScatter;
	float translucentFocus;
	RGB_Real translucentLnColor;
	
	float transparentThickness;
} EIShaderAttributesRec, *EIShaderAttributesPtr;

/* Shading pixel variables */

typedef struct _EIShaderPixelRec {
	double shadeX, shadeY, shadeZ;						/* P - the surface point at the current shading pixel */
	double shadeX1, shadeY1, shadeZ1;					/* P + du */
	double shadeX2, shadeY2, shadeZ2;					/* P + dv */
	double shadeNormalX, shadeNormalY, shadeNormalZ;		/* P' - the surface point at the current sub-pixel */
	double shadeNormalX1, shadeNormalY1, shadeNormalZ1;	/* P' + du */
	double shadeNormalX2, shadeNormalY2, shadeNormalZ2;	/* P' + dv */
	double shadeDXDU, shadeDYDU, shadeDZDU;				/* dPdu */
	double shadeDXDV, shadeDYDV, shadeDZDV;				/* dPdv */
	float shadeDistance;									/* |P| */
	float shadeXN, shadeYN, shadeZN;						/* P normalized */
	float shadeNormalA, shadeNormalB, shadeNormalC;		/* N at P' - the surface normal */
	float shadeNormalA0, shadeNormalB0, shadeNormalC0;	/* N at P' before any bumps have been applied */
	float shadeNormalA1, shadeNormalB1, shadeNormalC1;	/* N at P' + du */
	float shadeNormalA2, shadeNormalB2, shadeNormalC2;	/* N at P' + dv */
	float shadePlaneA, shadePlaneB, shadePlaneC;			/* The facet's constant normal */
	float shadeMinAmbientR, shadeMinAmbientG, shadeMinAmbientB;	/* The minimum ambient level */
	float shadeAmbientR, shadeAmbientG, shadeAmbientB;	/* The ambient illumination */
	ARGB_Real shadeColor;								/* The shaded pixel color and alpha */
	RGB_Real shadeTransparency;							/* The transparency color */
	RGB_Real shadeReflection;							/* The reflected ray color */
	float shadeReflectionBlur;							/* The reflection blur factor */
	float shadeClip;										/* The clip transparency (1 = solid, 0 = invisible) */
	float shadeTime;										/* The time for animated shaders */
	float shadeXS, shadeYS;								/* The X and Y screen coordinates in floating point */
	SInt32 shadeXI, shadeYI;							/* The X and Y screen coordinates in integer */
	Matrix4 *shadeCameraMatrix, *shadeCameraInvMatrix;	/* The camera matrix and inverse camera matrix */
	Matrix4 *shadeGroupMatrix, *shadeGroupInvMatrix;	/* The group matrix and inverse group matrix */
	void *shadeGroupReference;							/* The current group reference */
	void *shadeFacetReference;							/* The current facet reference */

	L_Rect shadeRect;									/* The rendering rectangle */
	float shadePixelWidth;								/* The antialiasing pixel width */
	float shadeSampleWidth;								/* The sampling pixel width */
	float shadePixelScale;								/* The zoom factor */
	float shadePixelRatio;								/* The y/x pixel ratio */
	float shadeOffsetX, shadeOffsetY;					/* The x and y image offset */
	UInt32 shadePerspectiveFlag;						/* 1 for perspective rendering, 0 for isometric rendering */
	
	/* Version 2 */
	
	RGB_Real shadeHighlight;							/* The highlight color */

#if 0
	float shadeRoughnessX;								/* The bi-directional highlight X roughness */
	float shadeRoughnessY;								/* The bi-directional highlight Y roughness */
	Coordinate shadeDirectionX;							/* The bi-directional highlight X direction */
	Coordinate shadeDirectionY;							/* The bi-directional highlight Y direction */
#else
	UInt32 shadeThreadIndex;							/* The shade thread index */
	UInt32 shadeCustomID;								/* The custom ID */
	void * shadeCustomData;								/* The pointer to custom data */
	struct _EIShaderPixelRec * shadeParentPixelPtr;		/* The pointer to parent shading pixel or NULL */
#if __LP64__
	UInt32 reserved[2];
#else	
	UInt32 reserved[4];
#endif
#endif
	
	float shadeRefractionIndexNear;						/* The refraction index on the ray's side of the surface */
	float shadeRefractionIndexFar;						/* The refraction index on the other side of the surface */
	
	UInt32 shadeRayLevel;								/* The ray recursion level (0 for the top level) */
	RGB_Real shadeRefraction;							/* The refracted ray color */
	SInt32 shadeXI0, shadeYI0;							/* The X and Y screen coordinates of the top-left corner of the shaded pixel */
	float shadeRayLength;								/* The total distance along the rays between the camera and current point */
} EIShaderPixelRec, *EIShaderPixelPtr;

/* Mapping variables */

typedef struct {
	RMatrix4 mapMatrix;					/* The 4 by 4 transformation matrix for the original group coordinates */
	RMatrix4 mapWrapMatrix;				/* The 4 by 4 transformation from shader space back into camera space */
	RMatrix3 mapBumpMatrix;				/* The 3 by 3 pixel bump alignment matrix */
	float mapBumpScale;					/* The bump scaling factor (applied by the host) */
	float mapOpacity;					/* The opacity factor (applied by the host) */
	float map_X0, map_Y0, map_Z0;		/* Mapping coordinates at the current screen pixel */
	float map_X1, map_Y1, map_Z1;		/* Mapping coordinates at screen pixel x + 1 */
	float map_X2, map_Y2, map_Z2;		/* Mapping coordinates at screen pixel y - 1 */
	float map_DX, map_DY, map_DZ;		/* Mapping pixel coordinate deltas */
	unsigned char mapPerspectiveFlag;	/* True if the mapMatrix contains a non-linear transformation */
	RMatrix3 mapUV1Matrix;				/* The UV pre-matrix */
	RMatrix3 mapUV2Matrix;				/* The UV post-matrix */
} EIShaderMapRec, *EIShaderMapPtr;

/* Lightsource variables */

typedef struct {
	Matrix4 lightMatrix;									/* The lightsource's matrix (from light's space into camera's space) */
	Matrix4 lightBlurMatrix;								/* The lightsource's motion blur matrix (from light's space into camera's space) */
	DCoordinate lightPosition;								/* The lightsource's camera position in camera space*/
	DCoordinate lightBlurPosition;							/* The lightsource's motion blur camera position in camera space */
	DCoordinate lightWorldPosition;							/* The lightsource's position in world space */
	DCoordinate lightBlurWorldPosition;						/* The ligthsource's motion blur position in world space */
	DCoordinate lightNormal;								/* The lightsource's normalized direction vector in camera space */
	DCoordinate lightBlurNormal;							/* The lightsource's motion blur normalized direction vector in camera space */
	DCoordinate lightDirection;								/* The lightsource's tube direction in camera space */

	RGB_Real lightColor;									/* The lightsource's color */

	UInt32 lightIndex;										/* The lightsource's index */
	UInt32 lightFlags;										/* The lightsource's flags */

	float lightRadius;										/* The lightsource's radius (for lens flares) */
	float lightDropoff;										/* The lightsource's dropoff radius */
	float lightShadowDropoff;								/* The lightsource's ray trace shadow dropoff distance */
	
	float lightTubeLength;									/* Tube lightsource tube length */

	float lightConeFactor;									/* Spotlight dropoff between inner and outer cones */
	float lightConeInner;									/* Spotlight inner cone cosine angle */
	float lightConeOuter;									/* Spotlight outer cone cosine angle */
	
	float lightSoftShadowRadius;								/* The lightsource's soft shadow radius */
	RGB_Real lightPixelColorNoShadow;						/* The lightsource's pixel color before apply shadow */								

} EIShaderLightRec, *EIShaderLightPtr;

/* Prototypes for host callback functions */

typedef void* (*EIHostMallocProc)(ulong theSize);				/* MUTABLE 32/64 long */
typedef void (*EIHostFreeProc)(void *theData);
typedef void (*EIHostLightProc)(char **hostLight, UInt32 *diffuse, UInt32 *specular, float *dot, float *x, float *y, float *z, float *r, float *g, float *b);
typedef void (*EIHostLightInfoProc)(char **hostLight, EIShaderLightPtr *theLight);
typedef void (*EIHostChallengeProc)(IFFType challengeType, unsigned char *challengeRec);
typedef void (*EIHostCastRayProc)(RGB_Real *theColor, double *t, double xDir, double yDir, double zDir, double x, double y, double z);
typedef SInt32 (*EIHostInterfaceSetupProc)(void *theReference, UInt32 theCommand, UInt32 theParam1, void * theParam2);
typedef SInt32 (*EILayerShadeProc)(SInt32 theCommand, UInt32 theID, ARGB_Real * color);
typedef SInt32 (*EIModelShadeProc)(SInt32 theCommand, UInt32 theCommandSize, void * theCommandData);

/* Parameters for shader routines */

typedef struct {
	UInt32 shaderInterfaceSize;		/* The size of the shader's user interface parameters */
	char *shaderInterface;			/* Pointer to the shader's user interface parameters */
	UInt32 shaderAccessFlags;		/* Flags specifying which host parameters the shader will write */
	UInt32 shaderFeatureFlags;		/* Flags specifying which features the shader supports */
	
	/* Version 2 */
	
	UInt32 shaderAPIVersion;		/* The version number of the shader's API */
} EIShaderInformationRec, *EIShaderInformationPtr;

typedef struct {
	void *shaderData;					/* Pointer to the shader's private data */
	EIHostMallocProc HostMalloc;		/* Memory allocation function callback */
	EIHostFreeProc HostFree;			/* Memory free function callback */
	UInt32 shaderInterfaceSize;			/* The size of the shader's user interface parameters */
	char *shaderInterface;				/* Pointer to the shader's user interface parameters */
	
	/* Version 2 */
	
	void *shaderPluginData;				/* The plugin data block (usually nil) */
	UInt32 shaderPluginID;				/* The ID of the plugin that created the data block */
	UInt32 shaderPluginSize;		/* The size of the plugin data block */
	EIHostLightInfoProc HostLightInfo;	/* The lightsource information callback */
	EIShaderMapRec *shaderMap;			/* Mapping parameters */
} EIShaderInitializeRec, *EIShaderInitializePtr;

typedef struct {
	void *shaderData;					/* Pointer to the shader's private data */
	EIHostMallocProc HostMalloc;		/* Memory allocation function callback */
	EIHostFreeProc HostFree;			/* Memory free function callback */
} EIShaderFinishRec, *EIShaderFinishPtr;

typedef struct {
	void *shaderData;					/* Pointer to the shader's private data */
	EIHostLightProc HostLight;			/* The lightsource access callback */
	EIShaderPixelRec *shaderPixel;		/* Pixel perameters */
	EIShaderAttributesRec *shaderAttributes;	/* Shading attributes */
	EIShaderMapRec *shaderMap;			/* Mapping parameters */
	UInt32 shaderAccessFlags;			/* Flags specifying which host parameters the shader may write */
	UInt32 shaderFeatureFlags;			/* Flags specifying which features the shader supports */
	float shaderDX, shaderDY, shaderDZ;	/* The displacement vector */
	float shaderBumpA, shaderBumpB, shaderBumpC;	/* The bump perturbation vector */
	float shaderClip;					/* The shader clip factor */
	RGB_Real shaderTransparency;		/* The shader transparency color */
	RGB_Real shaderReflection;			/* The shader reflection color */
	ARGB_Real shaderColor;				/* The shader color and opacity */
	
	/* Version 2.0 */
	
	EIHostCastRayProc HostCastRay;		/* The ray trace callback */
	RGB_Real shaderHighlight;			/* The shader highlight color */
	RGB_Real shaderRefraction;			/* The shader refraction color */
	
	/* Version EI 7.0 */
	
	EILayerShadeProc  HostLayerShade;	/* The update layer(s) callback */

	/* Version EI 8.0 */
	
	EIModelShadeProc HostModelShade;	/* The model data access callback */
	
} EIShaderShadeRec, *EIShaderShadePtr;

typedef struct {
	EIHostChallengeProc HostChallenge;	/* The function call to get the host's serial number */
} EIShaderSerialRec, *EIShaderSerialPtr;

typedef struct {
	void *shaderData;					/* Pointer to the shader's private data */
	EIHostMallocProc HostMalloc;		/* Memory allocation function callback */
	EIHostFreeProc HostFree;			/* Memory free function callback */
	UInt32 shaderInterfaceSize;			/* The size of the shader's user interface parameters */
	char *shaderInterface;				/* Pointer to the shader's user interface parameters */
} EIShaderReinitializeRec, *EIShaderReinitializePtr;

typedef struct {
	UInt32 hostID;						/* The host's four character ID */
	UInt32 hostAPIVersion;				/* The host's EIShader API version */
} EIShaderHostInfoRec;


typedef struct {
	void *	hostDialogReference;					/* The host's shader dialog reference */
	SInt32  hostResourceFile;						/* The host's shader resource file */
	EIHostInterfaceSetupProc HostInterfaceSetup;	/* Interface setup function callback */
} EIShaderInterfaceSetupRec, *EIShaderInterfaceSetupPtr;

/* Prototypes for shader functions */

#define EIShaderInformationName		PS("EIShaderInformation")
#define EIShaderInitializeName		PS("EIShaderInitialize")
#define EIShaderFinishName			PS("EIShaderFinish")
#define EIShaderShadeName			PS("EIShaderShade")
#define EIShaderSerialName			PS("EIShaderSerial")
#define EIShaderReinitializeName	PS("EIShaderReinitialize")
#define EIShaderHostInfoName		PS("EIShaderHostInfo")
#define EIShaderInterfaceSetupName	PS("EIShaderInterfaceSetup")

typedef SInt32 (*EIShaderInformationProc)(EIShaderInformationRec *theInformationPtr);
typedef SInt32 (*EIShaderInitializeProc)(EIShaderInitializeRec *theInitializePtr);
typedef SInt32 (*EIShaderFinishProc)(EIShaderFinishRec *theFinishPtr);
typedef SInt32 (*EIShaderShadeProc)(EIShaderShadeRec *theShadePtr);
typedef SInt32 (*EIShaderSerialProc)(EIShaderSerialRec *theSerialPtr);
typedef SInt32 (*EIShaderReinitializeProc)(EIShaderReinitializeRec *theReinitializePtr);
typedef SInt32 (*EIShaderHostInfoProc)(EIShaderHostInfoRec *theHostInfoPtr);
typedef SInt32 (*EIShaderInterfaceSetupProc)(EIShaderInterfaceSetupRec *theHostInterfaceSetup);
typedef SInt32 (*EIShaderLayersProc)(void *theLayersDataPtr);

/* Shader connection structure */

typedef struct {
	slong shaderConnectionID;						/* Shared library connection ID */
	EIShaderInformationProc EIShaderInformation;	/* Get the shader feature and access flags */
	EIShaderInitializeProc EIShaderInitialize;		/* Initialize the shader instance */
	EIShaderFinishProc EIShaderFinish;				/* Finish the shader instance */
	EIShaderShadeProc EIShaderShade;				/* Shade a pixel */
	EIShaderSerialProc EIShaderSerial;				/* Send the serial number to the shader */
	EIShaderReinitializeProc EIShaderReinitialize;	/* Reinitialize the shader with new parameters */
	EIShaderHostInfoProc EIShaderHostInfo;			/* Send the ID and API version */
	EIShaderInterfaceSetupProc EIShaderInterfaceSetup; /* Setup shader's custom interface */
	EIShaderLayersProc EIShaderInitLayers;			/* Layers creating */
	EIShaderLayersProc EIShaderFillLayers;			/* Layers filling */
	EIShaderLayersProc EIShaderSlabLayers;			/* Layers slab process */
	EIShaderLayersProc EIShaderProcLayers;			/* Layers post-process */
} EIShaderInfoRec, *EIShaderInfoPtr;

#ifdef __MACINTOSH__
#pragma options align = reset
#endif

/* Export function prototypes */

SInt32 EIShaderInformation(EIShaderInformationRec *theInformationPtr);
SInt32 EIShaderInitialize(EIShaderInitializeRec *theInitializePtr);
SInt32 EIShaderFinish(EIShaderFinishRec *theFinishPtr);
SInt32 EIShaderShade(EIShaderShadeRec *theShadePtr);
SInt32 EIShaderSerial(EIShaderSerialRec *theSerialPtr);
SInt32 EIShaderReinitialize(EIShaderReinitializeRec *theReinitializePtr);
SInt32 EIShaderHostInfo(EIShaderHostInfoRec *theHostInfoPtr);
SInt32 EIShaderInterfaceSetup(EIShaderInterfaceSetupRec * theHostInterfaceSetupPtr);

/* EIShaderGetFunction will get a function from the shader or host */

void EIShaderGetFunction(slong theConnectionID, PStrPtr theFunctionName, char **theFunction, SInt32 *theResult);

/* EIShaderOpenConnection will open commumications with the shader and return the entry points */

SInt32 EIShaderOpenConnection(void *theFile, EIShaderInfoRec *theShaderInfo);

/* EIShaderCloseConnection will close commumications with the shader */

SInt32 EIShaderCloseConnection(EIShaderInfoRec *theShaderInfo);

#ifdef __cplusplus
}
#endif

#endif
