/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __EIShaderModel_h__
#define __EIShaderModel_h__

#ifndef __FACTStuff_h__
#include "FACTStuff.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __MACINTOSH__
#pragma options align = native
#endif

/* Commands passed by shader in  EIModelShadeProc */
 
#define  shaderModelGetGroup		QUADWORD('GETG')
#define  shaderModelGetFacetRef		QUADWORD('FREF')
#define  shaderModelGetVertexRef	QUADWORD('VREF')
#define  shaderModelGetFacet		QUADWORD('GETF')
#define  shaderModelGetVertex		QUADWORD('GETV')

/* Error codes */

#define  shaderModelCommandUnknError	-1
#define  shaderModelCommandSizeError	-2
#define  shaderModelCommandParamError	-3

typedef void * TGroupRef;
typedef void * TFacetRef;
typedef void * TVertexRef;

typedef struct {
	TGroupRef	theGroupReference;			/* => group reference 						*/
	UInt32		theGroupVertices;			/* <= returns group's count of vertices 	*/
	UInt32		theGroupFacets;				/* <= returns group's count of facets		*/
	UInt32		theGroupVertexFlags;		/* <= returns group's vertices flags		*/
} ShaderModelGetGroupRec, * ShaderModelGetGroupPtr;

typedef struct {
	TGroupRef		theGroupReference;			/* => group reference 						*/
	UInt32			theFacetIndexStart;			/* => facet index to read from (1-based)	*/
	UInt32			theFacetCount;				/* => count of facets to read				*/
	TFacetRef *  	theFacets;					/* <=> fills array with facets references	*/
} ShaderModelGetFacetRefRec, * ShaderModelGetFacetRefPtr;

typedef struct {
	TGroupRef		theGroupReference;			/* => group reference 						*/
	UInt32			theVertexIndexStart;		/* => vertex index to read from (1-based)	*/
	UInt32			theVertexCount;				/* => count of vertex to read				*/
	TVertexRef *	theVertices;				/* <=> fills array with vertex references	*/
} ShaderModelGetVertexRefRec, * ShaderModelGetVertexRefPtr;

typedef struct {
	TGroupRef		theGroupReference;			/* => group reference 						*/
	TFacetRef	    theFacetReference;			/* => group reference						*/
	ARGB 		    theFacetColor;				/* <= returns the facet color				*/
	UInt32			theFacetVertexCount;		/* <= returns the facet's vertices count	*/
	TVertexRef 		theVertices[4];				/* <= returns the facet's vertices			*/
} ShaderModelGetFacetRec, * ShaderModelGetFacetPtr;

typedef struct {
	TGroupRef		theGroupReference;			/* => group reference 						*/
	TVertexRef	    theVertexReference;			/* => vertex reference						*/
	ARGB 		    theVertexColor;				/* <= returns the vertex color				*/
	DCoordinate		thePosition;				/* <= returns the vertex position			*/
	Coordinate		theNormal;					/* <= returns the vertex normal				*/
	Coordinate		theTexturePosition;			/* <= returns the vertex texture position	*/
	Coordinate		theBlurPosition;			/* <= returns the vertex blur position		*/
} ShaderModelGetVertexRec, * ShaderModelGetVertexPtr;

SInt32 EIModelShade( SInt32 theCommand, UInt32 theCommandSize, void * theCommandData );

#ifdef __MACINTOSH__
#pragma options align = reset
#endif


#ifdef __cplusplus
}
#endif

#endif
