/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __FPU_h__
#define __FPU_h__

#ifndef __System_h__
#include "System.h"
#endif

#ifndef __FP__  /* fp.h has already been loaded */
	#include <math.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef PI
#define PI				3.14159265358979323846
#endif

#ifndef PI05
#define PI05			1.570796326794896619			/* 0.5 * pi */
#endif

#ifndef PI15
#define PI15			4.712388980384689858			/* 1.5 * pi */
#endif

#ifndef PI2
#define PI2				6.283185307179586478			/* 2.0 * pi */
#endif

#ifndef PINeg
#define PINeg		   	-3.14159265358979323846			/* -pi */
#endif

#ifndef PINeg05
#define PINeg05			-1.570796326794896619			/* -0.5 * pi */
#endif

#ifndef PINeg15
#define PINeg15			-4.712388980384689858			/* -1.5 * pi */
#endif

#ifndef PINeg2
#define PINeg2			-6.283185307179586478			/* -2.0 * pi */
#endif

#ifndef PIFac
#define PIFac			0.0174532925199432957			/* pi / 180 */
#endif

#ifndef INFHEX
#define INFHEX			0x7F800000						/* Single precision INF in Hex */
#endif

#ifndef NEGINFHEX
#define NEGINFHEX		0xFF800000						/* Single precision -INF in Hex */
#endif

#ifndef NANHEX
#define NANHEX			0x7FFFFFFF						/* Illegal floating point number */
#endif

#ifndef ZEROHEX
#define ZEROHEX			0x00000000						/* Single precision 0.0 in Hex */
#endif


#define roundToNearest	0
#define roundToZero		1
#define roundToCeil		2
#define roundToFloor	3

#ifdef __WIN32__
#define round(n)		floor((n) + 0.5)				/* The round function which ANSI forgot */
#endif

#ifdef __SUN__
#if 1
#define trunc(a) ((double)((SInt32)a))					/* Limited to 32 bit integer resolution */
#else
#define trunc(a) ((a) >= 0.0 ? floor(a) : ceil(a))		/* Possible side effects and slower */
#endif
#endif

#ifdef __MACINTOSH__

/* Default Rounding Direction */

#define ROUNDDIR(dir) {\
	RealRecord _RR_;\
	_RR_.r = __setflm(0.0);\
	_RR_.l &= 0xFFFFFFFC;\
	_RR_.l |= (dir);\
	__setflm(_RR_.r);\
}

#else

#define ROUNDDIR(dir)

#endif

//#if defined(__MACINTOSH__) && defined(powerc)
#if 0

/* Motorola Math Library */

#define SIN	Sin
#define COS	Cos
#define SQRT Sqrt
#define ATAN ATan
#define ATAN2 ATan2
#define EXP Exp
#define LOG Log
#define LOG10 Log10
#define POW Pow

/* Missing math functions */

#define TAN Tan
#define ASIN ASin
#define ACOS ACos
#define SINH SinH
#define COSH CosH
#define TANH TanH

#ifdef __cplusplus
extern "C" {
#endif
	double Sin(double x);
	double Cos(double x);
	double Sqrt(double x);
	double ATan(double x);
	double ATan2(double x, double y);
	double Exp(double x);
	double Log(double x);
	double Log10(double x);
	double Pow(double x, double y);

	double Tan(double x);
	double ASin(double x);
	double ACos(double x);
	double SinH(double x);
	double CosH(double x);
	double TanH(double x);
#ifdef __cplusplus
}
#endif

#else

/* Standard Math Library */

#define SIN			sin
#define COS			cos
#define SQRT		sqrt
#define TAN			tan
#define ATAN		atan
#define ATAN2		atan2
#define ASIN		asin
#define ACOS		acos
#define SINH		sinh
#define COSH		cosh
#define TANH		tanh
#define EXP			exp
#define LOG			log
#define LOG10		log10
#define POW			pow

#endif

/* 8, 10, 12 Byte Floating Point Conversions */

void ConvertFloat12To10(void* the12, void* the10);
void ConvertFloat10To12(void* the10, void* the12);
void ConvertFloat10To8(void* the10, void* the8);
void ConvertFloat8To10(void* the8, void* the10);
void ConvertFloat12To8(void* the12, void* the8);
void ConvertFloat8To12(void* the8, void* the12);

/* Convert a binary hex constant to a float */

float HexFloat(SInt32 h);

/* Convert a float to its binary representation */

SInt32 FloatHex(float f);

/* Vaidate and fix floating point garbage values */

unsigned char ValidateReal(float* theValue);
unsigned char ValidateDouble(double* theValue);
float FixReal(float theValue);
double FixDouble(double theValue);

#ifdef __cplusplus
}
#endif

#endif
