/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __Matrix_h__
#define __Matrix_h__

#ifndef __FACTStuff_h__
#include "FACTStuff.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define EQUAL1(n)		((float)(n) == 1.0f)
#define EQUAL0(n)		EQUAL1((n) + 1.0)
#define EQUAL2(a, b)	EQUAL0((a) - (b))

#define VECTORPRODUCT(a1, b1, c1, a2, b2, c2, a3, b3, c3) {\
	a3 = b1 * c2 - c1 * b2;\
	b3 = c1 * a2 - a1 * c2;\
	c3 = a1 * b2 - b1 * a2;\
}

#define DOTPRODUCT(a1, b1, c1, a2, b2, c2) (a1 * a2 + b1 * b2 + c1 * c2)

/* Math Routines */

void Normalize(double* a, double* b, double* c);
void NormalizeR(float* a, float* b, float* c);
unsigned char CheckNormalize(double* a, double* b, double* c);
unsigned char CheckNormalizeR(float* a, float* b, float* c);
void Normalize2D(double* a, double* b);
void Normalize2DR(float* a, float* b);
void VectorProduct(double a1, double b1, double c1, double a2, double b2, double c2, double* a3, double* b3, double* c3);
void VectorProductR(float a1, float b1, float c1, float a2, float b2, float c2, float* a3, float* b3, float* c3);
double DotProduct(double a1, double b1, double c1, double a2, double b2, double c2);
float DotProductR(float a1, float b1, float c1, float a2, float b2, float c2);
void AlignVectors(double axisA, double axisB, double axisC, double srcA, double srcB, double srcC, double *dstA, double *dstB, double *dstC);
void AlignVectorsR(float axisA, float axisB, float axisC, float srcA, float srcB, float srcC, float *dstA, float *dstB, float *dstC);
double TriangleArea(double x1, double y1, double x2, double y2, double x3, double y3);
double TriangleArea3(double x1, double y1, double z1, double x2, double y2, double z2, double x3, double y3, double z3);
double QuadrangleArea(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4);
double QuadrangleArea3(double x1, double y1, double z1, double x2, double y2, double z2, double x3, double y3, double z3, double x4, double y4, double z4);
double CalcDotValue(double theAngle);
double CalcAngle(double dotValue);
void Rot(double sinTheta, double cosTheta, double* n1, double* n2);
void RotR(float sinTheta, float cosTheta, float* n1, float* n2);
long Sgn(double n);
unsigned char Equal0(float n);
unsigned char Equal1(float n);
double LineDistance(double xp1, double yp1, double zp1, double xn1, double yn1, double zn1, double length1, double xp2, double yp2, double zp2, double xn2, double yn2, double zn2, double length2);
void Line1Closest(double* xp1, double* yp1, double* zp1, double xn1, double yn1, double zn1, double length1, double xp2, double yp2, double zp2, double xn2, double yn2, double zn2);
void Line2Closest(double* xp1, double* yp1, double* zp1, double xn1, double yn1, double zn1, double length1, double* xp2, double* yp2, double* zp2, double xn2, double yn2, double zn2, double length2);
void CalcVector(CoordPtr coord1, CoordPtr coord2, CoordPtr coord3, double* aNum, double* bNum, double* cNum);
unsigned char CalcVector2(CoordPtr coord1, CoordPtr coord2, CoordPtr coord3, double* aNum, double* bNum, double* cNum);
void CalcDVector(DCoordPtr coord1, DCoordPtr coord2, DCoordPtr coord3, double* aNum, double* bNum, double* cNum);
unsigned char CalcDVector2(DCoordPtr coord1, DCoordPtr coord2, DCoordPtr coord3, double* aNum, double* bNum, double* cNum);
unsigned char CalcNormal(CoordPtr coord1, CoordPtr coord2, CoordPtr coord3, double* aNum, double* bNum, double* cNum);
unsigned char CalcDNormal(DCoordPtr coord1, DCoordPtr coord2, DCoordPtr coord3, double* aNum, double* bNum, double* cNum);
void CalcPlaneFormula(CoordPtr coord1, CoordPtr coord2, CoordPtr coord3, double* aNum, double* bNum, double* cNum, double* dNum);
void CalcDPlaneFormula(DCoordPtr coord1, DCoordPtr coord2, DCoordPtr coord3, double* aNum, double* bNum, double* cNum, double* dNum);

/* Matrix Routines */

unsigned char TestIdentityMatrix4(Matrix4Ptr theMatrix);
unsigned char TestIdentityRMatrix4(RMatrix4Ptr theMatrix);
unsigned char TestIdentityMatrix3(Matrix3Ptr theMatrix);
unsigned char TestIdentityRMatrix3(RMatrix3Ptr theMatrix);

#define TESTIDENTITYMATRIX3(matrix) (EQUAL1((matrix)->m.m00) && EQUAL1((matrix)->m.m11) && EQUAL1((matrix)->m.m22) && EQUAL0((matrix)->m.m01) && EQUAL0((matrix)->m.m02) && EQUAL0((matrix)->m.m10) && EQUAL0((matrix)->m.m12) && EQUAL0((matrix)->m.m20) && EQUAL0((matrix)->m.m21))
#define TESTIDENTITYMATRIX4(matrix) (TESTIDENTITYMATRIX3(matrix) && EQUAL1((matrix)->m.m33) && EQUAL0((matrix)->m.m03) && EQUAL0((matrix)->m.m13) && EQUAL0((matrix)->m.m23) && EQUAL0((matrix)->m.m30) && EQUAL0((matrix)->m.m31) && EQUAL0((matrix)->m.m32))

unsigned char TestZeroMatrix4(Matrix4Ptr theMatrix);
unsigned char TestZeroRMatrix4(RMatrix4Ptr theMatrix);
unsigned char TestZeroMatrix3(Matrix3Ptr theMatrix);
unsigned char TestZeroRMatrix3(RMatrix3Ptr theMatrix);

void IdentityMatrix4(Matrix4Ptr theMatrix);
void IdentityRMatrix4(RMatrix4Ptr theMatrix);
void IdentityMatrix3(Matrix3Ptr theMatrix);
void IdentityRMatrix3(RMatrix3Ptr theMatrix);

#define IDENTITYMATRIX4(theMatrix) {\
	(theMatrix)->m.m00 = 1.0f; (theMatrix)->m.m01 = 0.0f; (theMatrix)->m.m02 = 0.0f; (theMatrix)->m.m03 = 0.0f;\
	(theMatrix)->m.m10 = 0.0f; (theMatrix)->m.m11 = 1.0f; (theMatrix)->m.m12 = 0.0f; (theMatrix)->m.m13 = 0.0f;\
	(theMatrix)->m.m20 = 0.0f; (theMatrix)->m.m21 = 0.0f; (theMatrix)->m.m22 = 1.0f; (theMatrix)->m.m23 = 0.0f;\
	(theMatrix)->m.m30 = 0.0f; (theMatrix)->m.m31 = 0.0f; (theMatrix)->m.m32 = 0.0f; (theMatrix)->m.m33 = 1.0f;\
}
#define IDENTITYMATRIX3(theMatrix) {\
	(theMatrix)->m.m00 = 1.0f; (theMatrix)->m.m01 = 0.0f; (theMatrix)->m.m02 = 0.0f;\
	(theMatrix)->m.m10 = 0.0f; (theMatrix)->m.m11 = 1.0f; (theMatrix)->m.m12 = 0.0f;\
	(theMatrix)->m.m20 = 0.0f; (theMatrix)->m.m21 = 0.0f; (theMatrix)->m.m22 = 1.0f;\
}

/* SenseMatrix tests the handedness of the matrix.  Returns 1 if the third row of the */
/* matrix points in the same direction as the cross products of the first two rows */

unsigned char SenseMatrix4(Matrix4Ptr theMatrix);
unsigned char SenseRMatrix4(RMatrix4Ptr theMatrix);
unsigned char SenseMatrix3(Matrix3Ptr theMatrix);
unsigned char SenseRMatrix3(RMatrix3Ptr theMatrix);

void InvertMatrix4(Matrix4Ptr theMatrix, Matrix4 srcMatrix);
void InvertRMatrix4(RMatrix4Ptr theMatrix, RMatrix4 srcMatrix);
void InvertMatrix3(Matrix3Ptr theMatrix, Matrix3 srcMatrix);
void InvertRMatrix3(RMatrix3Ptr theMatrix, RMatrix3 srcMatrix);

#define TRANSPOSEMATRIX4(theMatrix) {\
	double _temp;\
	_temp = (theMatrix)->m.m01; (theMatrix)->m.m01 = (theMatrix)->m.m10; (theMatrix)->m.m10 = _temp;\
	_temp = (theMatrix)->m.m02; (theMatrix)->m.m02 = (theMatrix)->m.m20; (theMatrix)->m.m20 = _temp;\
	_temp = (theMatrix)->m.m03; (theMatrix)->m.m03 = (theMatrix)->m.m30; (theMatrix)->m.m30 = _temp;\
	_temp = (theMatrix)->m.m12; (theMatrix)->m.m12 = (theMatrix)->m.m21; (theMatrix)->m.m21 = _temp;\
	_temp = (theMatrix)->m.m13; (theMatrix)->m.m13 = (theMatrix)->m.m31; (theMatrix)->m.m31 = _temp;\
	_temp = (theMatrix)->m.m23; (theMatrix)->m.m23 = (theMatrix)->m.m32; (theMatrix)->m.m32 = _temp;\
}

#define TRANSPOSEMATRIX3(theMatrix) {\
	double _temp;\
	_temp = (theMatrix)->m.m01; (theMatrix)->m.m01 = (theMatrix)->m.m10; (theMatrix)->m.m10 = _temp;\
	_temp = (theMatrix)->m.m02; (theMatrix)->m.m02 = (theMatrix)->m.m20; (theMatrix)->m.m20 = _temp;\
	_temp = (theMatrix)->m.m12; (theMatrix)->m.m12 = (theMatrix)->m.m21; (theMatrix)->m.m21 = _temp;\
}

void TransposeMatrix4(Matrix4Ptr theMatrix, Matrix4 srcMatrix);
void TransposeRMatrix4(RMatrix4Ptr theMatrix, RMatrix4 srcMatrix);
void TransposeMatrix3(Matrix3Ptr theMatrix, Matrix3 srcMatrix);
void TransposeRMatrix3(RMatrix3Ptr theMatrix, RMatrix3 srcMatrix);

double Det(Matrix3Ptr a);
float DetR(RMatrix3Ptr a);

void AppendMatrix4(Matrix4Ptr theMatrix, Matrix4Ptr b);
void AppendRMatrix4(RMatrix4Ptr theMatrix, RMatrix4Ptr b);
void AppendMatrix3(Matrix3Ptr theMatrix, Matrix3Ptr b);
void AppendRMatrix3(RMatrix3Ptr theMatrix, RMatrix3Ptr b);

void InsertMatrix4(Matrix4Ptr theMatrix, Matrix4Ptr a);
void InsertRMatrix4(RMatrix4Ptr theMatrix, RMatrix4Ptr a);
void InsertMatrix3(Matrix3Ptr theMatrix, Matrix3Ptr a);
void InsertRMatrix3(RMatrix3Ptr theMatrix, RMatrix3Ptr a);

void NormalMatrix4(Matrix4Ptr theMatrix);
void NormalRMatrix4(RMatrix4Ptr theMatrix);
void NormalMatrix3(Matrix3Ptr theMatrix);
void NormalRMatrix3(RMatrix3Ptr theMatrix);

void NormalizeMatrix3(Matrix3Ptr theMatrix);
void NormalizeRMatrix3(RMatrix3Ptr theMatrix);

void SwapXYMatrix4(Matrix4Ptr theMatrix, Matrix4 srcMatrix);
void SwapXYRMatrix4(RMatrix4Ptr theMatrix, RMatrix4 srcMatrix);
void SwapXYMatrix3(Matrix3Ptr theMatrix, Matrix3 srcMatrix);
void SwapXYRMatrix3(RMatrix3Ptr theMatrix, RMatrix3 srcMatrix);

void SwapXZMatrix4(Matrix4Ptr theMatrix, Matrix4 srcMatrix);
void SwapXZRMatrix4(RMatrix4Ptr theMatrix, RMatrix4 srcMatrix);
void SwapXZMatrix3(Matrix3Ptr theMatrix, Matrix3 srcMatrix);
void SwapXZRMatrix3(RMatrix3Ptr theMatrix, RMatrix3 srcMatrix);

void SwapYZMatrix4(Matrix4Ptr theMatrix, Matrix4 srcMatrix);
void SwapYZRMatrix4(RMatrix4Ptr theMatrix, RMatrix4 srcMatrix);
void SwapYZMatrix3(Matrix3Ptr theMatrix, Matrix3 srcMatrix);
void SwapYZRMatrix3(RMatrix3Ptr theMatrix, RMatrix3 srcMatrix);

#define TRANSLATEMATRIX4(theMatrix, tx, ty, tz) {\
	(theMatrix)->m.m00 += (theMatrix)->m.m03 * (tx);\
	(theMatrix)->m.m01 += (theMatrix)->m.m03 * (ty);\
	(theMatrix)->m.m02 += (theMatrix)->m.m03 * (tz);\
	(theMatrix)->m.m10 += (theMatrix)->m.m13 * (tx);\
	(theMatrix)->m.m11 += (theMatrix)->m.m13 * (ty);\
	(theMatrix)->m.m12 += (theMatrix)->m.m13 * (tz);\
	(theMatrix)->m.m20 += (theMatrix)->m.m23 * (tx);\
	(theMatrix)->m.m21 += (theMatrix)->m.m23 * (ty);\
	(theMatrix)->m.m22 += (theMatrix)->m.m23 * (tz);\
	(theMatrix)->m.m30 += (theMatrix)->m.m33 * (tx);\
	(theMatrix)->m.m31 += (theMatrix)->m.m33 * (ty);\
	(theMatrix)->m.m32 += (theMatrix)->m.m33 * (tz);\
}
void TranslateMatrix4(Matrix4Ptr theMatrix, double tx, double ty, double tz);
void TranslateRMatrix4(RMatrix4Ptr theMatrix, float tx, float ty, float tz);

#define XROTATEMATRIX4(theMatrix, sinTheta, cosTheta) {\
	double temp0 = (theMatrix)->m.m01;\
	double temp1 = (theMatrix)->m.m02;\
	(theMatrix)->m.m01 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m02 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m11;\
	temp1 = (theMatrix)->m.m12;\
	(theMatrix)->m.m11 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m12 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m21;\
	temp1 = (theMatrix)->m.m22;\
	(theMatrix)->m.m21 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m22 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m31;\
	temp1 = (theMatrix)->m.m32;\
	(theMatrix)->m.m31 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m32 = temp0 * (sinTheta) + temp1 * (cosTheta);\
}
#define XROTATEMATRIX3(theMatrix, sinTheta, cosTheta) {\
	double temp0 = (theMatrix)->m.m01;\
	double temp1 = (theMatrix)->m.m02;\
	(theMatrix)->m.m01 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m02 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m11;\
	temp1 = (theMatrix)->m.m12;\
	(theMatrix)->m.m11 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m12 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m21;\
	temp1 = (theMatrix)->m.m22;\
	(theMatrix)->m.m21 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m22 = temp0 * (sinTheta) + temp1 * (cosTheta);\
}
void XRotateMatrix4(Matrix4Ptr theMatrix, double sinTheta, double cosTheta);
void XRotateRMatrix4(RMatrix4Ptr theMatrix, float sinTheta, float cosTheta);
void XRotateMatrix3(Matrix3Ptr theMatrix, double sinTheta, double cosTheta);
void XRotateRMatrix3(RMatrix3Ptr theMatrix, float sinTheta, float cosTheta);

#define YROTATEMATRIX4(theMatrix, sinTheta, cosTheta) {\
	double temp0 = (theMatrix)->m.m00;\
	double temp1 = (theMatrix)->m.m02;\
	(theMatrix)->m.m00 = temp0 * (cosTheta) + temp1 * (sinTheta);\
	(theMatrix)->m.m02 = temp1 * (cosTheta) - temp0 * (sinTheta);\
	temp0 = (theMatrix)->m.m10;\
	temp1 = (theMatrix)->m.m12;\
	(theMatrix)->m.m10 = temp0 * (cosTheta) + temp1 * (sinTheta);\
	(theMatrix)->m.m12 = temp1 * (cosTheta) - temp0 * (sinTheta);\
	temp0 = (theMatrix)->m.m20;\
	temp1 = (theMatrix)->m.m22;\
	(theMatrix)->m.m20 = temp0 * (cosTheta) + temp1 * (sinTheta);\
	(theMatrix)->m.m22 = temp1 * (cosTheta) - temp0 * (sinTheta);\
	temp0 = (theMatrix)->m.m30;\
	temp1 = (theMatrix)->m.m32;\
	(theMatrix)->m.m30 = temp0 * (cosTheta) + temp1 * (sinTheta);\
	(theMatrix)->m.m32 = temp1 * (cosTheta) - temp0 * (sinTheta);\
}
#define YROTATEMATRIX3(theMatrix, sinTheta, cosTheta) {\
	double temp0 = (theMatrix)->m.m00;\
	double temp1 = (theMatrix)->m.m02;\
	(theMatrix)->m.m00 = temp0 * (cosTheta) + temp1 * (sinTheta);\
	(theMatrix)->m.m02 = temp1 * (cosTheta) - temp0 * (sinTheta);\
	temp0 = (theMatrix)->m.m10;\
	temp1 = (theMatrix)->m.m12;\
	(theMatrix)->m.m10 = temp0 * (cosTheta) + temp1 * (sinTheta);\
	(theMatrix)->m.m12 = temp1 * (cosTheta) - temp0 * (sinTheta);\
	temp0 = (theMatrix)->m.m20;\
	temp1 = (theMatrix)->m.m22;\
	(theMatrix)->m.m20 = temp0 * (cosTheta) + temp1 * (sinTheta);\
	(theMatrix)->m.m22 = temp1 * (cosTheta) - temp0 * (sinTheta);\
}
void YRotateMatrix4(Matrix4Ptr theMatrix, double sinTheta, double cosTheta);
void YRotateRMatrix4(RMatrix4Ptr theMatrix, float sinTheta, float cosTheta);
void YRotateMatrix3(Matrix3Ptr theMatrix, double sinTheta, double cosTheta);
void YRotateRMatrix3(RMatrix3Ptr theMatrix, float sinTheta, float cosTheta);

#define ZROTATEMATRIX4(theMatrix, sinTheta, cosTheta) {\
	double temp0 = (theMatrix)->m.m00;\
	double temp1 = (theMatrix)->m.m01;\
	(theMatrix)->m.m00 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m01 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m10;\
	temp1 = (theMatrix)->m.m11;\
	(theMatrix)->m.m10 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m11 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m20;\
	temp1 = (theMatrix)->m.m21;\
	(theMatrix)->m.m20 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m21 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m30;\
	temp1 = (theMatrix)->m.m31;\
	(theMatrix)->m.m30 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m31 = temp0 * (sinTheta) + temp1 * (cosTheta);\
}
#define ZROTATEMATRIX3(theMatrix, sinTheta, cosTheta) {\
	double temp0 = (theMatrix)->m.m00;\
	double temp1 = (theMatrix)->m.m01;\
	(theMatrix)->m.m00 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m01 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m10;\
	temp1 = (theMatrix)->m.m11;\
	(theMatrix)->m.m10 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m11 = temp0 * (sinTheta) + temp1 * (cosTheta);\
	temp0 = (theMatrix)->m.m20;\
	temp1 = (theMatrix)->m.m21;\
	(theMatrix)->m.m20 = temp0 * (cosTheta) - temp1 * (sinTheta);\
	(theMatrix)->m.m21 = temp0 * (sinTheta) + temp1 * (cosTheta);\
}
void ZRotateMatrix4(Matrix4Ptr theMatrix, double sinTheta, double cosTheta);
void ZRotateRMatrix4(RMatrix4Ptr theMatrix, float sinTheta, float cosTheta);
void ZRotateMatrix3(Matrix3Ptr theMatrix, double sinTheta, double cosTheta);
void ZRotateRMatrix3(RMatrix3Ptr theMatrix, float sinTheta, float cosTheta);

void RotateMatrix4(Matrix4Ptr theMatrix, double a, double b, double c, double theta);
void RotateRMatrix4(RMatrix4Ptr theMatrix, float a, float b, float c, float theta);
void RotateMatrix3(Matrix3Ptr theMatrix, double a, double b, double c, double theta);
void RotateRMatrix3(RMatrix3Ptr theMatrix, float a, float b, float c, float theta);

#define SCALEMATRIX4(theMatrix, sx, sy, sz) {\
	(theMatrix)->m.m00 *= (sx);\
	(theMatrix)->m.m01 *= (sy);\
	(theMatrix)->m.m02 *= (sz);\
	(theMatrix)->m.m10 *= (sx);\
	(theMatrix)->m.m11 *= (sy);\
	(theMatrix)->m.m12 *= (sz);\
	(theMatrix)->m.m20 *= (sx);\
	(theMatrix)->m.m21 *= (sy);\
	(theMatrix)->m.m22 *= (sz);\
	(theMatrix)->m.m30 *= (sx);\
	(theMatrix)->m.m31 *= (sy);\
	(theMatrix)->m.m32 *= (sz);\
}
#define SCALEMATRIX3(theMatrix, sx, sy, sz) {\
	(theMatrix)->m.m00 *= (sx);\
	(theMatrix)->m.m01 *= (sy);\
	(theMatrix)->m.m02 *= (sz);\
	(theMatrix)->m.m10 *= (sx);\
	(theMatrix)->m.m11 *= (sy);\
	(theMatrix)->m.m12 *= (sz);\
	(theMatrix)->m.m20 *= (sx);\
	(theMatrix)->m.m21 *= (sy);\
	(theMatrix)->m.m22 *= (sz);\
}
void ScaleMatrix4(Matrix4Ptr theMatrix, double sx, double sy, double sz);
void ScaleRMatrix4(RMatrix4Ptr theMatrix, float sx, float sy, float sz);
void ScaleMatrix3(Matrix3Ptr theMatrix, double sx, double sy, double sz);
void ScaleRMatrix3(RMatrix3Ptr theMatrix, float sx, float sy, float sz);

#define NEGATEXMATRIX4(theMatrix) {\
	(theMatrix)->m.m00 = -(theMatrix)->m.m00;\
	(theMatrix)->m.m10 = -(theMatrix)->m.m10;\
	(theMatrix)->m.m20 = -(theMatrix)->m.m20;\
	(theMatrix)->m.m30 = -(theMatrix)->m.m30;\
}
#define NEGATEXMATRIX3(theMatrix) {\
	(theMatrix)->m.m00 = -(theMatrix)->m.m00;\
	(theMatrix)->m.m10 = -(theMatrix)->m.m10;\
	(theMatrix)->m.m20 = -(theMatrix)->m.m20;\
}
void NegateXMatrix4(Matrix4Ptr theMatrix);
void NegateXRMatrix4(RMatrix4Ptr theMatrix);
void NegateXMatrix3(Matrix3Ptr theMatrix);
void NegateXRMatrix3(RMatrix3Ptr theMatrix);

#define NEGATEYMATRIX4(theMatrix) {\
	(theMatrix)->m.m01 = -(theMatrix)->m.m01;\
	(theMatrix)->m.m11 = -(theMatrix)->m.m11;\
	(theMatrix)->m.m21 = -(theMatrix)->m.m21;\
	(theMatrix)->m.m31 = -(theMatrix)->m.m31;\
}
#define NEGATEYMATRIX3(theMatrix) {\
	(theMatrix)->m.m01 = -(theMatrix)->m.m01;\
	(theMatrix)->m.m11 = -(theMatrix)->m.m11;\
	(theMatrix)->m.m21 = -(theMatrix)->m.m21;\
}
void NegateYMatrix4(Matrix4Ptr theMatrix);
void NegateYRMatrix4(RMatrix4Ptr theMatrix);
void NegateYMatrix3(Matrix3Ptr theMatrix);
void NegateYRMatrix3(RMatrix3Ptr theMatrix);

#define NEGATEZMATRIX4(theMatrix) {\
	(theMatrix)->m.m02 = -(theMatrix)->m.m02;\
	(theMatrix)->m.m12 = -(theMatrix)->m.m12;\
	(theMatrix)->m.m22 = -(theMatrix)->m.m22;\
	(theMatrix)->m.m32 = -(theMatrix)->m.m32;\
}
#define NEGATEZMATRIX3(theMatrix) {\
	(theMatrix)->m.m02 = -(theMatrix)->m.m02;\
	(theMatrix)->m.m12 = -(theMatrix)->m.m12;\
	(theMatrix)->m.m22 = -(theMatrix)->m.m22;\
}
void NegateZMatrix4(Matrix4Ptr theMatrix);
void NegateZRMatrix4(RMatrix4Ptr theMatrix);
void NegateZMatrix3(Matrix3Ptr theMatrix);
void NegateZRMatrix3(RMatrix3Ptr theMatrix);

#define SKEWMATRIX4(theMatrix, skew) {\
	double _temp_;\
	_temp_ = (theMatrix)->m.m02 * (skew);\
	(theMatrix)->m.m00 += _temp_;\
	(theMatrix)->m.m01 += _temp_;\
	_temp_ = (theMatrix)->m.m12 * (skew);\
	(theMatrix)->m.m10 += _temp_;\
	(theMatrix)->m.m11 += _temp_;\
	_temp_ = (theMatrix)->m.m22 * (skew);\
	(theMatrix)->m.m20 += _temp_;\
	(theMatrix)->m.m21 += _temp_;\
	_temp_ = (theMatrix)->m.m32 * (skew);\
	(theMatrix)->m.m30 += _temp_;\
	(theMatrix)->m.m31 += _temp_;\
}
#define SKEWMATRIX3(theMatrix, skew) {\
	double _temp_;\
	_temp_ = (theMatrix)->m.m02 * (skew);\
	(theMatrix)->m.m00 += _temp_;\
	(theMatrix)->m.m01 += _temp_;\
	_temp_ = (theMatrix)->m.m12 * (skew);\
	(theMatrix)->m.m10 += _temp_;\
	(theMatrix)->m.m11 += _temp_;\
	_temp_ = (theMatrix)->m.m22 * (skew);\
	(theMatrix)->m.m20 += _temp_;\
	(theMatrix)->m.m21 += _temp_;\
}
void SkewMatrix4(Matrix4Ptr theMatrix, double skew);
void SkewRMatrix4(RMatrix4Ptr theMatrix, float skew);
void SkewMatrix3(Matrix3Ptr theMatrix, double skew);
void SkewRMatrix3(RMatrix3Ptr theMatrix, float skew);

void ECSMatrix4(Matrix4Ptr theMatrix, double dx, double dy, double dz);
void ECSRMatrix4(RMatrix4Ptr theMatrix, float dx, float dy, float dz);
void ECSMatrix3(Matrix3Ptr theMatrix, double dx, double dy, double dz);
void ECSRMatrix3(RMatrix3Ptr theMatrix, float dx, float dy, float dz);

void TransformVector(double* a, double* b, double* c, Matrix4Ptr theMatrix);
void TransformNormal(double* a, double* b, double* c, Matrix4Ptr theMatrix);
void TransformPoint(double* x, double* y, double* z, Matrix4Ptr theMatrix);
void TransformPointR(double* x, double* y, double* z, Matrix4Ptr theMatrix);
void TransformPointH(double* x, double* y, double* z, Matrix4Ptr theMatrix);
void TransformXPoint(double* x, double* y, double* z, XMatrix4Ptr theMatrix);
void TransformRPoint(float* x, float* y, float* z, Matrix4Ptr theMatrix);
void TransformRPointH(float* x, float* y, float* z, Matrix4Ptr theMatrix);
void TransformCoordinate(CoordPtr theCoordinate, Matrix4Ptr theMatrix);
void TransformCoordinateH(CoordPtr theCoordinate, Matrix4Ptr theMatrix);
void TransformDCoordinate(DCoordPtr theCoordinate, Matrix4Ptr theMatrix);
void TransformDCoordinateH(DCoordPtr theCoordinate, Matrix4Ptr theMatrix);
void TransformPoint3(double* x, double* y, double* z, Matrix3Ptr theMatrix);
#define TRANSFORMPOINT4(x, y, z, matrix) {\
	double __tempX__ = x;\
	double __tempY__ = y;\
	double __tempZ__ = z;\
	x = __tempX__ * (matrix).m.m00 + __tempY__ * (matrix).m.m10 + __tempZ__ * (matrix).m.m20 + (matrix).m.m30;\
	y = __tempX__ * (matrix).m.m01 + __tempY__ * (matrix).m.m11 + __tempZ__ * (matrix).m.m21 + (matrix).m.m31;\
	z = __tempX__ * (matrix).m.m02 + __tempY__ * (matrix).m.m12 + __tempZ__ * (matrix).m.m22 + (matrix).m.m32;\
}
#define TRANSFORMRPOINT4(x, y, z, matrix) {\
	float __tempX__ = x;\
	float __tempY__ = y;\
	float __tempZ__ = z;\
	x = (float)(__tempX__ * (matrix).m.m00 + __tempY__ * (matrix).m.m10 + __tempZ__ * (matrix).m.m20 + (matrix).m.m30);\
	y = (float)(__tempX__ * (matrix).m.m01 + __tempY__ * (matrix).m.m11 + __tempZ__ * (matrix).m.m21 + (matrix).m.m31);\
	z = (float)(__tempX__ * (matrix).m.m02 + __tempY__ * (matrix).m.m12 + __tempZ__ * (matrix).m.m22 + (matrix).m.m32);\
}
#define TRANSFORMPOINT3(x, y, z, matrix) {\
	double __tempX__ = x;\
	double __tempY__ = y;\
	double __tempZ__ = z;\
	x = __tempX__ * (matrix).m.m00 + __tempY__ * (matrix).m.m10 + __tempZ__ * (matrix).m.m20;\
	y = __tempX__ * (matrix).m.m01 + __tempY__ * (matrix).m.m11 + __tempZ__ * (matrix).m.m21;\
	z = __tempX__ * (matrix).m.m02 + __tempY__ * (matrix).m.m12 + __tempZ__ * (matrix).m.m22;\
}
#define TRANSFORMRPOINT3(x, y, z, matrix) {\
	float __tempX__ = x;\
	float __tempY__ = y;\
	float __tempZ__ = z;\
	x = (float)(__tempX__ * (matrix).m.m00 + __tempY__ * (matrix).m.m10 + __tempZ__ * (matrix).m.m20);\
	y = (float)(__tempX__ * (matrix).m.m01 + __tempY__ * (matrix).m.m11 + __tempZ__ * (matrix).m.m21);\
	z = (float)(__tempX__ * (matrix).m.m02 + __tempY__ * (matrix).m.m12 + __tempZ__ * (matrix).m.m22);\
}
#define TRANSFORMPOINTH(x, y, z, matrix) {\
	double __tempX__ = x;\
	double __tempY__ = y;\
	double __tempZ__ = z;\
	double __h__ = __tempX__ * (matrix).m.m03 + __tempY__ * (matrix).m.m13 + __tempZ__ * (matrix).m.m23 + (matrix).m.m33;\
	x = __tempX__ * (matrix).m.m00 + __tempY__ * (matrix).m.m10 + __tempZ__ * (matrix).m.m20 + (matrix).m.m30;\
	y = __tempX__ * (matrix).m.m01 + __tempY__ * (matrix).m.m11 + __tempZ__ * (matrix).m.m21 + (matrix).m.m31;\
	z = __tempX__ * (matrix).m.m02 + __tempY__ * (matrix).m.m12 + __tempZ__ * (matrix).m.m22 + (matrix).m.m32;\
	if (__h__ != 1.0) {\
		if (__h__ != 0.0) __h__ = 1.0 / __h__;\
		x *= __h__;\
		y *= __h__;\
		z *= __h__;\
	}\
}
#define TRANSFORMRPOINTH(x, y, z, matrix) {\
	float __tempX__ = x;\
	float __tempY__ = y;\
	float __tempZ__ = z;\
	float __h__ = (float)(__tempX__ * (matrix).m.m03 + __tempY__ * (matrix).m.m13 + __tempZ__ * (matrix).m.m23 + (matrix).m.m33);\
	x = (float)(__tempX__ * (matrix).m.m00 + __tempY__ * (matrix).m.m10 + __tempZ__ * (matrix).m.m20 + (matrix).m.m30);\
	y = (float)(__tempX__ * (matrix).m.m01 + __tempY__ * (matrix).m.m11 + __tempZ__ * (matrix).m.m21 + (matrix).m.m31);\
	z = (float)(__tempX__ * (matrix).m.m02 + __tempY__ * (matrix).m.m12 + __tempZ__ * (matrix).m.m22 + (matrix).m.m32);\
	if (__h__ != 1.0f) {\
		if (__h__ != 0.0f) __h__ = 1.0f / __h__;\
		x *= __h__;\
		y *= __h__;\
		z *= __h__;\
	}\
}
#define TRANSFORMNORMAL(x, y, z, matrix) {\
	float __tempX__ = x;\
	float __tempY__ = y;\
	float __tempZ__ = z;\
	x = (float)(__tempX__ * (matrix).m.m00 + __tempY__ * (matrix).m.m01 + __tempZ__ * (matrix).m.m02);\
	y = (float)(__tempX__ * (matrix).m.m10 + __tempY__ * (matrix).m.m11 + __tempZ__ * (matrix).m.m12);\
	z = (float)(__tempX__ * (matrix).m.m20 + __tempY__ * (matrix).m.m21 + __tempZ__ * (matrix).m.m22);\
}

void CopyMatrix4To3(Matrix3Ptr theMatrix, Matrix4Ptr b);
void CopyRMatrix4To3(RMatrix3Ptr theMatrix, RMatrix4Ptr b);
void CopyMatrix3To4(Matrix4Ptr theMatrix, Matrix3Ptr b);
void CopyRMatrix3To4(RMatrix4Ptr theMatrix, RMatrix3Ptr b);
#define CopyMatrix4To4(dst, src) {\
	(dst)->m.m00 = (src)->m.m00;\
	(dst)->m.m01 = (src)->m.m01;\
	(dst)->m.m02 = (src)->m.m02;\
	(dst)->m.m03 = (src)->m.m03;\
	(dst)->m.m10 = (src)->m.m10;\
	(dst)->m.m11 = (src)->m.m11;\
	(dst)->m.m12 = (src)->m.m12;\
	(dst)->m.m13 = (src)->m.m13;\
	(dst)->m.m20 = (src)->m.m20;\
	(dst)->m.m21 = (src)->m.m21;\
	(dst)->m.m22 = (src)->m.m22;\
	(dst)->m.m23 = (src)->m.m23;\
	(dst)->m.m30 = (src)->m.m30;\
	(dst)->m.m31 = (src)->m.m31;\
	(dst)->m.m32 = (src)->m.m32;\
	(dst)->m.m33 = (src)->m.m33;\
}
#define CopyRMatrix4To4(dst, src) {\
	(dst)->m.m00 = (float)(src)->m.m00;\
	(dst)->m.m01 = (float)(src)->m.m01;\
	(dst)->m.m02 = (float)(src)->m.m02;\
	(dst)->m.m03 = (float)(src)->m.m03;\
	(dst)->m.m10 = (float)(src)->m.m10;\
	(dst)->m.m11 = (float)(src)->m.m11;\
	(dst)->m.m12 = (float)(src)->m.m12;\
	(dst)->m.m13 = (float)(src)->m.m13;\
	(dst)->m.m20 = (float)(src)->m.m20;\
	(dst)->m.m21 = (float)(src)->m.m21;\
	(dst)->m.m22 = (float)(src)->m.m22;\
	(dst)->m.m23 = (float)(src)->m.m23;\
	(dst)->m.m30 = (float)(src)->m.m30;\
	(dst)->m.m31 = (float)(src)->m.m31;\
	(dst)->m.m32 = (float)(src)->m.m32;\
	(dst)->m.m33 = (float)(src)->m.m33;\
}
#define CopyMatrix3To3(dst, src) {\
	(dst)->m.m00 = (src)->m.m00;\
	(dst)->m.m01 = (src)->m.m01;\
	(dst)->m.m02 = (src)->m.m02;\
	(dst)->m.m10 = (src)->m.m10;\
	(dst)->m.m11 = (src)->m.m11;\
	(dst)->m.m12 = (src)->m.m12;\
	(dst)->m.m20 = (src)->m.m20;\
	(dst)->m.m21 = (src)->m.m21;\
	(dst)->m.m22 = (src)->m.m22;\
}
#define CopyRMatrix3To3(dst, src) {\
	(dst)->m.m00 = (float)(src)->m.m00;\
	(dst)->m.m01 = (float)(src)->m.m01;\
	(dst)->m.m02 = (float)(src)->m.m02;\
	(dst)->m.m10 = (float)(src)->m.m10;\
	(dst)->m.m11 = (float)(src)->m.m11;\
	(dst)->m.m12 = (float)(src)->m.m12;\
	(dst)->m.m20 = (float)(src)->m.m20;\
	(dst)->m.m21 = (float)(src)->m.m21;\
	(dst)->m.m22 = (float)(src)->m.m22;\
}

#define COPYMATRIX4TO4 CopyMatrix4To4
#define COPYRMATRIX4TO4 CopyRMatrix4To4
#define COPYMATRIX3TO3 CopyMatrix3To3
#define COPYRMATRIX3TO3 CopyRMatrix3To3
#define COPYMATRIX3TO4(dst, src) {\
	(dst)->m.m00 = (src)->m.m00;\
	(dst)->m.m01 = (src)->m.m01;\
	(dst)->m.m02 = (src)->m.m02;\
	(dst)->m.m03 = 0.0;\
	(dst)->m.m10 = (src)->m.m10;\
	(dst)->m.m11 = (src)->m.m11;\
	(dst)->m.m12 = (src)->m.m12;\
	(dst)->m.m13 = 0.0;\
	(dst)->m.m20 = (src)->m.m20;\
	(dst)->m.m21 = (src)->m.m21;\
	(dst)->m.m22 = (src)->m.m22;\
	(dst)->m.m23 = 0.0;\
	(dst)->m.m30 = 0.0;\
	(dst)->m.m31 = 0.0;\
	(dst)->m.m32 = 0.0;\
	(dst)->m.m33 = 1.0;\
}
#define COPYRMATRIX3TO4(dst, src) {\
	(dst)->m.m00 = (float)(src)->m.m00;\
	(dst)->m.m01 = (float)(src)->m.m01;\
	(dst)->m.m02 = (float)(src)->m.m02;\
	(dst)->m.m03 = 0.0f;\
	(dst)->m.m10 = (float)(src)->m.m10;\
	(dst)->m.m11 = (float)(src)->m.m11;\
	(dst)->m.m12 = (float)(src)->m.m12;\
	(dst)->m.m13 = 0.0f;\
	(dst)->m.m20 = (float)(src)->m.m20;\
	(dst)->m.m21 = (float)(src)->m.m21;\
	(dst)->m.m22 = (float)(src)->m.m22;\
	(dst)->m.m23 = 0.0f;\
	(dst)->m.m30 = 0.0f;\
	(dst)->m.m31 = 0.0f;\
	(dst)->m.m32 = 0.0f;\
	(dst)->m.m33 = 1.0f;\
}
#define COPYMATRIX4TO3 CopyMatrix3To3
#define COPYRMATRIX4TO3 CopyRMatrix3To3

/* Quaternion Routines --- Q = A * i + B * j + C * k + R = q0 * i + q1 * j + q2 * k + q3 */

void AddQuaternion(QuaternionPtr p, QuaternionPtr q, QuaternionPtr r);
void SubtractQuaternion(QuaternionPtr p, QuaternionPtr q, QuaternionPtr r);
void MultiplyQuaternion(QuaternionPtr p, QuaternionPtr q, QuaternionPtr r);
void DivideQuaternion(QuaternionPtr p, QuaternionPtr q, QuaternionPtr r);
void ConjugateQuaternion(QuaternionPtr q, QuaternionPtr r);
void InvertQuaternion(QuaternionPtr q, QuaternionPtr r);
void ExponentiateQuaternion(QuaternionPtr q, QuaternionPtr r);
void LogarithmQuaternion(QuaternionPtr q, QuaternionPtr r);
void SquareRouteQuaternion(QuaternionPtr q, QuaternionPtr r);
void SquareQuaternion(QuaternionPtr q, QuaternionPtr r);
void ScaleQuaternion(double c, QuaternionPtr q, QuaternionPtr r);
double DotQuaternion(QuaternionPtr p, QuaternionPtr q);
void DoubleQuaternion(QuaternionPtr p, QuaternionPtr q, QuaternionPtr r);
void BisectQuaternion(QuaternionPtr p, QuaternionPtr q, QuaternionPtr r);
void NormalizeQuaternion(QuaternionPtr q, QuaternionPtr r);
void LinearInterpolateQuaternion(double t, QuaternionPtr p, QuaternionPtr q, QuaternionPtr r);
void BezierInteroplateQuaternion(double t, QuaternionPtr q0, QuaternionPtr q1, QuaternionPtr q2, QuaternionPtr q3, QuaternionPtr r);
void Quaternion2Matrix4(QuaternionPtr theQuaternion, Matrix4Ptr theMatrix);
void Matrix42Quaternion(Matrix4Ptr theMatrix, QuaternionPtr theQuaternion);
void QuaternionToMatrix4(QuaternionPtr theQuaternion, Matrix4Ptr theMatrix);
void Matrix4ToQuaternion(Matrix4Ptr theMatrix, QuaternionPtr theQuaternion);
void Matrix4ToAngles(Matrix4Ptr theMatrix, double* x, double* y, double* z);
void RMatrix4ToAngles(RMatrix4Ptr theMatrix, double* x, double* y, double* z);

#ifdef __cplusplus
}
#endif

#endif
