/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __Noise_h__
#define __Noise_h__

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __System_h__
#include "System.h"
#endif

#define noisePoints		4096	/* Fits inside 16K L1 cache */
#define noiseMask		(noisePoints - 1)

/* Noise global variables */

extern float *noiseData;

/* Random number generator */
/* seed must be an 32-bits unsigned long integer */

#define RANDOMSEED(seed) ((seed) = (UInt32)((SInt32)(seed) * 1103515245 + 12345))	/* pick a new random seed */
#define RANDOMBITS(seed, bits) (RANDOMSEED(seed) >> (32 - (bits)))					/* create a random number with a specific number of bits */
#define ROUNDINGBITS(bits) (1 << ((bits) - 1))									/* create a rounding number with a specific number of bits */
#define RANDOMNUM(seed) (RANDOMSEED(seed) * 2.328306436E-10f)						/* yields a number between 0 and <1 */
#define RANDOMFACTOR(seed) ((SInt32)(RANDOMSEED(seed)) * 4.656612873E-10f)			/* yields a number between -1 and 1 */
#define RANDOMFACTOR2(seed) ((SInt32)(RANDOMSEED(seed)) * 2.328306436E-10f)			/* yields a number between -0.5 and 0.5 */

/* InitNoise must be called before the following macros are used */

#define RANDOMNOISE(seed) (noiseData[RANDOMSEED(seed) & (noisePoints - 1)])	/* yields a number between 0 and <1 */
#define RANDOMTABLE(seed) (noiseData[(++(seed)) & (noisePoints - 1)])		/* yields a number between 0 and <1 */

typedef struct {
	UInt32 index;
	float vx, vy, vz;
} CellularDotData;

typedef struct {
	SInt32 cellXI, cellYI, cellZI;
	UInt32 cellDotCount;
	CellularDotData cellDotList[243];		/* Maximum of 243 (3x3x3x9) dots */
} CellularDotInfo;

typedef struct {
	UInt32 index;
	float distance;
	float vx, vy, vz;
} CellularData;

typedef void *(*NoiseMallocFunction)(ulong theSize);
typedef void (*NoiseFreeFunction)(void *thePtr);

/* Random number functions */

short MacRandom(UInt32 *randomSeed);

/* Perlin noise basis functions */

SInt32 InitNoise(NoiseMallocFunction noiseMalloc, UInt32 *noiseSeed);
void FinishNoise(NoiseFreeFunction noiseFree);
float Noise1D(float x);
float Noise1D2(float x);
float Noise2D(float x, float y);
float Noise2D2(float x, float y);
void DNoise2D(float x, float y, float* aNum, float* bNum);
float DNoise2D2(float x, float y, float* aNum, float* bNum);
float Noise(float x, float y, float z);
float Noise2(float x, float y, float z);
float Noise4D(float x, float y, float z, float t);
float Noise4D2(float x, float y, float z, float t);
void DNoise(float x, float y, float z, float* aNum, float* bNum, float* cNum);
float DNoise2(float x, float y, float z, float* aNum, float* bNum, float* cNum);

/* Cellular noise basis functions */

SInt32 InitCellularNoise(NoiseMallocFunction noiseMalloc, UInt32 *noiseSeed);
void FinishCellularNoise(NoiseFreeFunction noiseFree);
void SetupCellularDots(CellularDotInfo *theInfo);
void DoCellularDots(CellularDotData *f, UInt32 *n, SInt32 xi, SInt32 yi, SInt32 zi, float xr, float yr, float zr);
void BuildCellularDots(CellularDotInfo *theInfo, float x, float y, float z);
void CellularNoise(CellularData *f, UInt32 n, float xr, float yr, float zr);
void CellularNoise2D(CellularData *f, UInt32 n, float xr, float yr);

/* Fractal texture routines */

float FractalNoise(float x, float y, float z, float noiseFactor, float noiseScale, SInt32 detail1, SInt32 detail2);
float FractalNoise2D(float x, float y, float noiseFactor, float noiseScale, SInt32 detail1, SInt32 detail2);
float CellularFractalNoise(float x, float y, float z, float xOff, float yOff, float zOff, float noiseFactor, float noiseScale, SInt32 detail1, SInt32 detail2);
float CellularFractalNoise2D(float x, float y, float xOff, float yOff, float noiseFactor, float noiseScale, SInt32 detail1, SInt32 detail2);
float MarbleNoise(float x, float y, float z, float noiseFactor, float offset, float turbulence, float scale, SInt32 detail1, SInt32 detail2);
float MarbleNoise2D(float x, float y, float noiseFactor, float offset, float turbulence, float scale, SInt32 detail1, SInt32 detail2);
SInt32 Mandelbrot(double xPos, double yPos, SInt32 crunch);

#ifdef __cplusplus
}
#endif

#endif
