/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __PString_h__
#define __PString_h__

#ifdef __cplusplus
extern "C" {
#endif

#include "System.h"

typedef unsigned char PStr255[256];
typedef unsigned char PStr63[64];
typedef unsigned char PStr31[32];
typedef unsigned char *PStrPtr;
typedef char *CStrPtr;

#define PStrLength(PStr)			(*(PStrPtr)(PStr))
#define PStrSetLength(PStr, length)	(*(PStrPtr)(PStr) = (unsigned char)(length))
#define PStrData(PStr)				((PStrPtr)(PStr) + 1)
#define PStrSize(PStr)				(PStrLength(PStr) + 1)
#define PStrEmpty(PStr)			(PStrLength(PStr) == 0)
#define PStrNotEmpty(PStr)			(PStrLength(PStr) != 0)
#define PStrMakeEmpty(PStr)		(PStrSetLength(PStr, 0))
#define PStrEq(a, b)				(PStrCompare(a, b) == 0)
#define PStrNe(a, b)				(PStrCompare(a, b) != 0)

#define applPool	0				/* PString from Application */
#define utilPool	1				/* PString from Utils Libs */

#define PStrNULL	0L				/* Termination value for PStrConcat */

/* PString global variables */

extern unsigned char aEmptyPStr[1];

/* Return the length of a C string */

UInt32 CStrLength(const char *theString);

/* Return the position of ch in theString, return -1 if ch is not found */

SInt32 CStrIn(unsigned char ch, const char *theString);

/* Copy src C string to dst C string */

void CStrCpy(char *dst, const char *src);

/* Return a pointer to the next PStr255 in the PString queue, do not use directly in apps */

PStrPtr PStrNext(void);

/* Convert between Pascal and C strings inline, DO NOT use with string constants */

PStrPtr PStrC2P(CStrPtr CStr);
CStrPtr PStrP2C(PStrPtr PStr);

/* Convert between Pascal and C strings by copying, if dest string is nil use queue */

PStrPtr PStrCToP(CStrPtr CStr, PStrPtr PStr);
CStrPtr PStrPToC(PStrPtr PStr, CStrPtr CStr);

/* Convert a C string to a Pascal string and return it as a new string pointer (may be used with constants) */

PStrPtr PS(const char *CStr);

/* Convert a Pascal string to a C string and return it as a new string pointer */

CStrPtr CS(PStrPtr PStr);

/* Convert an IFFType to a string */

PStrPtr PStrType(UInt32 theType);

/* Convert a long integer into a string */

PStrPtr PStrNum(slong theNum);

/* Compare to Pascal strings, 0 if a == b, -1 if a < b, +1 if a > b */

SInt32 PStrCompare(PStrPtr a, PStrPtr b);

/* Copy src Pascal string to dst Pascal string, if dest string is nil use queue */

PStrPtr PStrCpy(PStrPtr dst, PStrPtr src);

/* Concatenate src Pascal string to then end of dst Pascal string */

PStrPtr PStrAppend(PStrPtr dst, PStrPtr src);

/* Append a single character to the end of a Pascal string */

PStrPtr PStrAppendChar(PStrPtr theStr, unsigned char theChar);

/* Concatenate a variable number of strings together, nil is list terminator */

PStrPtr PStrConcat(PStrPtr firstStr, ...);

/* Return the position of ch in theString, return zero if ch is not found */

UInt32 PStrIn(unsigned char ch, PStrPtr theString);

/* Find the substring with the string, return the index of the first character or zero */

UInt32 PStrPos(PStrPtr subStr, PStrPtr theString);

/* Return the substring count characters from position source[index] */

PStrPtr PStrSub(PStrPtr source, UInt32 index, UInt32 count);

/* Remove count chars from dest beginning at dest[index] */

void PStrDelete(PStrPtr dest, UInt32 index, UInt32 count);

/* Same as PStrDelete execpt that str is not affected */

PStrPtr PStrOmit(PStrPtr str, UInt32 index, UInt32 count);

/* Insert source into dest starting at dest[index] */

void PStrInsert(PStrPtr source, PStrPtr dest, UInt32 index);

/* Same as PStrInsert execpt that str is not affected */

PStrPtr PStrInclude(PStrPtr source, PStrPtr str, UInt32 index);

/* Convert a long to a string */

PStrPtr PStrLongStr(SInt32 num);

/* Convert an unsigned long to a string */

PStrPtr PStrULongStr(ulong num);

/* Convert a character to a string */

PStrPtr PStrCharStr(unsigned char theChar);

/* Convert a floating point number to a string */

PStrPtr PStrFloatStr(double theNum, SInt32 theDigits);

/* Convert an intenger number into a string with commas */

PStrPtr PStrNumericalStr(SInt32 theNum);

/* Convert a time in seconds into a human readable string */

PStrPtr PStrTimeStr(SInt32 seconds);

/* Convert a standard C date into a human readable string */

PStrPtr PStrDateStr(ulong theSecond, ulong curSecond);

/* Convert an unsigned memory size into a human readable string */

PStrPtr PStrKStr(long long theNum);

/* Convert a signed memory size into a human readable string */

PStrPtr PStrKSStr(long long theNum);

/* Convert a string to upper case */

void PStrUpperCase(PStrPtr theString);
PStrPtr PStrUpCase(PStrPtr theString);

/* Strip all of the spaces from a string */

void PStrStrip(PStrPtr theString);

/* Return a pointer to a C string from a Pascal string, for printing with %s in printf */

CStrPtr PrintPStr(PStrPtr PStr);

/* Return a pointer to a C string containing 1 or 0, for printing with %s in printf */

CStrPtr PrintBoolean(unsigned char b);

/* Create a IFF/OS/ResType, a long via 4 chars, from a C string */

UInt32 MakeType(CStrPtr str);

/* Append a suffix to file name, extension is preserved */

PStrPtr AppendSuffix2FileName(PStrPtr theName, PStrPtr theSuffix);

#ifdef __cplusplus
}
#endif

#endif
