/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __PluginUtils_h__
#define __PluginUtils_h__

#ifndef __FACTStuff_h__
#include "FACTStuff.h"
#endif

#ifndef __PString_h__
#include "PString.h"
#endif

#ifndef __System_h__
#include "System.h"
#endif

#ifndef EI_TYPES_H
#include "EI_Types.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define anyCreator				'\?\?\?\?'
#define anyType					'\?\?\?\?'

#define errorDialogID			9000
#define warningDialogID			9002
#define messageDialogID			9003
#define cautionDialogID			9004

extern boolean errorFlag, hideErrorFlag, inBackground;
/* Type Conversion Utilities */

double FixDouble(double theValue);
PStrPtr CharStr(uchar theChar);
PStrPtr NumStr(SInt32 theNum);
double Str2Num(PStrPtr s);
void Num2Str(SInt32 digits, double x, PStrPtr s);
PStrPtr RealTrimStr(double theNum, short theDigits);
int Str2Int(PStrPtr s);

/* Variable Setting Routines */
void SetFlag(UInt32* theFlagFields, SInt32 theFlag, boolean theState);
void SetIntegerFlag(ushort* theFlagFields, SInt32 theFlag, boolean theState);
void SetBooleanFlag(UInt32* dst, SInt32 theFlag, boolean src, boolean* flag);
void SetBoolean(boolean* dst, boolean src, boolean* flag);
void SetByte(char* dst, short src, boolean* flag);
void SetInteger(short* dst, short src, boolean* flag);
void SetLongInt(SInt32* dst, SInt32 src, boolean* flag);
void SetReal(float* dst, float src, boolean* flag);
void SetDouble(double* dst, double src, boolean* flag);
void SetExtended(double* dst, double src, boolean* flag);
void SetRGBColor(RGB_Color* dst, RGB_Color* src, boolean* flag);
void SetTheARGB(ARGB* dst, ARGB* src, boolean* flag);

/* Event Utilities */

#if 0
boolean OptionKeyDown(void);
boolean CheckForAbort(void);
#endif

/* Window And Dialog Utilities */

EI_ResourceFile *OpenResourceFile(const char *iResourceFileName);
void SetColorButtonIDARGB(const EI_Dialog *iDialog,
							int iControlID,
							const ARGB* iColor);
void GetColorButtonIDARGB(const EI_Dialog *iDialog,
							int iControlID,
							ARGB* oColor);
void SetUserControlIDFunctions(EI_Dialog *iDialog,
								int iUserControlID,
								EI_UserDrawFunction iDraw,
								EI_UserClickFunction iClick);
void SetControlIDHitFunction(EI_Dialog *iDialog,
								int iControlID,
								EI_ControlHitFunction iFunction);
void SetControlIDVisible(EI_Dialog *iDialog,
							int iControlID,
							int iVisible);
void SetControlIDTitle(EI_Dialog *iDialog,
						int iControlID,
						const char *iTitle);
void SetEditTextIDString(EI_Dialog *iDialog,
							int iControlID,
							const char *iString);

/* Old Fashioned Functions (Deprecated, will go away): */

void SetButton(const EI_Dialog *theDialog, int theItemID, int theValue);
int GetButton(const EI_Dialog *theDialog, int theItemID);
int ToggleButton(const EI_Dialog *theDialog, int theItemID);
void SetButtonName(const EI_Dialog *theDialog, int theItemID, PStrPtr theName);
PStrPtr GetButtonName(const EI_Dialog *theDialog, int theItemID, PStrPtr theName);
void SelectItem(const EI_Dialog *theDialog, int theItemID);
void SetTheIText(const EI_Dialog *theDialog, int theItemID, PStrPtr theText);
void GetTheIText(const EI_Dialog *theDialog, int theItemID, PStrPtr theText);
void SetTheINum(const EI_Dialog *theDialog, int theItemID, SInt32 theNum);
SInt32 GetTheINum(const EI_Dialog *theDialog, int theItemID);
void SetTheIReal(const EI_Dialog *theDialog, int theItemID, double theNum, short theDigits);
double GetTheIReal(const EI_Dialog *theDialog, int theItemID);
void SetTheIPopUp(const EI_Dialog *theDialog, int theItemID, short theValue);
short GetTheIPopUp(const EI_Dialog *theDialog, int theItemID);
EI_ControlType GetTheIType(const EI_Dialog *theDialog, int theItemID);
void SetTheIRect(const EI_Dialog *theDialog, int theItemID, const EI_Rect *theRect);
void GetTheIRect(const EI_Dialog *theDialog, int theItemID, EI_Rect *theRect);

/* Error Utilities */

void Error(EI_ResourceFile *iResourceFile, PStrPtr theMessage, SInt32 theErr);
void StandardError(EI_ResourceFile *iResourceFile, PStrPtr theMessage);
void Stop(EI_ResourceFile *iResourceFile, PStrPtr theMessage);
void Caution(EI_ResourceFile *iResourceFile, PStrPtr theMessage);
void Message(EI_ResourceFile *iResourceFile, PStrPtr theMessage);
int Warning(EI_ResourceFile *iResourceFile, PStrPtr theMessage);
int StandardDialog(EI_ResourceFile *iResFile, short iDialogID,
					const unsigned char* iText0,
					const unsigned char* iText1, const unsigned char* iText2,
					const unsigned char* iText3);

/* UtilLibs Setup */

void InitUtils(void);
void FinishUtils(void);
#ifdef __cplusplus
}
#endif

#endif
