/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __RoundUtils_h__
#define __RoundUtils_h__

#ifdef __cplusplus
extern "C" {
#endif

#include "PlatformTypes.h"

#define LIMIT01(n)		((n) <= 0.0f ? 0.0f : (n) >= 1.0f ? 1.0f : (n))
#define INTEGERLIMIT(a) ((a) < -32767 ? -32767 : ((a) > 32767 ? 32767 : (a)))

//extern double RU_Magic;

#ifdef __LITTLE_ENDIAN__
#define LONGTOFLOAT(d, n) {\
	UInt32 _temp[2];\
	_temp[0] = (UInt32)(n);\
	_temp[1] = 0x43380000 - (_temp[0] >> 31);\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* LONGTOFLOAT */

#define ULONGTOFLOAT(d, n) {\
	UInt32 _temp[2];\
	_temp[0] = (UInt32)(n);\
	_temp[1] = 0x43380000;\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* ULONGTOFLOAT */
#else
#define LONGTOFLOAT(d, n) {\
	UInt32 _temp[2];\
	_temp[1] = (UInt32)(n);\
	_temp[0] = 0x43380000 - (_temp[1] >> 31);\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* LONGTOFLOAT */

#define ULONGTOFLOAT(d, n) {\
	UInt32 _temp[2];\
	_temp[1] = (UInt32)(n);\
	_temp[0] = 0x43380000;\
	(d) = *((double*)_temp) - 6755399441055744.0;\
}  /* ULONGTOFLOAT */
#endif

#ifdef __ROUNDTOFLOOR__

/* Automaticly rounding towards negative infinity */

SInt32 Floor(double n);
SInt32 Floor2(double* n);
#define FLOOR(n) ((SInt32)(n))

SInt32 Ceil(double n);
SInt32 Ceil2(double* n);
#define CEIL(d, n) (-(SInt32)(-(n)))

SInt32 CalcFactor(double n);
SInt32 CalcFactor2(double* n);
#define CALCFACTOR(n) ((SInt32)((n) * 1024.0) & 1023)
	
SInt32 CalcLFactor(double n);
SInt32 CalcLFactor2(double* n);
#define CALCLFACTOR(n) ((SInt32)((n) * 1024.0) & 16383)

double CalcRFactor(double n);
double CalcRFactor2(double* n);
void CalcRFactor3(double* n);
#define CALCRFACTOR(d, n) ((n) - (SInt32)(n))

SInt32 SplitFactor(double* n);
#define SPLITFACTOR(d, n) {\
	d = n;\
	n -= d;\
}

SInt32 ARound(double n);
SInt32 ARound2(double* n);
#define AROUND(n) ((SInt32)((n) + 0.5))

SInt32 ARoundI(double n);
SInt32 ARoundI2(double* n);
#define AROUNDI(d, n) {\
	SInt32 _TEMP_ = (SInt32)((n) + 0.5);\
	d = INTEGERLIMIT(_TEMP_);\
}

SInt32 LRound(double n);
SInt32 LRound2(double* n);
#define LROUND(n) ((SInt32)((n) * 65536.0))

SInt32 IRound(double n);
SInt32 IRound2(double* n);
#define IROUND(d, n) {\
	SInt32 _TEMP_ = (n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}

SInt32 IRoundR(float n);
SInt32 IRoundR2(float* n);
#define IROUNDR(d, n) {\
	SInt32 _TEMP_ = (float)(n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}
	
SInt32 ITrunc(double n);
SInt32 ITrunc2(double* n);
#define ITRUNC(d, n) {\
	SInt32 _TEMP_ = (n);\
	d = INTEGERLIMIT(d);\
}

SInt32 FTrunc(double n);
SInt32 FTrunc2(double* n);
#define FTRUNC(d, n) (d = (SInt32)((n) * 1024.0))

SInt32 TruncLo(double n);
SInt32 TruncLo2(double* n);
#define TRUNCLO(d, n) {double _n_ = (n) + 1.0; d = FLOOR(_n_)}

SInt32 TruncHi(double n);
SInt32 TruncHi2(double* n);
#define TRUNCHI FLOORF

#else

SInt32 Floor(double n);
SInt32 Floor2(double* n);
#define FLOOR(n) ((n) >= 0.0 ? (SInt32)(n) : (SInt32)((double)(n) - 0.999999999))

SInt32 Ceil(double n);
SInt32 Ceil2(double* n);
#define CEIL(n) ((n) < 0.0 ? (SInt32)(n) : (SInt32)((double)(n) + 0.999999999));

SInt32 CalcFactor(double n);
SInt32 CalcFactor2(double* n);
#define CALCFACTOR(n) ((SInt32)((n) * 1024.0) & 1023)
	
SInt32 CalcLFactor(double n);
SInt32 CalcLFactor2(double* n);
#define CALCLFACTOR(n) ((SInt32)((n) * 16384.0) & 16383)

double CalcRFactor(double n);
double CalcRFactor2(double* n);
void CalcRFactor3(double* n);
#define CALCRFACTOR(n) ((n) - FLOOR(n))

SInt32 SplitFactor(double* n);
#define SPLITFACTOR(d, n) {\
	d = FLOOR(n);\
	n -= d;\
}

/* ARoundI(n) == (SInt32)floor(n + 0.5) */

SInt32 ARound(double n);
SInt32 ARound2(double* n);
#define AROUND(n) ((n) < 0.0 ? (SInt32)((n) - 0.5) : (SInt32)((n) + 0.5))

/* ARoundI(n) == (short)floor(n + 0.5) */

SInt32 ARoundI(double n);
SInt32 ARoundI2(double* n);
#define AROUNDI(d, n) {\
	if (n < 0.0) {\
		SInt32 _TEMP_ = n - 0.5;\
		if (_TEMP_ <= -32767)\
			d = -32767;\
		else\
			d = _TEMP_;\
	} else {\
		SInt32 _TEMP_ = n + 0.5;\
		if (_TEMP_ >= 32767)\
			d = 32767;\
		else\
			d = _TEMP_;\
	} \
}

/* IRound(n) == (SInt32)(n * 65536.0) */

SInt32 LRound(double n);
SInt32 LRound2(double* n);
#define LROUND(n) ((SInt32)((n) * 65536.0))

/* IRound(n) == (short)(n * 16384.0) */

SInt32 IRound(double n);
SInt32 IRound2(double* n);
#define IROUND(d, n) {\
	SInt32 _TEMP_ = (n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}

/* IRoundR(n) == (short)((float)n * 16384.0) */

SInt32 IRoundR(float n);
SInt32 IRoundR2(float* n);
#define IROUNDR(d, n) {\
	SInt32 _TEMP_ = (float)(n) * 16384.0;\
	d = INTEGERLIMIT(_TEMP_);\
}

/* ITrunc(n) == (short)floor(n) */

SInt32 ITrunc(double n);
SInt32 ITrunc2(double* n);
#define ITRUNC(d, n) {\
	if (n <= -32767.0)\
		d = -32767.0;\
	else if (n >= 32767.0)\
		d = 32767.0;\
	else\
		d = FLOOR(n);\
}

/* FTrunc(n) == (SInt32)floor(n * 1024.0) */

SInt32 FTrunc(double n);
SInt32 FTrunc2(double* n);
#define FTRUNC(d, n) {\
	double _T_ = (n) * 1024.0;\
	d = FLOOR(_T_);\
}

/* TruncLo(n) == (SInt32)floor(n + 1.0) */

SInt32 TruncLo(double n);
SInt32 TruncLo2(double* n);
#define TRUNCLO(d, n) {SInt32 _n_ = (SInt32)(n); d = (_n_ != MAXINT) && (_n_ <= (n)) ? _n_ + 1 : _n_;}

/* TruncHi(n) == (SInt32)floor(n) */

SInt32 TruncHi(double n);
SInt32 TruncHi2(double* n);
#define TRUNCHI(d, n) {SInt32 _n_ = (SInt32)(n); d = (_n_ != MININT) && (_n_ > (n)) ? _n_ - 1 : _n_;}

#endif

#ifdef __cplusplus
}
#endif

#endif
