/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#ifndef __System_h__
#define __System_h__

#ifdef __cplusplus
extern "C" {
#endif

/*
 *		User settable #defines
 *
 *		OS Selection:
 *		define __MACINTOSH__ if running under the Macintsoh OS
 *		define __SGI__ if running on the SGI OS
 *		define __SUN__ if running on the Sun Solaris Unix
 *		define __WIN32__ if running on Windows NT/Windows 95
 *		define __WIN64__ if running on Windows 64
 *
 *		Determined from the OS Selection:
 *		define __UNIX_BSD__ if running under vanilla BSD 4.X Unix
 *		define __UNIX_SYSV__ if running under vanilla System V Unix
 *		define __LITTLE_ENDIAN__ if running on a little endian processor
 *		define __EXTENDED8__ if a long double is 8 bytes, IEEE double precision
 *		define __EXTENDED10__ if a long double is 10 bytes, IEEE extended precision
 *		define __IEEE__ if processor uses IEEE floating point format
 */

#include "PlatformTypes.h"

/*
 * Standard defines for the Mac
 */

#ifdef  __MACINTOSH__
	#define __EXTENDED8__
	#define __IEEE__
	#define alignSize 8
#if !__LP64__	
	#define cacheAlignSize 32
#else	
	#define cacheAlignSize 48
#endif	
	#ifndef __QUICKTIME__
		#define __QUICKTIME__ 1
	#endif
	#ifndef PATH_SEPERATOR
		#ifdef MACH_O
			#define PATH_SEPERATOR '/'
		#else
			#define PATH_SEPERATOR ':'
		#endif
	#endif

#elif defined __SGI__

/*
 * Standard defines for the SGI
 */
 
	#define __UNIX_SYSV__
	#define __EXTENDED8__
	#define __IEEE__
	#define alignSize 8
	#define cacheAlignSize 32
	#ifndef __QUICKTIME__
		#define __QUICKTIME__ 0
	#endif
	#ifndef PATH_SEPERATOR
		#define PATH_SEPERATOR '/'
	#endif

#elif defined __SUN__

	/*
	 * Standard defines for the Sun
	 */

	#define __UNIX_SYSV__
	#define __EXTENDED8__
	#define __IEEE__
	#define alignSize 8
	#define cacheAlignSize 32
	#ifndef __QUICKTIME__
		#define __QUICKTIME__ 0
	#endif
	#ifndef PATH_SEPERATOR
		#define PATH_SEPERATOR '/'
	#endif

#elif defined(__WIN32__) || defined(__WIN64__) 

	/*
	 * Standard defines for Windows
	 */

	#define __EXTENDED8__
	#define __IEEE__
	#define __LITTLE_ENDIAN__
	#define alignSize 8
	#define cacheAlignSize 32
	#ifndef __QUICKTIME__
		#define __QUICKTIME__ 0
	#endif
	#ifndef PATH_SEPERATOR
		#define PATH_SEPERATOR '\\'
	#endif

#else

	/*
	 * Standard defines for generic ANSI
	 */

	#define __EXTENDED8__
	#define __IEEE__
	#define alignSize 8
	#define cacheAlignSize 32
	#ifndef __QUICKTIME__
		#define __QUICKTIME__ 0
	#endif
	#ifndef PATH_SEPERATOR
		#define PATH_SEPERATOR '/'
	#endif
	
#endif

/*
 *		End of user settable #defines, the rest should auto configure
 */

#if defined(__UNIX_BSD__) || defined(__UNIX_SYSV__)
	#define __UNIX__
#else
	#undef __UNIX__
#endif

/*
 *		nil pointer
 */
#ifndef nil
	#define nil	0UL
#endif

/*
*		Ignore a parameter to avoid a warning
*/

#define Ignore(a)			(a)

/*
*		And, or, exclusive or, and not bit field macros
*/
#define BAND(a, b)			((a) & (b))
#define BOR(a, b)				((a) | (b))
#define BXOR(a, b)			((a) ^ (b))
#define BNOT(a)				(!(a))
#define BIT(a)				(1UL << (a))
#define BITMASK(a)			(0xFFFFFFFF >> (32 - (a)))

/*
 *		Set, clear, and test a bit in a variable, bit numbers start at 0
 */
#define BSET(var, bitNum)	((var) |= BIT(bitNum))
#define BCLR(var, bitNum)	((var) &= ~BIT(bitNum))
#define BTST(var, bitNum)	(((var) & BIT(bitNum)) != 0)
#define SETFLAG(theFlagFields, theFlag, theState) ((theState) ? BSET(theFlagFields, theFlag) : BCLR(theFlagFields, theFlag))

/*
 *		Bit shift left and right, promote up to a long, and do not sign extend
 */
#define BSR(var, count)		((SInt32)((UInt32)(var) >> (count)))
#define BSL(var, count)		((SInt32)((UInt32)(var) << (count)))

/*
 *		Extract the low and high short from a long
 */
#define LO_WORD(theValue)	((short)(theValue))
#define HI_WORD(theValue)	((short)((UInt32)theValue >> 16))

/*
 *		Unsafe macros, beware of side affects
 */
#define SQR(a)				((a) * (a))
#define ABS(a)				((a) < 0 ? -(a) : (a))
#define SGN(a)				((a) < 0 ? -1 : +1)
#ifndef MIN
	#define MIN(a, b)			((a) < (b) ? (a) : (b))
#endif
#define MIN3(a, b, c)			((a) < (b) ? ((a) < (c) ? (a) : (c)) : ((b) < (c) ? (b) : (c)))
#define MIN4(a, b, c, d)		((a) < (b) ? ((a) < (c) ? ((a) < (d) ? (a) : (d)) : ((c) < (d) ? (c) : (d))) : ((b) < (c) ? ((b) < (d) ? (b) : (d)) : ((c) < (d) ? (c) : (d))))
#ifndef MAX
	#define MAX(a, b)			((a) > (b) ? (a) : (b))
#endif
#define MAX3(a, b, c)			((a) > (b) ? ((a) > (c) ? (a) : (c)) : ((b) > (c) ? (b) : (c)))
#define MAX4(a, b, c, d)		((a) > (b) ? ((a) > (c) ? ((a) > (d) ? (a) : (d)) : ((c) > (d) ? (c) : (d))) : ((b) > (c) ? ((b) > (d) ? (b) : (d)) : ((c) > (d) ? (c) : (d))))
#define ODD(a)				(((a) & 1L) == 1)
#define MIX(a, b, f)			((a) + (f) * ((b) - (a)))
#define REVERSELONG(a)			(((a) << 24) | (((a) & 0xFF00) << 8) | (((a) & 0xFF0000) >> 8) | ((a) >> 24))


/*
	QUADWORD is deprecated, use QUADBYTE (below) instead.
	
	Do not use 'abcd'; it produces implementation specific results.
*/
#ifdef __SUN__
#define QUADWORD(a) REVERSELONG(a)
#else
#define QUADWORD(a) (a)
#endif

#define QUADBYTE(a, b, c, d) \
		(((UInt32) ((a) & 0xFF) << 24) | \
		((UInt32) ((b) & 0xFF) << 16) | \
		((UInt32) ((c) & 0xFF) << 8) | \
		((UInt32) ((d) & 0xFF)))

/*
 *		A safe version of ABS which will not recompute (b)
 */
 
#define ABS2(a, b)			{if ((SInt32)(a = (b)) < 0) a = -(SInt32)a;}

/*
 *		ABORT
 *		Abort into the debugger
 */
#ifdef __MACINTOSH__
	#define EIABORT	Debugger()
#else
	#define	EIABORT	abort()
#endif

/* Fundamental data types */

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef uchar boolean;

/* Numeric constants */

#ifndef MAXINT
	#define MAXINT 0x7FFFFFFF
	#define MININT 0x80000000
#endif

void ReverseField(void* theField, SInt32 size);
#define FixField(a) ReverseField(&a, sizeof(a))
#ifdef __LITTLE_ENDIAN__
	#define FIXFIELD(a) FixField(a)
#else
	#define FIXFIELD(a)
#endif
#define UMCopyMem(sourcePtr, destPtr, byteCount) memmove(destPtr, sourcePtr, byteCount)
#define CopyField(src, dst) UMCopyMem(src, &dst, sizeof(dst))
#define ReverseLong(a) (((UInt32)(a) >> 24) | (((UInt32)(a) >> 8) & 0x0000FF00) | (((UInt32)(a) << 8) & 0x00FF0000) | ((UInt32)(a) << 24))
#define ALIGN(theSize, theAlignment) ((((theSize) + ((theAlignment) - 1)) / (theAlignment)) * (theAlignment))
#define FixAlignment(theSize) ALIGN(theSize, alignSize)

#ifdef __MACINTOSH__
	#include <DateTimeUtils.h>
#if		TARGET_API_MAC_CARBON
	#include <Events.h>
	#define TICKCOUNT TickCount
#else
	#include <LowMem.h>
	#define TICKCOUNT LMGetTicks
#endif
	#define GETDATETIME GetDateTime
#elif defined __SGI__
	extern UInt32 gTickCount;
	#ifdef __DEBUG__
        #define TICKCOUNT() (clock() * (60.0 / CLOCKS_PER_SEC))
	#else
		#define TICKCOUNT() gTickCount
	#endif
	#define GETDATETIME(secs) (*(secs) = 0)
#elif defined __WIN32__
//	#define TICKCOUNT EIMacTickCount
//	#define GETDATETIME EIMacGetDateTime
	#define TICKCOUNT() 0
	#define GETDATETIME(secs) (*(secs) = 0)
#else /* ANSI */
	#define TICKCOUNT() 0
	#define GETDATETIME(secs) (*(secs) = 0)
#endif

#ifndef MAINARGS
	#define MAINARGS void
#endif

short IAbs(short value);
SInt32 LAbs(SInt32 value);
float RAbs(float value);
double DAbs(double value);

/*
 *		Inline assembly functions
 */
#ifdef __MOTOC__
	#define IABS(a)				abs(a)
	#define LABS(a)				abs(a)
	#define FABS(a)				fabs(a)
	#define FNABS(a)			-FABS(a)
#else
	#if defined(__MWERKS__) && defined(powerc)
		#define IABS(a)				__abs(a)
		#define LABS(a)				__labs(a)
		#define FABS(a)				__fabs(a)
		#define FNABS(a)			__fnabs(a)
	#else
		#define IABS(a)				IAbs(a)
		#if defined(__WIN32__) && defined __USE_RUNTIME_ABS__
			#define LABS(a)			labs(a)
		#else
			#define LABS(a)			LAbs(a)
		#endif
		#define FABS(a)				fabs(a)
		#define FNABS(a)			-FABS(a)
	#endif
#endif

#ifdef __cplusplus
}
#endif

/* Additional includes */

#ifdef __SGI__
//#include "MToolBox.h"
#endif

#endif
