/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#include "FACTStuff.h"
#ifdef __MACINTOSH__
#include <DateTimeUtils.h>
#else
#include <time.h>
#endif
#include <math.h>

/* Data initialization functions */

void SetARGB(ARGBPtr theARGB, unsigned char aVal, unsigned char rVal, unsigned char gVal, unsigned char bVal)
{
	theARGB->argb.a = aVal;
	theARGB->argb.r = rVal;
	theARGB->argb.g = gVal;
	theARGB->argb.b = bVal;
}  /* SetARGB */

ARGB MakeARGB(unsigned char aVal, unsigned char rVal, unsigned char gVal, unsigned char bVal)
{
	ARGB theARGB;

	theARGB.argb.a = aVal;
	theARGB.argb.r = rVal;
	theARGB.argb.g = gVal;
	theARGB.argb.b = bVal;
	return theARGB;
}  /* MakeARGB */

/* Pack floating point rgb colors into an ARGB structure */

void PackRGB(ARGB *theARGB, float r, float g, float b)
{
	int e;
	SInt32 ri, gi, bi;
	float v;
	
	if (r >= g)
		if (r >= b)
			v = r;
		else
			v = b;
	else
		if (g >= b)
			v = g;
		else
			v = b;
	if (v > 32767.0) {
		v = 32767.0f;
		if (r > 32767.0f)
			r = 32767.0f;
		if (g > 32767.0f)
			g = 32767.0f;
		if (b > 32767.0f)
			b = 32767.0f;
	}
	v = (float)(frexp(v, &e) * 512.0 / v);
	if (e <= -16) {
		theARGB->argb.a = 0;
		theARGB->argb.r = 0;
		theARGB->argb.g = 0;
		theARGB->argb.b = 0;
	} else {
		r *= v;
		g *= v;
		b *= v;
		ri = (SInt32)(r);
		gi = (SInt32)(g);
		bi = (SInt32)(b);
		
		theARGB->argb.a = (unsigned char)(((ri >> 8) << 7) + ((gi >> 8) << 6) + ((bi >> 8) << 5) + (e + 16));
		theARGB->argb.r = (unsigned char)(ri);
		theARGB->argb.g = (unsigned char)(gi);
		theARGB->argb.b = (unsigned char)(bi);
	}
}  /* PackRGB */

/* Unpack floating point rgb colors from an ARGB structure */

void UnpackRGB(ARGB *theARGB, float *r, float *g, float *b)
{
	static float lde[32] = {	/* ldexp(1.0 / 512.0, index - 16) */
		1.0f / (1 << 25),
		1.0f / (1 << 24),
		1.0f / (1 << 23),
		1.0f / (1 << 22),
		1.0f / (1 << 21),
		1.0f / (1 << 20),
		1.0f / (1 << 19),
		1.0f / (1 << 18),
		1.0f / (1 << 17),
		1.0f / (1 << 16),
		1.0f / (1 << 15),
		1.0f / (1 << 14),
		1.0f / (1 << 13),
		1.0f / (1 << 12),
		1.0f / (1 << 11),
		1.0f / (1 << 10),
		1.0f / (1 << 9),
		1.0f / (1 << 8),
		1.0f / (1 << 7),
		1.0f / (1 << 6),
		1.0f / (1 << 5),
		1.0f / (1 << 4),
		1.0f / (1 << 3),
		1.0f / (1 << 2),
		1.0f / (1 << 1),
		1.0f,
		2.0f,
		4.0f,
		8.0f,
		16.0f,
		32.0f,
		64.0f
	};

	if (theARGB->argb.a) {
		SInt32 a = theARGB->argb.a;
/*		float v = ldexp(1.0 / 512.0, (a & 31) - 16); */
		float v = lde[a & 31];
		
		*r = ((theARGB->argb.r + ((1 & (a >> 7)) << 8)) + 0.5f) * v;
		*g = ((theARGB->argb.g + ((1 & (a >> 6)) << 8)) + 0.5f) * v;
		*b = ((theARGB->argb.b + ((1 & (a >> 5)) << 8)) + 0.5f) * v;
	} else {
		*r = 0.0f;
		*g = 0.0f;
		*b = 0.0f;
	}
}  /* UnpackRGB */

void SetSRect(SRectPtr theSRect, short theLeft, short theTop, short theRight, short theBottom)
{
	theSRect->top = theTop;
	theSRect->left = theLeft;
	theSRect->bottom = theBottom;
	theSRect->right = theRight;
}  /* SetSRect */

void OffsetSRect(SRectPtr theSRect, SInt32 hOffset, SInt32 vOffset)
{
	theSRect->left += (short)hOffset;
	theSRect->top += (short)vOffset;
	theSRect->right += (short)hOffset;
	theSRect->bottom += (short)vOffset;
}  /* OffsetSRect */

void SetLRect(LRectPtr theLRect, SInt32 theLeft, SInt32 theTop, SInt32 theRight, SInt32 theBottom)
{
	theLRect->top = theTop;
	theLRect->left = theLeft;
	theLRect->bottom = theBottom;
	theLRect->right = theRight;
	theLRect->width = (UInt32)(theLRect->right - theLRect->left);
	theLRect->height = (UInt32)(theLRect->bottom - theLRect->top);
}  /* SetLRect */

void SRectToLRect(SRect *theRect, LRectPtr theLRect)
{
	theLRect->top = theRect->top;
	theLRect->left = theRect->left;
	theLRect->bottom = theRect->bottom;
	theLRect->right = theRect->right;
	theLRect->width = (UInt32)(theLRect->right - theLRect->left);
	theLRect->height = (UInt32)(theLRect->bottom - theLRect->top);
}  /* SRectToLRect */

void LRectToSRect(LRectPtr theLRect, SRect *theRect)
{
	theRect->top = (short)theLRect->top;
	theRect->left = (short)theLRect->left;
	theRect->bottom = (short)theLRect->bottom;
	theRect->right = (short)theLRect->right;
}  /* LRectToSRect */

void OffsetLRect(LRectPtr theLRect, SInt32 hOffset, SInt32 vOffset)
{
	theLRect->left += hOffset;
	theLRect->top += vOffset;
	theLRect->right += hOffset;
	theLRect->bottom += vOffset;
}  /* OffsetLRect */

void InsetLRect(LRectPtr theLRect, SInt32 x, SInt32 y)
{
	theLRect->left += x;
	theLRect->top += y;
	theLRect->right -= x;
	theLRect->bottom -= y;
	theLRect->width = (UInt32)(theLRect->right - theLRect->left);
	theLRect->height = (UInt32)(theLRect->bottom - theLRect->top);
	if ((theLRect->width == 0) || (theLRect->height == 0)) {
		theLRect->left = 0;
		theLRect->top = 0;
		theLRect->right = 0;
		theLRect->bottom = 0;
		theLRect->width = 0;
		theLRect->height = 0;
	}
}  /* InsetLRect */

void OutsetLRect(LRectPtr theLRect, SInt32 x, SInt32 y)
{
	theLRect->left -= x;
	theLRect->top -= y;
	theLRect->right += x;
	theLRect->bottom += y;
	theLRect->width = (UInt32)(theLRect->right - theLRect->left);
	theLRect->height = (UInt32)(theLRect->bottom - theLRect->top);
	if ((theLRect->width == 0) || (theLRect->height == 0)) {
		theLRect->left = 0;
		theLRect->top = 0;
		theLRect->right = 0;
		theLRect->bottom = 0;
		theLRect->width = 0;
		theLRect->height = 0;
	}
}  /* OutsetLRect */

int EmptyLRect(LRectPtr theLRect)
{
	return (theLRect->width == 0) || (theLRect->height == 0);
} /* EmptyLRect */

void UnionLRect(LRectPtr srcA, LRectPtr srcB, LRectPtr dst)
{
	int aEmpty = (srcA->right - srcA->left <= 0) || (srcA->bottom - srcA->top <= 0);
	int bEmpty = (srcB->right - srcB->left <= 0) || (srcB->bottom - srcB->top <= 0);
	
	if (aEmpty && bEmpty) {
		dst->left = 0;
		dst->top = 0;
		dst->right = 0;
		dst->bottom = 0;
		dst->width = 0;
		dst->height = 0;
	} else if (aEmpty)
		*dst = *srcB;
	else if (bEmpty)
		*dst = *srcA;
	else {
		if (srcA->left <= srcB->left)
			dst->left = srcA->left;
		else
			dst->left = srcB->left;
		if (srcA->top <= srcB->top)
			dst->top = srcA->top;
		else
			dst->top = srcB->top;
		if (srcA->right >= srcB->right)
			dst->right = srcA->right;
		else
			dst->right = srcB->right;
		if (srcA->bottom >= srcB->bottom)
			dst->bottom = srcA->bottom;
		else
			dst->bottom = srcB->bottom;
		dst->width = (UInt32)(dst->right - dst->left);
		dst->height = (UInt32)(dst->bottom - dst->top);
	}
} /* UnionLRect */

int SectLRect(LRectPtr srcA, LRectPtr srcB, LRectPtr dst)
{
	SInt32 aLeft = srcA->left;
	SInt32 aTop = srcA->top;
	SInt32 aRight = srcA->right;
	SInt32 aBottom = srcA->bottom;
	SInt32 bLeft = srcB->left;
	SInt32 bTop = srcB->top;
	SInt32 bRight = srcB->right;
	SInt32 bBottom = srcB->bottom;
	
	if ((aLeft >= bRight) || (aTop >= bBottom) || (aRight <= bLeft) || (aBottom <= bTop)) {
		dst->left = 0;
		dst->top = 0;
		dst->right = 0;
		dst->bottom = 0;
		dst->width = 0;
		dst->height = 0;
		return 0;
	} else {
		SInt32 left = dst->left = aLeft >= bLeft ? aLeft : bLeft;
		SInt32 top = dst->top = aTop >= bTop ? aTop : bTop;
		SInt32 right = dst->right = aRight <= bRight ? aRight : bRight;
		SInt32 bottom = dst->bottom = aBottom <= bBottom ? aBottom : bBottom;
		UInt32 width = dst->width = (UInt32)(right - left);
		UInt32 height = dst->height = (UInt32)(bottom - top);
		
		if ((width == 0) || (height == 0)) {
			dst->left = 0;
			dst->top = 0;
			dst->right = 0;
			dst->bottom = 0;
			return 0;
		}
		return 1;
	}
}  /* SectLRect */

void InitMatrix3(Matrix3Ptr theMatrix)
{
	static Matrix3 ident = {1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0};

	*theMatrix = ident;
}  /* InitMatrix3 */

void InitXMatrix3(XMatrix3Ptr theMatrix)
{
	static XMatrix3 ident = {1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0};

	*theMatrix = ident;
}  /* InitXMatrix3 */

void InitRMatrix3(RMatrix3Ptr theMatrix)
{
	static RMatrix3 ident = {1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f};

	*theMatrix = ident;
}  /* InitRMatrix3 */

void InitXMatrix4(XMatrix4Ptr theMatrix)
{
	static XMatrix4 m = {1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0};
	
	*theMatrix = m;
}  /* InitXMatrix4 */

void InitMatrix4(Matrix4Ptr theMatrix)
{
	static Matrix4 m = {1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0};
	
	*theMatrix = m;
}  /* InitMatrix4 */

void InitRMatrix4(RMatrix4Ptr theMatrix)
{
	static RMatrix4 m = {1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f};
	
	*theMatrix = m;
}  /* InitRMatrix4 */

void XMatrix3ToMatrix3(XMatrix3Ptr srcMatrix, Matrix3Ptr dstMatrix)
{
	SInt32 i, j;
	
	for (i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			dstMatrix->matrix[i][j] = srcMatrix->matrix[i][j];
}  /* XMatrix3ToMatrix3 */

void Matrix3ToXMatrix3(Matrix3Ptr srcMatrix, XMatrix3Ptr dstMatrix)
{
	SInt32 i, j;
	
	for (i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			dstMatrix->matrix[i][j] = srcMatrix->matrix[i][j];
}  /* Matrix3ToXMatrix3 */

void XMatrix4ToMatrix4(XMatrix4Ptr srcMatrix, Matrix4Ptr dstMatrix)
{
	SInt32 i, j;
	
	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			dstMatrix->matrix[i][j] = srcMatrix->matrix[i][j];
}  /* XMatrix4ToMatrix4 */

void Matrix4ToXMatrix4(Matrix4Ptr srcMatrix, XMatrix4Ptr dstMatrix)
{
	SInt32 i, j;
	
	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			dstMatrix->matrix[i][j] = srcMatrix->matrix[i][j];
}  /* Matrix4ToXMatrix4 */

void InitFINF(FINFPtr theFINF)
{
	theFINF->totalCoordinates = 0;
	theFINF->totalPolygons = 0;
	theFINF->totalGroups = 0;
	theFINF->extents.minCoord.x = 0.0f;
	theFINF->extents.minCoord.y = 0.0f;
	theFINF->extents.minCoord.z = 0.0f;
	theFINF->extents.maxCoord.x = 0.0f;
	theFINF->extents.maxCoord.y = 0.0f;
	theFINF->extents.maxCoord.z = 0.0f;
	theFINF->flags = 0;
	theFINF->anchor.x = 0.0f;
	theFINF->anchor.y = 0.0f;
	theFINF->anchor.z = 0.0f;
}  /* InitFINF */

void InitGINF(GINFPtr theGINF)
{
	theGINF->totalCoordinates = 0;
	theGINF->totalPolygons = 0;
	theGINF->groupTemp = 0;
	theGINF->extents.minCoord.x = 0.0f;
	theGINF->extents.minCoord.y = 0.0f;
	theGINF->extents.minCoord.z = 0.0f;
	theGINF->extents.maxCoord.x = 0.0f;
	theGINF->extents.maxCoord.y = 0.0f;
	theGINF->extents.maxCoord.z = 0.0f;
	theGINF->flags = 0;
	theGINF->flags |= (1 << groupInfoBreakFacetFlag);		/* set the break flag */
	theGINF->name[0] = 0;
#ifdef __MACINTOSH__
#if __LP64__
	theGINF->groupDate = (unsigned int)(CFAbsoluteTimeGetCurrent() + kCFAbsoluteTimeIntervalSince1904);
#else
	GetDateTime(&theGINF->groupDate);
#endif	
#else
	time((time_t*)&theGINF->groupDate);
#endif
	theGINF->groupID = 0;
	InitXMatrix4(&theGINF->coordinateAbsMatrix);
	InitXMatrix4(&theGINF->normalAbsMatrix);
	InitXMatrix4(&theGINF->coordinateRelMatrix);
	InitXMatrix4(&theGINF->normalRelMatrix);
	theGINF->childCount = 0;
	theGINF->childTotal = 0;
	theGINF->cycleCount = 0;
}  /* InitGINF */

void InitGLNK(GLNKPtr theGLNK)
{
	theGLNK->lType = linkFree;
	theGLNK->positionMinLimit.x = 0.0;
	theGLNK->positionMinLimit.y = 0.0;
	theGLNK->positionMinLimit.z = 0.0;
	theGLNK->positionMaxLimit.x = 0.0;
	theGLNK->positionMaxLimit.y = 0.0;
	theGLNK->positionMaxLimit.z = 0.0;
	theGLNK->rotationMinLimit.x = 0.0;
	theGLNK->rotationMinLimit.y = 0.0;
	theGLNK->rotationMinLimit.z = 0.0;
	theGLNK->rotationMaxLimit.x = 0.0;
	theGLNK->rotationMaxLimit.y = 0.0;
	theGLNK->rotationMaxLimit.z = 0.0;
	theGLNK->localOffset.x = 0.0;
	theGLNK->localOffset.y = 0.0;
	theGLNK->localOffset.z = 0.0;
	theGLNK->parentOffset.x = 0.0;
	theGLNK->parentOffset.y = 0.0;
	theGLNK->parentOffset.z = 0.0;
	theGLNK->parentRotation.x = 0.0;
	theGLNK->parentRotation.y = 0.0;
	theGLNK->parentRotation.z = 0.0;
	theGLNK->localRotation.x = 0.0;
	theGLNK->localRotation.y = 0.0;
	theGLNK->localRotation.z = 0.0;
	theGLNK->preScale.x = 1.0;
	theGLNK->preScale.y = 1.0;
	theGLNK->preScale.z = 1.0;

	/* The parentRotation matrix will be used by default if the preRotation matrix is all zeros */

	InitXMatrix4(&theGLNK->preRotation);
	theGLNK->lFlag = 0;
}  /* InitGLNK */

void InitGSTR(GSTRPtr theGSTR)
{
	theGSTR->offset.x = 0.0;
	theGSTR->offset.y = 0.0;
	theGSTR->offset.z = 0.0;
	theGSTR->center.x = 0.0;
	theGSTR->center.y = 0.0;
	theGSTR->center.z = 0.0;
	theGSTR->rotation.x = 0.0;
	theGSTR->rotation.y = 0.0;
	theGSTR->rotation.z = 0.0;
	theGSTR->scale.x = 1.0;
	theGSTR->scale.y = 1.0;
	theGSTR->scale.z = 1.0;
}  /* InitGSRT */

void InitGBLR(GBLRPtr theGBLR)
{
	theGBLR->blurFlags = 0;
	theGBLR->blurIntensity = 0.0f;
	theGBLR->blurLayer = 0;
	InitXMatrix4(&theGBLR->blurMatrix);
}  /* InitGBLR */

void InitGOUT(GOUTPtr theGOUT)
{
	theGOUT->outlineFlags = 0;
	theGOUT->outlinePolygonColor.argb.a = 255;
	theGOUT->outlinePolygonColor.argb.r = 0;
	theGOUT->outlinePolygonColor.argb.g = 0;
	theGOUT->outlinePolygonColor.argb.b = 0;
	theGOUT->outlinePolygonThickness = 1.0f;
	theGOUT->outlineEdgeColor.argb.a = 255;
	theGOUT->outlineEdgeColor.argb.r = 0;
	theGOUT->outlineEdgeColor.argb.g = 0;
	theGOUT->outlineEdgeColor.argb.b = 0;
	theGOUT->outlineEdgeThickness = 1.0f;
	theGOUT->outlineSilhouetteColor.argb.a = 255;
	theGOUT->outlineSilhouetteColor.argb.r = 0;
	theGOUT->outlineSilhouetteColor.argb.g = 0;
	theGOUT->outlineSilhouetteColor.argb.b = 0;
	theGOUT->outlineSilhouetteThickness = 1.0f;
}  /* InitGOUT */

void InitGATR(GATRPtr theGATR)
{
	theGATR->shadeFlags = defShadeFlags;
	theGATR->reference.argb.a = defReferenceA;
	theGATR->reference.argb.r = defReferenceR;
	theGATR->reference.argb.g = defReferenceG;
	theGATR->reference.argb.b = defReferenceB;
	theGATR->ambient.argb.a = defAmbientA;
	theGATR->ambient.argb.r = defAmbientR;
	theGATR->ambient.argb.g = defAmbientG;
	theGATR->ambient.argb.b = defAmbientB;
	theGATR->specular.argb.a = defSpecularA;
	theGATR->specular.argb.r = defSpecularR;
	theGATR->specular.argb.g = defSpecularG;
	theGATR->specular.argb.b = defSpecularB;
	theGATR->spread = (float)defSpread;
	theGATR->transparency.argb.a = defTransparencyA;
	theGATR->transparency.argb.r = defTransparencyR;
	theGATR->transparency.argb.g = defTransparencyG;
	theGATR->transparency.argb.b = defTransparencyB;
	theGATR->refraction = (float)defRefraction;
	theGATR->reflection.argb.a = defReflectionA;
	theGATR->reflection.argb.r = defReflectionR;
	theGATR->reflection.argb.g = defReflectionG;
	theGATR->reflection.argb.b = defReflectionB;
	theGATR->luminance.argb.a = defLuminanceA;
	theGATR->luminance.argb.r = defLuminanceR;
	theGATR->luminance.argb.g = defLuminanceG;
	theGATR->luminance.argb.b = defLuminanceB;
	theGATR->lineWeight = (float)defLineWeight;
	theGATR->edgeTransparencyDropoff = (float)defEdgeDropoff;
	theGATR->edgeTransparency = theGATR->transparency;
	theGATR->diffuse.argb.a = defDiffuseA;
	theGATR->diffuse.argb.r = defDiffuseR;
	theGATR->diffuse.argb.g = defDiffuseG;
	theGATR->diffuse.argb.b = defDiffuseB;
	theGATR->terminator = (float)1.0;
	theGATR->highlight = (float)1.0;
	theGATR->edgeDiffuseValue = (float)defEdgeValue;
	theGATR->edgeSpecularValue = (float)defEdgeValue;
	theGATR->edgeReflectionValue = (float)defEdgeValue;
	theGATR->edgeAmbientValue = (float)defEdgeValue;
	theGATR->edgeLuminanceValue = (float)defEdgeValue;
	theGATR->shadeFlags2 = defShadeFlags2;
	theGATR->transmission.argb.a = defTransmissionA;
	theGATR->transmission.argb.r = defTransmissionR;
	theGATR->transmission.argb.g = defTransmissionG;
	theGATR->transmission.argb.b = defTransmissionB;
	theGATR->edgeTransmissionValue = (float)defEdgeValue;
	theGATR->glow.argb.a = defGlowA;
	theGATR->glow.argb.r = defGlowR;
	theGATR->glow.argb.g = defGlowG;
	theGATR->glow.argb.b = defGlowB;
	theGATR->edgeDiffuseDropoff = (float)defEdgeDropoff;
	theGATR->edgeSpecularDropoff = (float)defEdgeDropoff;
	theGATR->edgeReflectionDropoff = (float)defEdgeDropoff;
	theGATR->edgeAmbientDropoff = (float)defEdgeDropoff;
	theGATR->edgeLuminanceDropoff = (float)defEdgeDropoff;
	theGATR->edgeTransmissionDropoff = (float)defEdgeDropoff;
	theGATR->edgeTransparencyValue = (float)defEdgeValue;
	theGATR->reflectionBlur	= (float)defReflectionBlur;
	theGATR->refractionBlur = (float)defRefractionBlur;
	theGATR->reflectionBlurType = defReflectionBlurType;
	theGATR->reflectionBlurSamples = defReflectionBlurSamples;
	theGATR->reflectionBlurAngle   = (float)defReflectionBlurAngle;
	theGATR->refractionBlurType    = defRefractionBlurType;
	theGATR->refractionBlurSamples = defRefractionBlurSamples;
	theGATR->refractionBlurAngle   = (float)defRefractionBlurAngle;
	theGATR->refrCausticsAmount	   = (float)defCausticsAmount;
	theGATR->refrCausticsFactor	   = (float)defCausticsFactor;
	theGATR->refrCausticsDropoff   = (float)defCausticsDropoff;
	theGATR->refrCausticsReceive   = (float)defCausticsReceive;
	theGATR->reflCausticsAmount	   = (float)defCausticsAmount;
	theGATR->reflCausticsColor.argb.a = defSpecularA;
	theGATR->reflCausticsColor.argb.r = defSpecularR;
	theGATR->reflCausticsColor.argb.g = defSpecularG;
	theGATR->reflCausticsColor.argb.b = defSpecularB;
	theGATR->reflCausticsFactor	   = (float)defCausticsFactor;
	theGATR->reflCausticsDropoff   = (float)defCausticsDropoff;
	theGATR->reflCausticsReceive   = (float)defCausticsReceive;
	theGATR->translucentAmount	   = (float)defTranslucentAmount;	
	theGATR->translucentThickness  = (float)defTranslucentThickness;
	theGATR->translucentColor.argb.a = defTranslucentA;
	theGATR->translucentColor.argb.r = defTranslucentR;
	theGATR->translucentColor.argb.g = defTranslucentG;
	theGATR->translucentColor.argb.b = defTranslucentB;
	theGATR->translucentScatter		 = (float)defTranslucentScatter;
	theGATR->translucentFocus		 = (float)defTranslucentFocus;
	theGATR->transparentThickness	 = (float)defTransparentThickness;
}  /* InitGATR */

void InitTMAP(TMAPPtr theTMAP)
{
	theTMAP->textureFlags = 0;
	theTMAP->textureType = 0;
	theTMAP->textureTypeName[0] = 0;
	theTMAP->mappingType = mapNone;
	theTMAP->mappingTypeName[0] = 0;
	InitXMatrix4(&theTMAP->tMatrix);
	theTMAP->textureName[0] = 0;
	theTMAP->textureRect.top = 0;
	theTMAP->textureRect.left = 0;
	theTMAP->textureRect.right = 0;
	theTMAP->textureRect.bottom = 0;
	theTMAP->offset.x = 0.0;
	theTMAP->offset.y = 0.0;
	theTMAP->offset.z = 0.0;
	theTMAP->scale.x = 1.0;
	theTMAP->scale.y = 1.0;
	theTMAP->scale.z = 1.0;
	theTMAP->textureReference = 0;
	theTMAP->textureVolName[0] = 0;
	theTMAP->bumpScale = 1.0;
	InitXMatrix4(&theTMAP->bumpMatrix);
	theTMAP->textureFrame = 1;
	theTMAP->textureTime = 0.0f;
	theTMAP->textureDensity = 1.0f;
	theTMAP->textureCutoff = 0;
	theTMAP->backOffset = theTMAP->offset;
	theTMAP->backScale = theTMAP->scale;
	theTMAP->leftOffset = theTMAP->offset;
	theTMAP->leftScale = theTMAP->scale;
	theTMAP->rightOffset = theTMAP->offset;
	theTMAP->rightScale = theTMAP->scale;
	theTMAP->topOffset = theTMAP->offset;
	theTMAP->topScale = theTMAP->scale;
	theTMAP->bottomOffset = theTMAP->offset;
	theTMAP->bottomScale = theTMAP->scale;
}  /* InitTMAP */

void InitTATR(TATRPtr theTATR)
{
	InitGATR(theTATR);
	theTATR->shadeFlags = 0x7FFF;		/* Change all but the reference color */
}  /* InitTATR */

void InitTMEI(TMEIPtr theTMEI, UInt32 theMapType)
{
	theTMEI->texFlags = 0;
	theTMEI->fullRect.top = 0;
	theTMEI->fullRect.left = 0;
	theTMEI->fullRect.bottom = 0;
	theTMEI->fullRect.right = 0;
	theTMEI->position.x = 0.0;
	theTMEI->position.y = 0.0;
	theTMEI->position.z = 0.0;
	theTMEI->offset.x = 0.0;
	theTMEI->offset.y = 0.0;
	theTMEI->offset.z = 0.0;
	theTMEI->rotation.x = 0.0;
	theTMEI->rotation.y = 0.0;
	theTMEI->rotation.z = 0.0;
	theTMEI->textureVolumeIndex = 0;
	theTMEI->textureDirID = 0;
	switch (theMapType) {
	case mapNone:
	case mapFlat:
	case mapCubic:
	case mapPlanar: 
		theTMEI->u.Flat.imageScaleX = 1.0;
		theTMEI->u.Flat.imageScaleY = 1.0;
		theTMEI->u.Flat.imageScaleZ = 1.0;
		break;
	case mapCylindrical: 
		theTMEI->u.Cylindrical.cylinderRadius = 1.0;
		theTMEI->u.Cylindrical.cylinderWrapAngle = 360.0;
		theTMEI->u.Cylindrical.cylinderHeight = 1.0;
		break;
	case mapSpherical: 
		theTMEI->u.Spherical.sphereRadius = 1.0;
		theTMEI->u.Spherical.sphereWrapAngle = 360.0;
		theTMEI->u.Spherical.sphereBandAngle = 180.0;
		theTMEI->u.Spherical.sphereBandAngleStart = 0.0;
		break;
	default:
		break;
	}
}  /* InitTMEI */

void InitBMAP(BMAPPtr theBMAP)
{
	InitTMAP(theBMAP);
}  /* InitBMAP */

void InitBATR(BATRPtr theBATR)
{
	InitTATR(theBATR);
}  /* InitBMEI */

void InitBMEI(BMEIPtr theBMEI, UInt32 theMapType)
{
	InitTMEI(theBMEI, theMapType);
}  /* InitBMEI */

void InitRMAP(RMAPPtr theRMAP)
{
	InitTMAP(theRMAP);
}  /* InitRMAP */

void InitRMEI(RMEIPtr theRMEI, UInt32 theMapType)
{
	InitTMEI(theRMEI, theMapType);
}  /* InitRMEI */
