/*  Copyright 1993-2011 EIAS3D, This Source code may be included in plug-ins, 
shaders, and utilities intended for use with the Electric Image Animation System.
This copyright notice must be included in its original form in all such products. 
This  source code may not be offered for sale or distributed by any entity 
without the express permission of EIAS3D.*/

#include "RoundUtils.h"

#define MAXINT 0x7FFFFFFF
#define MININT 0x80000000

// double RU_Magic = (double)((long long)((long long)1 << 51) + (long long)((long long)1 << 52));

#ifdef __ROUNDTOFLOOR__

SInt32 CalcFactor(double n)
{
	return (n - (SInt32)n) * 1024.0;
}  /* CalcFactor */

SInt32 CalcFactor2(double* n)
{
	return (*n - (SInt32)*n) * 1024.0;
}  /* CalcFactor2 */

SInt32 CalcLFactor(double n)
{
	return (n - (SInt32)n) * 16384.0;
}  /* CalcLFactor */

SInt32 CalcLFactor2(double* n)
{
	return (*n - (SInt32)*n) * 16384.0;
}  /* CalcLFactor2 */

double CalcRFactor(double n)
{
	return n - (SInt32)n;
}  /* CalcRFactor */

double CalcRFactor2(double* n)
{
	return *n - (SInt32)*n;
}  /* CalcRFactor2 */

void CalcRFactor3(double* n)
{
	*n -= (SInt32)*n;
}  /* CalcRFactor3 */

SInt32 SplitFactor(double* n)
{
	SInt32 temp = *n;
	*n -= temp;
	return temp;
}  /* SplitFactor */

SInt32 ARound(double n)
{
	if (n < 0.0)
		return n - 0.5;
	return n + 0.5;
}  /* ARound */

SInt32 ARound2(double* n)
{
	if (*n < 0.0)
		return *n - 0.5;
	return *n + 0.5;
}  /* ARound2 */

SInt32 ARoundI(double n)
{
	SInt32 temp = n;
	return INTEGERLIMIT(temp);
}  /* ARoundI */

SInt32 ARoundI2(double* n)
{
	SInt32 temp = *n;
	return INTEGERLIMIT(temp);
}  /* ARoundI2 */

SInt32 LRound(double n)
{
	return n * 65536.0;
}  /* LRound */

SInt32 LRound2(double* n)
{
	return *n * 65536.0;
}  /* LRound2 */

SInt32 IRound(double n)
{
	SInt32 temp = n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRound */

SInt32 IRound2(double* n)
{
	SInt32 temp = *n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRound2 */

SInt32 IRoundR(float n)
{
	SInt32 temp = n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRoundR */

SInt32 IRoundR2(float* n)
{
	SInt32 temp = *n * 16384.0;
	return INTEGERLIMIT(temp);
}  /* IRoundR2 */

SInt32 ITrunc(double n)
{
	SInt32 temp = n;
	return INTEGERLIMIT(temp);
}  /* ITrunc */

SInt32 ITrunc2(double* n)
{
	SInt32 temp = *n;
	return INTEGERLIMIT(temp);
}  /* ITrunc2 */

SInt32 FTrunc(double n)
{
	return n * 1024.0;
}  /* FTrunc */

SInt32 FTrunc2(double* n)
{
	return *n * 1024.0;
}  /* FTrunc2 */

SInt32 TruncLo(double n)
{
	return n + 1.0;
}  /* TruncLo */

SInt32 TruncLo2(double* n)
{
	return *n + 1.0;
}  /* TruncLo2 */

SInt32 TruncHi(double n)
{
	return n;
}  /* TruncHi */

SInt32 TruncHi2(double* n)
{
	return *n;
}  /* TruncHi2 */

SInt32 Floor(double n)
{
	return n;
}  /* Floor */

SInt32 Floor2(double* n)
{
	return *n;
}  /* Floor2 */

SInt32 Ceil(double n)
{
	return -(SInt32)(-n);
}  /* Ceil */

SInt32 Ceil2(double* n)
{
	return -(SInt32)(-*n);
}  /* Ceil2 */

#else

SInt32 CalcFactor(double n)
{
	SInt32 temp = (SInt32)n;
	if (temp > n)
		temp--;
	return (SInt32)((n - temp) * 1024.0);
}  /* CalcFactor */

SInt32 CalcFactor2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if (temp > *n)
		temp--;
	return (SInt32)((*n - temp) * 1024.0);
}  /* CalcFactor2 */

SInt32 CalcLFactor(double n)
{
	SInt32 temp = (SInt32)n;
	if (temp > n)
		temp--;
	return (SInt32)((n - temp) * 16384.0);
}  /* CalcLFactor */

SInt32 CalcLFactor2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if (temp > *n)
		temp--;
	return (SInt32)((*n - temp) * 16384.0);
}  /* CalcLFactor2 */

double CalcRFactor(double n)
{
	SInt32 temp = (SInt32)n;
	if (temp > n)
		temp--;
	return n - temp;
}  /* CalcRFactor */

double CalcRFactor2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if (temp > *n)
		temp--;
	return *n - temp;
}  /* CalcRFactor2 */

void CalcRFactor3(double* n)
{
	SInt32 temp = (SInt32)*n;
	if (temp > *n)
		temp--;
	*n -= temp;
}  /* CalcRFactor3 */

SInt32 SplitFactor(double* n)
{
	SInt32 temp = (SInt32)*n;

	*n -= temp;
	if ((temp != MININT) && (*n < 0.0)) {
		*n += 1.0;
		return temp - 1;
	}
	return temp;
}  /* SplitFactor */

SInt32 ARound(double n)
{
	if (n < 0.0)
		return (SInt32)(n - 0.5);
	else
		return (SInt32)(n + 0.5);
}  /* ARound */

SInt32 ARound2(double* n)
{
	if (*n < 0.0)
		return (SInt32)(*n - 0.5);
	else
		return (SInt32)(*n + 0.5);
}  /* ARound2 */

SInt32 ARoundI(double n)
{
	if (n < 0.0) {
		SInt32 temp = (SInt32)(n - 0.5);
		if (temp <= -32767)
			return -32767;
		else
			return temp;
	} else {
		SInt32 temp = (SInt32)(n + 0.5);
		if (temp >= 32767)
			return 32767;
		else
			return temp;
	}
}  /* ARoundI */

SInt32 ARoundI2(double* n)
{
	if (*n < 0.0) {
		SInt32 temp = (SInt32)(*n - 0.5);
		if (temp <= -32767)
			return -32767;
		else
			return temp;
	} else {
		SInt32 temp = (SInt32)(*n + 0.5);
		if (temp >= 32767)
			return 32767;
		else
			return temp;
	}
}  /* ARoundI2 */

SInt32 LRound(double n)
{
	return (SInt32)(n * 65536.0);
}  /* LRound */

SInt32 LRound2(double* n)
{
	return (SInt32)(*n * 65536.0);
}  /* LRound2 */

SInt32 IRound(double n)
{
	SInt32 temp = (SInt32)(n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRound */

SInt32 IRound2(double* n)
{
	SInt32 temp = (SInt32)(*n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRound2 */

SInt32 IRoundR(float n)
{
	SInt32 temp = (SInt32)(n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRoundR */

SInt32 IRoundR2(float* n)
{
	SInt32 temp = (SInt32)(*n * 16384.0);
	return INTEGERLIMIT(temp);
}  /* IRoundR2 */

SInt32 ITrunc(double n)
{
	SInt32 temp = (SInt32)n;
	if (temp <= -32767)
		return -32767;
	else if (temp >= 32767)
		return 32767;
	else if (temp > n)
		return temp - 1;
	else
		return temp;
}  /* ITrunc */

SInt32 ITrunc2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if (temp <= -32767)
		return -32767;
	else if (temp >= 32767)
		return 32767;
	else if (temp > *n)
		return temp - 1;
	else
		return temp;
}  /* ITrunc2 */

SInt32 FTrunc(double n)
{
	SInt32 temp;

	n *= 1024.0;
	temp = (SInt32)n;
	if ((temp != MININT) && (temp > n))
		return temp - 1;
	else
		return temp;
}  /* FTrunc */

SInt32 FTrunc2(double* n)
{
	double t = *n * 1024.0;
	SInt32 temp = (SInt32)t;
	
	if ((temp != MININT) && (temp > t))
		return temp - 1;
	else
		return temp;
}  /* FTrunc2 */

SInt32 TruncLo(double n)
{
	SInt32 temp = (SInt32)n;
	
	if ((temp != MAXINT) && (temp <= n))
		return temp + 1;
	else
		return temp;
}  /* TruncLo */

SInt32 TruncLo2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if ((temp != MAXINT) && (temp <= *n))
		return temp + 1;
	else
		return temp;
}  /* TruncLo2 */

SInt32 TruncHi(double n)
{
	SInt32 temp = (SInt32)n;
	if ((temp != MININT) && (temp > n))
		return temp - 1;
	else
		return temp;
}  /* TruncHi */

SInt32 TruncHi2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if ((temp != MININT) && (temp > *n))
		return temp - 1;
	else
		return temp;
}  /* TruncHi2 */

SInt32 Floor(double n)
{
	SInt32 temp = (SInt32)n;
	if ((temp != MININT) && (temp > n))
		return temp - 1;
	else
		return temp;
}  /* Floor */

SInt32 Floor2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if ((temp != MININT) && (temp > *n))
		return temp - 1;
	else
		return temp;
}  /* Floor2 */

SInt32 Ceil(double n)
{
	SInt32 temp = (SInt32)n;
	if ((temp != MAXINT) && (temp < n))
		return temp + 1;
	else
		return temp;
}  /* Ceil */

SInt32 Ceil2(double* n)
{
	SInt32 temp = (SInt32)*n;
	if ((temp != MAXINT) && (temp < *n))
		return temp + 1;
	else
		return temp;
}  /* Ceil2 */

#endif
